declare module 'lucide'

export interface SVGProps extends Partial<SVGElement> {
  "xmlns": "http://www.w3.org/2000/svg",
  "width": 24,
  "height": 24,
  "viewBox": "0 0 24 24",
  "fill": "none",
  "stroke": "currentColor",
  "stroke-width": 2,
  "stroke-linecap": "round",
  "stroke-linejoin": "round"
}

export declare type IconNodeChild = readonly [string, object];
export declare type IconNode = readonly [tag: string, attrs: SVGProps, children?: IconNodeChild[]];
export declare type CustomAttrs = { [attr:string]: any }
export type Icons = { [key: string]: IconNode }

export interface CreateIconsOptions {
  /**
   * List of icons you want to replace
   *
   * For example: `{ Menu, Circle}`.
   *
   * For replace all icons in lucide library, import `icons` and use it.
   */
  icons: Icons;

  /**
   * Search HTML emelemt by `nameAttr` property.
   *
   * For example if define `<i lucide-icon="circle"></i>`, fill by `lucide-icon`.
   *
   * @default 'icon-name'
   */
  nameAttr?: string;

  /**
   * Change defult attribute for show like color, fill, width , ...
   *
   * @default undefined
   */
  attrs?: CustomAttrs;
}

export function createElement(icon: IconNode): SVGSVGElement;
export function createIcons(options: CreateIconsOptions): void;

export declare const icons: Icons;

// Generated icons
export declare const Activity: IconNode;
export declare const Airplay: IconNode;
export declare const AlarmCheck: IconNode;
export declare const AlarmClockOff: IconNode;
export declare const AlarmClock: IconNode;
export declare const AlarmMinus: IconNode;
export declare const AlarmPlus: IconNode;
export declare const Album: IconNode;
export declare const AlertCircle: IconNode;
export declare const AlertOctagon: IconNode;
export declare const AlertTriangle: IconNode;
export declare const AlignCenterHorizontal: IconNode;
export declare const AlignCenterVertical: IconNode;
export declare const AlignCenter: IconNode;
export declare const AlignEndHorizontal: IconNode;
export declare const AlignEndVertical: IconNode;
export declare const AlignHorizontalDistributeCenter: IconNode;
export declare const AlignHorizontalDistributeEnd: IconNode;
export declare const AlignHorizontalDistributeStart: IconNode;
export declare const AlignHorizontalJustifyCenter: IconNode;
export declare const AlignHorizontalJustifyEnd: IconNode;
export declare const AlignHorizontalJustifyStart: IconNode;
export declare const AlignHorizontalSpaceAround: IconNode;
export declare const AlignHorizontalSpaceBetween: IconNode;
export declare const AlignJustify: IconNode;
export declare const AlignLeft: IconNode;
export declare const AlignRight: IconNode;
export declare const AlignStartHorizontal: IconNode;
export declare const AlignStartVertical: IconNode;
export declare const AlignVerticalDistributeCenter: IconNode;
export declare const AlignVerticalDistributeEnd: IconNode;
export declare const AlignVerticalDistributeStart: IconNode;
export declare const AlignVerticalJustifyCenter: IconNode;
export declare const AlignVerticalJustifyEnd: IconNode;
export declare const AlignVerticalJustifyStart: IconNode;
export declare const AlignVerticalSpaceAround: IconNode;
export declare const AlignVerticalSpaceBetween: IconNode;
export declare const Anchor: IconNode;
export declare const Aperture: IconNode;
export declare const Archive: IconNode;
export declare const ArrowBigDown: IconNode;
export declare const ArrowBigLeft: IconNode;
export declare const ArrowBigRight: IconNode;
export declare const ArrowBigUp: IconNode;
export declare const ArrowDownCircle: IconNode;
export declare const ArrowDownLeft: IconNode;
export declare const ArrowDownRight: IconNode;
export declare const ArrowDown: IconNode;
export declare const ArrowLeftCircle: IconNode;
export declare const ArrowLeftRight: IconNode;
export declare const ArrowLeft: IconNode;
export declare const ArrowRightCircle: IconNode;
export declare const ArrowRight: IconNode;
export declare const ArrowUpCircle: IconNode;
export declare const ArrowUpLeft: IconNode;
export declare const ArrowUpRight: IconNode;
export declare const ArrowUp: IconNode;
export declare const Asterisk: IconNode;
export declare const AtSign: IconNode;
export declare const Award: IconNode;
export declare const Axe: IconNode;
export declare const Banknote: IconNode;
export declare const BarChart2: IconNode;
export declare const BarChart: IconNode;
export declare const Baseline: IconNode;
export declare const BatteryCharging: IconNode;
export declare const BatteryFull: IconNode;
export declare const BatteryLow: IconNode;
export declare const BatteryMedium: IconNode;
export declare const Battery: IconNode;
export declare const Beaker: IconNode;
export declare const BellMinus: IconNode;
export declare const BellOff: IconNode;
export declare const BellPlus: IconNode;
export declare const BellRing: IconNode;
export declare const Bell: IconNode;
export declare const Bike: IconNode;
export declare const Binary: IconNode;
export declare const Bitcoin: IconNode;
export declare const BluetoothConnected: IconNode;
export declare const BluetoothOff: IconNode;
export declare const BluetoothSearching: IconNode;
export declare const Bluetooth: IconNode;
export declare const Bold: IconNode;
export declare const BookOpen: IconNode;
export declare const Book: IconNode;
export declare const BookmarkMinus: IconNode;
export declare const BookmarkPlus: IconNode;
export declare const Bookmark: IconNode;
export declare const Bot: IconNode;
export declare const BoxSelect: IconNode;
export declare const Box: IconNode;
export declare const Briefcase: IconNode;
export declare const Brush: IconNode;
export declare const Bug: IconNode;
export declare const Building: IconNode;
export declare const Bus: IconNode;
export declare const Calculator: IconNode;
export declare const Calendar: IconNode;
export declare const CameraOff: IconNode;
export declare const Camera: IconNode;
export declare const Car: IconNode;
export declare const Carrot: IconNode;
export declare const Cast: IconNode;
export declare const CheckCircle2: IconNode;
export declare const CheckCircle: IconNode;
export declare const CheckSquare: IconNode;
export declare const Check: IconNode;
export declare const ChevronDown: IconNode;
export declare const ChevronFirst: IconNode;
export declare const ChevronLast: IconNode;
export declare const ChevronLeft: IconNode;
export declare const ChevronRight: IconNode;
export declare const ChevronUp: IconNode;
export declare const ChevronsDownUp: IconNode;
export declare const ChevronsDown: IconNode;
export declare const ChevronsLeft: IconNode;
export declare const ChevronsRight: IconNode;
export declare const ChevronsUpDown: IconNode;
export declare const ChevronsUp: IconNode;
export declare const Chrome: IconNode;
export declare const CircleSlashed: IconNode;
export declare const Circle: IconNode;
export declare const ClipboardCheck: IconNode;
export declare const ClipboardCopy: IconNode;
export declare const ClipboardList: IconNode;
export declare const ClipboardX: IconNode;
export declare const Clipboard: IconNode;
export declare const Clock1: IconNode;
export declare const Clock10: IconNode;
export declare const Clock11: IconNode;
export declare const Clock12: IconNode;
export declare const Clock2: IconNode;
export declare const Clock3: IconNode;
export declare const Clock4: IconNode;
export declare const Clock5: IconNode;
export declare const Clock6: IconNode;
export declare const Clock7: IconNode;
export declare const Clock8: IconNode;
export declare const Clock9: IconNode;
export declare const Clock: IconNode;
export declare const CloudDrizzle: IconNode;
export declare const CloudFog: IconNode;
export declare const CloudHail: IconNode;
export declare const CloudLightning: IconNode;
export declare const CloudMoon: IconNode;
export declare const CloudOff: IconNode;
export declare const CloudRainWind: IconNode;
export declare const CloudRain: IconNode;
export declare const CloudSnow: IconNode;
export declare const CloudSun: IconNode;
export declare const Cloud: IconNode;
export declare const Cloudy: IconNode;
export declare const Clover: IconNode;
export declare const Code2: IconNode;
export declare const Code: IconNode;
export declare const Codepen: IconNode;
export declare const Codesandbox: IconNode;
export declare const Coffee: IconNode;
export declare const Coins: IconNode;
export declare const Columns: IconNode;
export declare const Command: IconNode;
export declare const Compass: IconNode;
export declare const Contact: IconNode;
export declare const Contrast: IconNode;
export declare const Cookie: IconNode;
export declare const Copy: IconNode;
export declare const Copyleft: IconNode;
export declare const Copyright: IconNode;
export declare const CornerDownLeft: IconNode;
export declare const CornerDownRight: IconNode;
export declare const CornerLeftDown: IconNode;
export declare const CornerLeftUp: IconNode;
export declare const CornerRightDown: IconNode;
export declare const CornerRightUp: IconNode;
export declare const CornerUpLeft: IconNode;
export declare const CornerUpRight: IconNode;
export declare const Cpu: IconNode;
export declare const CreditCard: IconNode;
export declare const Crop: IconNode;
export declare const Cross: IconNode;
export declare const Crosshair: IconNode;
export declare const Crown: IconNode;
export declare const Currency: IconNode;
export declare const Database: IconNode;
export declare const Delete: IconNode;
export declare const Dice1: IconNode;
export declare const Dice2: IconNode;
export declare const Dice3: IconNode;
export declare const Dice4: IconNode;
export declare const Dice5: IconNode;
export declare const Dice6: IconNode;
export declare const Disc: IconNode;
export declare const DivideCircle: IconNode;
export declare const DivideSquare: IconNode;
export declare const Divide: IconNode;
export declare const DollarSign: IconNode;
export declare const DownloadCloud: IconNode;
export declare const Download: IconNode;
export declare const Dribbble: IconNode;
export declare const Droplet: IconNode;
export declare const Droplets: IconNode;
export declare const Drumstick: IconNode;
export declare const Edit2: IconNode;
export declare const Edit3: IconNode;
export declare const Edit: IconNode;
export declare const Egg: IconNode;
export declare const EqualNot: IconNode;
export declare const Equal: IconNode;
export declare const Euro: IconNode;
export declare const Expand: IconNode;
export declare const ExternalLink: IconNode;
export declare const EyeOff: IconNode;
export declare const Eye: IconNode;
export declare const Facebook: IconNode;
export declare const FastForward: IconNode;
export declare const Feather: IconNode;
export declare const Figma: IconNode;
export declare const FileCheck2: IconNode;
export declare const FileCheck: IconNode;
export declare const FileCode: IconNode;
export declare const FileDigit: IconNode;
export declare const FileInput: IconNode;
export declare const FileMinus2: IconNode;
export declare const FileMinus: IconNode;
export declare const FileOutput: IconNode;
export declare const FilePlus2: IconNode;
export declare const FilePlus: IconNode;
export declare const FileSearch: IconNode;
export declare const FileText: IconNode;
export declare const FileX2: IconNode;
export declare const FileX: IconNode;
export declare const File: IconNode;
export declare const Files: IconNode;
export declare const Film: IconNode;
export declare const Filter: IconNode;
export declare const FlagTriangleLeft: IconNode;
export declare const FlagTriangleRight: IconNode;
export declare const Flag: IconNode;
export declare const Flame: IconNode;
export declare const FlashlightOff: IconNode;
export declare const Flashlight: IconNode;
export declare const FlaskConical: IconNode;
export declare const FlaskRound: IconNode;
export declare const FolderMinus: IconNode;
export declare const FolderOpen: IconNode;
export declare const FolderPlus: IconNode;
export declare const Folder: IconNode;
export declare const FormInput: IconNode;
export declare const Forward: IconNode;
export declare const Framer: IconNode;
export declare const Frown: IconNode;
export declare const FunctionSquare: IconNode;
export declare const Gamepad2: IconNode;
export declare const Gamepad: IconNode;
export declare const Gauge: IconNode;
export declare const Gavel: IconNode;
export declare const Gem: IconNode;
export declare const Ghost: IconNode;
export declare const Gift: IconNode;
export declare const GitBranchPlus: IconNode;
export declare const GitBranch: IconNode;
export declare const GitCommit: IconNode;
export declare const GitFork: IconNode;
export declare const GitMerge: IconNode;
export declare const GitPullRequest: IconNode;
export declare const Github: IconNode;
export declare const Gitlab: IconNode;
export declare const Glasses: IconNode;
export declare const Globe2: IconNode;
export declare const Globe: IconNode;
export declare const Grab: IconNode;
export declare const GraduationCap: IconNode;
export declare const Grid: IconNode;
export declare const GripHorizontal: IconNode;
export declare const GripVertical: IconNode;
export declare const Hammer: IconNode;
export declare const HandMetal: IconNode;
export declare const Hand: IconNode;
export declare const HardDrive: IconNode;
export declare const HardHat: IconNode;
export declare const Hash: IconNode;
export declare const Haze: IconNode;
export declare const Headphones: IconNode;
export declare const Heart: IconNode;
export declare const HelpCircle: IconNode;
export declare const Hexagon: IconNode;
export declare const Highlighter: IconNode;
export declare const History: IconNode;
export declare const Home: IconNode;
export declare const ImageMinus: IconNode;
export declare const ImageOff: IconNode;
export declare const ImagePlus: IconNode;
export declare const Image: IconNode;
export declare const Import: IconNode;
export declare const Inbox: IconNode;
export declare const Indent: IconNode;
export declare const IndianRupee: IconNode;
export declare const Infinity: IconNode;
export declare const Info: IconNode;
export declare const Inspect: IconNode;
export declare const Instagram: IconNode;
export declare const Italic: IconNode;
export declare const JapaneseYen: IconNode;
export declare const Key: IconNode;
export declare const Keyboard: IconNode;
export declare const Landmark: IconNode;
export declare const Languages: IconNode;
export declare const Laptop2: IconNode;
export declare const Laptop: IconNode;
export declare const LassoSelect: IconNode;
export declare const Lasso: IconNode;
export declare const Layers: IconNode;
export declare const LayoutDashboard: IconNode;
export declare const LayoutGrid: IconNode;
export declare const LayoutList: IconNode;
export declare const LayoutTemplate: IconNode;
export declare const Layout: IconNode;
export declare const Library: IconNode;
export declare const LifeBuoy: IconNode;
export declare const LightbulbOff: IconNode;
export declare const Lightbulb: IconNode;
export declare const Link2Off: IconNode;
export declare const Link2: IconNode;
export declare const Link: IconNode;
export declare const Linkedin: IconNode;
export declare const ListChecks: IconNode;
export declare const ListMinus: IconNode;
export declare const ListOrdered: IconNode;
export declare const ListPlus: IconNode;
export declare const ListX: IconNode;
export declare const List: IconNode;
export declare const Loader2: IconNode;
export declare const Loader: IconNode;
export declare const LocateFixed: IconNode;
export declare const LocateOff: IconNode;
export declare const Locate: IconNode;
export declare const Lock: IconNode;
export declare const LogIn: IconNode;
export declare const LogOut: IconNode;
export declare const Mail: IconNode;
export declare const MapPin: IconNode;
export declare const Map: IconNode;
export declare const Maximize2: IconNode;
export declare const Maximize: IconNode;
export declare const Megaphone: IconNode;
export declare const Meh: IconNode;
export declare const Menu: IconNode;
export declare const MessageCircle: IconNode;
export declare const MessageSquare: IconNode;
export declare const MicOff: IconNode;
export declare const Mic: IconNode;
export declare const Minimize2: IconNode;
export declare const Minimize: IconNode;
export declare const MinusCircle: IconNode;
export declare const MinusSquare: IconNode;
export declare const Minus: IconNode;
export declare const MonitorOff: IconNode;
export declare const MonitorSpeaker: IconNode;
export declare const Monitor: IconNode;
export declare const Moon: IconNode;
export declare const MoreHorizontal: IconNode;
export declare const MoreVertical: IconNode;
export declare const MountainSnow: IconNode;
export declare const Mountain: IconNode;
export declare const MousePointer2: IconNode;
export declare const MousePointerClick: IconNode;
export declare const MousePointer: IconNode;
export declare const MoveDiagonal2: IconNode;
export declare const MoveDiagonal: IconNode;
export declare const MoveHorizontal: IconNode;
export declare const MoveVertical: IconNode;
export declare const Move: IconNode;
export declare const Music: IconNode;
export declare const Navigation2: IconNode;
export declare const Navigation: IconNode;
export declare const Network: IconNode;
export declare const Octagon: IconNode;
export declare const Option: IconNode;
export declare const Outdent: IconNode;
export declare const PackageCheck: IconNode;
export declare const PackageMinus: IconNode;
export declare const PackagePlus: IconNode;
export declare const PackageSearch: IconNode;
export declare const PackageX: IconNode;
export declare const Package: IconNode;
export declare const Palette: IconNode;
export declare const Palmtree: IconNode;
export declare const Paperclip: IconNode;
export declare const PauseCircle: IconNode;
export declare const PauseOctagon: IconNode;
export declare const Pause: IconNode;
export declare const PenTool: IconNode;
export declare const Pencil: IconNode;
export declare const Percent: IconNode;
export declare const PersonStanding: IconNode;
export declare const PhoneCall: IconNode;
export declare const PhoneForwarded: IconNode;
export declare const PhoneIncoming: IconNode;
export declare const PhoneMissed: IconNode;
export declare const PhoneOff: IconNode;
export declare const PhoneOutgoing: IconNode;
export declare const Phone: IconNode;
export declare const PieChart: IconNode;
export declare const PiggyBank: IconNode;
export declare const Pin: IconNode;
export declare const Pipette: IconNode;
export declare const Plane: IconNode;
export declare const PlayCircle: IconNode;
export declare const Play: IconNode;
export declare const PlugZap: IconNode;
export declare const PlusCircle: IconNode;
export declare const PlusSquare: IconNode;
export declare const Plus: IconNode;
export declare const Pocket: IconNode;
export declare const Podcast: IconNode;
export declare const Pointer: IconNode;
export declare const PoundSterling: IconNode;
export declare const PowerOff: IconNode;
export declare const Power: IconNode;
export declare const Printer: IconNode;
export declare const QrCode: IconNode;
export declare const Quote: IconNode;
export declare const RadioReceiver: IconNode;
export declare const Radio: IconNode;
export declare const Redo: IconNode;
export declare const RefreshCcw: IconNode;
export declare const RefreshCw: IconNode;
export declare const Regex: IconNode;
export declare const Repeat1: IconNode;
export declare const Repeat: IconNode;
export declare const ReplyAll: IconNode;
export declare const Reply: IconNode;
export declare const Rewind: IconNode;
export declare const Rocket: IconNode;
export declare const RockingChair: IconNode;
export declare const RotateCcw: IconNode;
export declare const RotateCw: IconNode;
export declare const Rss: IconNode;
export declare const Ruler: IconNode;
export declare const RussianRuble: IconNode;
export declare const Save: IconNode;
export declare const Scale: IconNode;
export declare const ScanLine: IconNode;
export declare const Scan: IconNode;
export declare const Scissors: IconNode;
export declare const ScreenShareOff: IconNode;
export declare const ScreenShare: IconNode;
export declare const Search: IconNode;
export declare const Send: IconNode;
export declare const SeparatorHorizontal: IconNode;
export declare const SeparatorVertical: IconNode;
export declare const ServerCrash: IconNode;
export declare const ServerOff: IconNode;
export declare const Server: IconNode;
export declare const Settings2: IconNode;
export declare const Settings: IconNode;
export declare const Share2: IconNode;
export declare const Share: IconNode;
export declare const Sheet: IconNode;
export declare const ShieldAlert: IconNode;
export declare const ShieldCheck: IconNode;
export declare const ShieldClose: IconNode;
export declare const ShieldOff: IconNode;
export declare const Shield: IconNode;
export declare const Shirt: IconNode;
export declare const ShoppingBag: IconNode;
export declare const ShoppingCart: IconNode;
export declare const Shovel: IconNode;
export declare const Shrink: IconNode;
export declare const Shuffle: IconNode;
export declare const SidebarClose: IconNode;
export declare const SidebarOpen: IconNode;
export declare const Sidebar: IconNode;
export declare const Sigma: IconNode;
export declare const SignalHigh: IconNode;
export declare const SignalLow: IconNode;
export declare const SignalMedium: IconNode;
export declare const SignalZero: IconNode;
export declare const Signal: IconNode;
export declare const SkipBack: IconNode;
export declare const SkipForward: IconNode;
export declare const Skull: IconNode;
export declare const Slack: IconNode;
export declare const Slash: IconNode;
export declare const Sliders: IconNode;
export declare const SmartphoneCharging: IconNode;
export declare const Smartphone: IconNode;
export declare const Smile: IconNode;
export declare const Snowflake: IconNode;
export declare const SortAsc: IconNode;
export declare const SortDesc: IconNode;
export declare const Speaker: IconNode;
export declare const Sprout: IconNode;
export declare const Square: IconNode;
export declare const StarHalf: IconNode;
export declare const Star: IconNode;
export declare const StopCircle: IconNode;
export declare const StretchHorizontal: IconNode;
export declare const StretchVertical: IconNode;
export declare const Strikethrough: IconNode;
export declare const Subscript: IconNode;
export declare const Sun: IconNode;
export declare const Sunrise: IconNode;
export declare const Sunset: IconNode;
export declare const Superscript: IconNode;
export declare const SwissFranc: IconNode;
export declare const SwitchCamera: IconNode;
export declare const Table: IconNode;
export declare const Tablet: IconNode;
export declare const Tag: IconNode;
export declare const Target: IconNode;
export declare const Tent: IconNode;
export declare const TerminalSquare: IconNode;
export declare const Terminal: IconNode;
export declare const TextCursorInput: IconNode;
export declare const TextCursor: IconNode;
export declare const ThermometerSnowflake: IconNode;
export declare const ThermometerSun: IconNode;
export declare const Thermometer: IconNode;
export declare const ThumbsDown: IconNode;
export declare const ThumbsUp: IconNode;
export declare const Ticket: IconNode;
export declare const TimerOff: IconNode;
export declare const TimerReset: IconNode;
export declare const Timer: IconNode;
export declare const ToggleLeft: IconNode;
export declare const ToggleRight: IconNode;
export declare const Tornado: IconNode;
export declare const Trash2: IconNode;
export declare const Trash: IconNode;
export declare const Trello: IconNode;
export declare const TrendingDown: IconNode;
export declare const TrendingUp: IconNode;
export declare const Triangle: IconNode;
export declare const Truck: IconNode;
export declare const Tv2: IconNode;
export declare const Tv: IconNode;
export declare const Twitch: IconNode;
export declare const Twitter: IconNode;
export declare const Type: IconNode;
export declare const Umbrella: IconNode;
export declare const Underline: IconNode;
export declare const Undo: IconNode;
export declare const Unlink2: IconNode;
export declare const Unlink: IconNode;
export declare const Unlock: IconNode;
export declare const UploadCloud: IconNode;
export declare const Upload: IconNode;
export declare const UserCheck: IconNode;
export declare const UserMinus: IconNode;
export declare const UserPlus: IconNode;
export declare const UserX: IconNode;
export declare const User: IconNode;
export declare const Users: IconNode;
export declare const Verified: IconNode;
export declare const Vibrate: IconNode;
export declare const VideoOff: IconNode;
export declare const Video: IconNode;
export declare const View: IconNode;
export declare const Voicemail: IconNode;
export declare const Volume1: IconNode;
export declare const Volume2: IconNode;
export declare const VolumeX: IconNode;
export declare const Volume: IconNode;
export declare const Wallet: IconNode;
export declare const Wand: IconNode;
export declare const Watch: IconNode;
export declare const Waves: IconNode;
export declare const Webcam: IconNode;
export declare const WifiOff: IconNode;
export declare const Wifi: IconNode;
export declare const Wind: IconNode;
export declare const WrapText: IconNode;
export declare const Wrench: IconNode;
export declare const XCircle: IconNode;
export declare const XOctagon: IconNode;
export declare const XSquare: IconNode;
export declare const X: IconNode;
export declare const Youtube: IconNode;
export declare const ZapOff: IconNode;
export declare const Zap: IconNode;
export declare const ZoomIn: IconNode;
export declare const ZoomOut: IconNode;
