/**
 * lucide v0.17.17 - ISC
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.lucide = {}));
})(this, (function (exports) { 'use strict';

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  /**
   * Creates a new HTMLElement from icon node
   * @param {string} tag
   * @param {object} attrs
   * @param {array} children
   * @returns {HTMLElement}
   */
  var createElement = function createElement(tag, attrs, children) {
    if (children === void 0) {
      children = [];
    }

    var element = document.createElementNS('http://www.w3.org/2000/svg', tag);
    Object.keys(attrs).forEach(function (name) {
      element.setAttribute(name, attrs[name]);
    });

    if (children.length) {
      children = children.forEach(function (child) {
        var childElement = createElement.apply(void 0, child);
        element.appendChild(childElement);
      });
    }

    return element;
  };
  /**
   * Creates a new HTMLElement from icon node
   * @param {[tag: string, attrs: object, children: array]} iconNode
   * @returns {HTMLElement}
   */


  var createElement$1 = (function (_ref) {
    var tag = _ref[0],
        attrs = _ref[1],
        children = _ref[2];
    return createElement(tag, attrs, children);
  });

  /**
   * Get the attributes of an HTML element.
   * @param {HTMLElement} element
   * @returns {Object}
   */

  var getAttrs = function getAttrs(element) {
    return Array.from(element.attributes).reduce(function (attrs, attr) {
      attrs[attr.name] = attr.value;
      return attrs;
    }, {});
  };
  /**
   * Gets the classNames of an attributes Object.
   * @param {Object} attrs
   * @returns {Array}
   */

  var getClassNames = function getClassNames(attrs) {
    if (typeof attrs === 'string') return attrs;
    if (!attrs || !attrs["class"]) return '';

    if (attrs["class"] && typeof attrs["class"] === 'string') {
      return attrs["class"].split(' ');
    }

    if (attrs["class"] && Array.isArray(attrs["class"])) {
      return attrs["class"];
    }

    return '';
  };
  /**
   * Combines the classNames of array of classNames to a String
   * @param {array} arrayOfClassnames
   * @returns {string}
   */

  var combineClassNames = function combineClassNames(arrayOfClassnames) {
    var classNameArray = arrayOfClassnames.flatMap(getClassNames);
    return classNameArray.map(function (classItem) {
      return classItem.trim();
    }).filter(Boolean).filter(function (value, index, self) {
      return self.indexOf(value) === index;
    }).join(' ');
  };

  var toPascalCase = function toPascalCase(string) {
    return string.replace(/(\w)(\w*)(_|-|\s*)/g, function (g0, g1, g2) {
      return g1.toUpperCase() + g2.toLowerCase();
    });
  };
  /**
   * ReplaceElement, replaces the given element with the created icon.
   * @param {HTMLElement} element
   * @param {{ nameAttr: string, icons: object, attrs: object }} options: { nameAttr, icons, attrs }
   * @returns {Function}
   */


  var replaceElement = (function (element, _ref) {
    var nameAttr = _ref.nameAttr,
        icons = _ref.icons,
        attrs = _ref.attrs;
    var iconName = element.getAttribute(nameAttr);
    var ComponentName = toPascalCase(iconName);
    var iconNode = icons[ComponentName];

    if (!iconNode) {
      return console.warn(element.outerHTML + " icon name was not found in the provided icons object.");
    }

    var elementAttrs = getAttrs(element);
    var tag = iconNode[0],
        iconAttributes = iconNode[1],
        children = iconNode[2];

    var iconAttrs = _extends({}, iconAttributes, {
      'icon-name': iconName
    }, attrs, elementAttrs);

    var classNames = combineClassNames(['lucide', "lucide-" + iconName, elementAttrs, attrs]);

    if (classNames) {
      iconAttrs["class"] = classNames;
    }

    var svgElement = createElement$1([tag, iconAttrs, children]);
    return element.parentNode.replaceChild(svgElement, element);
  });

  var defaultAttributes = {
    xmlns: 'http://www.w3.org/2000/svg',
    width: 24,
    height: 24,
    viewBox: '0 0 24 24',
    fill: 'none',
    stroke: 'currentColor',
    'stroke-width': 2,
    'stroke-linecap': 'round',
    'stroke-linejoin': 'round'
  };

  var Activity = ['svg', defaultAttributes, [['polyline', {
    points: '22 12 18 12 15 21 9 3 6 12 2 12'
  }]]];
  var Activity$1 = Activity;

  var Airplay = ['svg', defaultAttributes, [['path', {
    d: 'M5 17H4a2 2 0 01-2-2V5a2 2 0 012-2h16a2 2 0 012 2v10a2 2 0 01-2 2h-1'
  }], ['polygon', {
    points: '12 15 17 21 7 21 12 15'
  }]]];
  var Airplay$1 = Airplay;

  var AlarmCheck = ['svg', defaultAttributes, [['path', {
    d: 'M12 21a8 8 0 100-16 8 8 0 000 16z'
  }], ['path', {
    d: 'M5 3L2 6'
  }], ['path', {
    d: 'M22 6l-3-3'
  }], ['path', {
    d: 'M6 19l-2 2'
  }], ['path', {
    d: 'M18 19l2 2'
  }], ['path', {
    d: 'M9 13l2 2 4-4'
  }]]];
  var AlarmCheck$1 = AlarmCheck;

  var AlarmClockOff = ['svg', defaultAttributes, [['path', {
    d: 'M19.94 14A8 8 0 0010 5.25m8.13 12.89A8 8 0 116.87 6.86'
  }], ['path', {
    d: 'M22 6l-3-3'
  }], ['path', {
    d: 'M6 19l-2 2'
  }], ['path', {
    d: 'M2 2l20 20'
  }], ['path', {
    d: 'M4 4L2 6'
  }]]];
  var AlarmClockOff$1 = AlarmClockOff;

  var AlarmClock = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '13',
    r: '8'
  }], ['path', {
    d: 'M12 9v4l2 2'
  }], ['path', {
    d: 'M5 3L2 6'
  }], ['path', {
    d: 'M22 6l-3-3'
  }], ['path', {
    d: 'M6 19l-2 2'
  }], ['path', {
    d: 'M18 19l2 2'
  }]]];
  var AlarmClock$1 = AlarmClock;

  var AlarmMinus = ['svg', defaultAttributes, [['path', {
    d: 'M12 21a8 8 0 100-16 8 8 0 000 16z'
  }], ['path', {
    d: 'M5 3L2 6'
  }], ['path', {
    d: 'M22 6l-3-3'
  }], ['path', {
    d: 'M6 19l-2 2'
  }], ['path', {
    d: 'M18 19l2 2'
  }], ['path', {
    d: 'M9 13h6'
  }]]];
  var AlarmMinus$1 = AlarmMinus;

  var AlarmPlus = ['svg', defaultAttributes, [['path', {
    d: 'M12 21a8 8 0 100-16 8 8 0 000 16z'
  }], ['path', {
    d: 'M5 3L2 6'
  }], ['path', {
    d: 'M22 6l-3-3'
  }], ['path', {
    d: 'M6 19l-2 2'
  }], ['path', {
    d: 'M18 19l2 2'
  }], ['path', {
    d: 'M12 10v6'
  }], ['path', {
    d: 'M9 13h6'
  }]]];
  var AlarmPlus$1 = AlarmPlus;

  var Album = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['polyline', {
    points: '11 3 11 11 14 8 17 11 17 3'
  }]]];
  var Album$1 = Album;

  var AlertCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '12',
    y1: '8',
    x2: '12',
    y2: '12'
  }], ['line', {
    x1: '12',
    y1: '16',
    x2: '12.01',
    y2: '16'
  }]]];
  var AlertCircle$1 = AlertCircle;

  var AlertOctagon = ['svg', defaultAttributes, [['polygon', {
    points: '7.86 2 16.14 2 22 7.86 22 16.14 16.14 22 7.86 22 2 16.14 2 7.86 7.86 2'
  }], ['line', {
    x1: '12',
    y1: '8',
    x2: '12',
    y2: '12'
  }], ['line', {
    x1: '12',
    y1: '16',
    x2: '12.01',
    y2: '16'
  }]]];
  var AlertOctagon$1 = AlertOctagon;

  var AlertTriangle = ['svg', defaultAttributes, [['path', {
    d: 'M10.29 3.86L1.82 18a2 2 0 001.71 3h16.94a2 2 0 001.71-3L13.71 3.86a2 2 0 00-3.42 0z'
  }], ['line', {
    x1: '12',
    y1: '9',
    x2: '12',
    y2: '13'
  }], ['line', {
    x1: '12',
    y1: '17',
    x2: '12.01',
    y2: '17'
  }]]];
  var AlertTriangle$1 = AlertTriangle;

  var AlignCenterHorizontal = ['svg', defaultAttributes, [['path', {
    d: 'M2 12h20'
  }], ['path', {
    d: 'M10 16v4a2 2 0 01-2 2H6a2 2 0 01-2-2v-4'
  }], ['path', {
    d: 'M10 8V4a2 2 0 00-2-2H6a2 2 0 00-2 2v4'
  }], ['path', {
    d: 'M20 16v1a2 2 0 01-2 2h-2a2 2 0 01-2-2v-1'
  }], ['path', {
    d: 'M14 8V7c0-1.1.9-2 2-2h2a2 2 0 012 2v1'
  }]]];
  var AlignCenterHorizontal$1 = AlignCenterHorizontal;

  var AlignCenterVertical = ['svg', defaultAttributes, [['path', {
    d: 'M12 2v20'
  }], ['path', {
    d: 'M8 10H4a2 2 0 01-2-2V6c0-1.1.9-2 2-2h4'
  }], ['path', {
    d: 'M16 10h4a2 2 0 002-2V6a2 2 0 00-2-2h-4'
  }], ['path', {
    d: 'M8 20H7a2 2 0 01-2-2v-2c0-1.1.9-2 2-2h1'
  }], ['path', {
    d: 'M16 14h1a2 2 0 012 2v2a2 2 0 01-2 2h-1'
  }]]];
  var AlignCenterVertical$1 = AlignCenterVertical;

  var AlignCenter = ['svg', defaultAttributes, [['line', {
    x1: '21',
    y1: '6',
    x2: '3',
    y2: '6'
  }], ['line', {
    x1: '17',
    y1: '12',
    x2: '7',
    y2: '12'
  }], ['line', {
    x1: '19',
    y1: '18',
    x2: '5',
    y2: '18'
  }]]];
  var AlignCenter$1 = AlignCenter;

  var AlignEndHorizontal = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '2',
    width: '6',
    height: '16',
    rx: '2'
  }], ['rect', {
    x: '14',
    y: '9',
    width: '6',
    height: '9',
    rx: '2'
  }], ['path', {
    d: 'M22 22H2'
  }]]];
  var AlignEndHorizontal$1 = AlignEndHorizontal;

  var AlignEndVertical = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '4',
    width: '16',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '9',
    y: '14',
    width: '9',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M22 22V2'
  }]]];
  var AlignEndVertical$1 = AlignEndVertical;

  var AlignHorizontalDistributeCenter = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '5',
    width: '6',
    height: '14',
    rx: '2'
  }], ['rect', {
    x: '14',
    y: '7',
    width: '6',
    height: '10',
    rx: '2'
  }], ['path', {
    d: 'M17 22v-5'
  }], ['path', {
    d: 'M17 7V2'
  }], ['path', {
    d: 'M7 22v-3'
  }], ['path', {
    d: 'M7 5V2'
  }]]];
  var AlignHorizontalDistributeCenter$1 = AlignHorizontalDistributeCenter;

  var AlignHorizontalDistributeEnd = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '5',
    width: '6',
    height: '14',
    rx: '2'
  }], ['rect', {
    x: '14',
    y: '7',
    width: '6',
    height: '10',
    rx: '2'
  }], ['path', {
    d: 'M10 2v20'
  }], ['path', {
    d: 'M20 2v20'
  }]]];
  var AlignHorizontalDistributeEnd$1 = AlignHorizontalDistributeEnd;

  var AlignHorizontalDistributeStart = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '5',
    width: '6',
    height: '14',
    rx: '2'
  }], ['rect', {
    x: '14',
    y: '7',
    width: '6',
    height: '10',
    rx: '2'
  }], ['path', {
    d: 'M4 2v20'
  }], ['path', {
    d: 'M14 2v20'
  }]]];
  var AlignHorizontalDistributeStart$1 = AlignHorizontalDistributeStart;

  var AlignHorizontalJustifyCenter = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '5',
    width: '6',
    height: '14',
    rx: '2'
  }], ['rect', {
    x: '16',
    y: '7',
    width: '6',
    height: '10',
    rx: '2'
  }], ['path', {
    d: 'M12 2v20'
  }]]];
  var AlignHorizontalJustifyCenter$1 = AlignHorizontalJustifyCenter;

  var AlignHorizontalJustifyEnd = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '5',
    width: '6',
    height: '14',
    rx: '2'
  }], ['rect', {
    x: '12',
    y: '7',
    width: '6',
    height: '10',
    rx: '2'
  }], ['path', {
    d: 'M22 2v20'
  }]]];
  var AlignHorizontalJustifyEnd$1 = AlignHorizontalJustifyEnd;

  var AlignHorizontalJustifyStart = ['svg', defaultAttributes, [['rect', {
    x: '6',
    y: '5',
    width: '6',
    height: '14',
    rx: '2'
  }], ['rect', {
    x: '16',
    y: '7',
    width: '6',
    height: '10',
    rx: '2'
  }], ['path', {
    d: 'M2 2v20'
  }]]];
  var AlignHorizontalJustifyStart$1 = AlignHorizontalJustifyStart;

  var AlignHorizontalSpaceAround = ['svg', defaultAttributes, [['rect', {
    x: '9',
    y: '7',
    width: '6',
    height: '10',
    rx: '2'
  }], ['path', {
    d: 'M4 22V2'
  }], ['path', {
    d: 'M20 22V2'
  }]]];
  var AlignHorizontalSpaceAround$1 = AlignHorizontalSpaceAround;

  var AlignHorizontalSpaceBetween = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '5',
    width: '6',
    height: '14',
    rx: '2'
  }], ['rect', {
    x: '15',
    y: '7',
    width: '6',
    height: '10',
    rx: '2'
  }], ['path', {
    d: 'M3 2v20'
  }], ['path', {
    d: 'M21 2v20'
  }]]];
  var AlignHorizontalSpaceBetween$1 = AlignHorizontalSpaceBetween;

  var AlignJustify = ['svg', defaultAttributes, [['line', {
    x1: '3',
    y1: '6',
    x2: '21',
    y2: '6'
  }], ['line', {
    x1: '3',
    y1: '12',
    x2: '21',
    y2: '12'
  }], ['line', {
    x1: '3',
    y1: '18',
    x2: '21',
    y2: '18'
  }]]];
  var AlignJustify$1 = AlignJustify;

  var AlignLeft = ['svg', defaultAttributes, [['line', {
    x1: '21',
    y1: '6',
    x2: '3',
    y2: '6'
  }], ['line', {
    x1: '15',
    y1: '12',
    x2: '3',
    y2: '12'
  }], ['line', {
    x1: '17',
    y1: '18',
    x2: '3',
    y2: '18'
  }]]];
  var AlignLeft$1 = AlignLeft;

  var AlignRight = ['svg', defaultAttributes, [['line', {
    x1: '21',
    y1: '6',
    x2: '3',
    y2: '6'
  }], ['line', {
    x1: '21',
    y1: '12',
    x2: '9',
    y2: '12'
  }], ['line', {
    x1: '21',
    y1: '18',
    x2: '7',
    y2: '18'
  }]]];
  var AlignRight$1 = AlignRight;

  var AlignStartHorizontal = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '6',
    width: '6',
    height: '16',
    rx: '2'
  }], ['rect', {
    x: '14',
    y: '6',
    width: '6',
    height: '9',
    rx: '2'
  }], ['path', {
    d: 'M22 2H2'
  }]]];
  var AlignStartHorizontal$1 = AlignStartHorizontal;

  var AlignStartVertical = ['svg', defaultAttributes, [['rect', {
    x: '6',
    y: '14',
    width: '9',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '6',
    y: '4',
    width: '16',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M2 2v20'
  }]]];
  var AlignStartVertical$1 = AlignStartVertical;

  var AlignVerticalDistributeCenter = ['svg', defaultAttributes, [['rect', {
    x: '5',
    y: '14',
    width: '14',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '7',
    y: '4',
    width: '10',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M22 7h-5'
  }], ['path', {
    d: 'M7 7H1'
  }], ['path', {
    d: 'M22 17h-3'
  }], ['path', {
    d: 'M5 17H2'
  }]]];
  var AlignVerticalDistributeCenter$1 = AlignVerticalDistributeCenter;

  var AlignVerticalDistributeEnd = ['svg', defaultAttributes, [['rect', {
    x: '5',
    y: '14',
    width: '14',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '7',
    y: '4',
    width: '10',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M2 20h20'
  }], ['path', {
    d: 'M2 10h20'
  }]]];
  var AlignVerticalDistributeEnd$1 = AlignVerticalDistributeEnd;

  var AlignVerticalDistributeStart = ['svg', defaultAttributes, [['rect', {
    x: '5',
    y: '14',
    width: '14',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '7',
    y: '4',
    width: '10',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M2 14h20'
  }], ['path', {
    d: 'M2 4h20'
  }]]];
  var AlignVerticalDistributeStart$1 = AlignVerticalDistributeStart;

  var AlignVerticalJustifyCenter = ['svg', defaultAttributes, [['rect', {
    x: '5',
    y: '16',
    width: '14',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '7',
    y: '2',
    width: '10',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M2 12h20'
  }]]];
  var AlignVerticalJustifyCenter$1 = AlignVerticalJustifyCenter;

  var AlignVerticalJustifyEnd = ['svg', defaultAttributes, [['rect', {
    x: '5',
    y: '12',
    width: '14',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '7',
    y: '2',
    width: '10',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M2 22h20'
  }]]];
  var AlignVerticalJustifyEnd$1 = AlignVerticalJustifyEnd;

  var AlignVerticalJustifyStart = ['svg', defaultAttributes, [['rect', {
    x: '5',
    y: '16',
    width: '14',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '7',
    y: '6',
    width: '10',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M2 2h20'
  }]]];
  var AlignVerticalJustifyStart$1 = AlignVerticalJustifyStart;

  var AlignVerticalSpaceAround = ['svg', defaultAttributes, [['rect', {
    x: '7',
    y: '9',
    width: '10',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M22 20H2'
  }], ['path', {
    d: 'M22 4H2'
  }]]];
  var AlignVerticalSpaceAround$1 = AlignVerticalSpaceAround;

  var AlignVerticalSpaceBetween = ['svg', defaultAttributes, [['rect', {
    x: '5',
    y: '15',
    width: '14',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '7',
    y: '3',
    width: '10',
    height: '6',
    rx: '2'
  }], ['path', {
    d: 'M2 21h20'
  }], ['path', {
    d: 'M2 3h20'
  }]]];
  var AlignVerticalSpaceBetween$1 = AlignVerticalSpaceBetween;

  var Anchor = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '5',
    r: '3'
  }], ['line', {
    x1: '12',
    y1: '22',
    x2: '12',
    y2: '8'
  }], ['path', {
    d: 'M5 12H2a10 10 0 0020 0h-3'
  }]]];
  var Anchor$1 = Anchor;

  var Aperture = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '14.31',
    y1: '8',
    x2: '20.05',
    y2: '17.94'
  }], ['line', {
    x1: '9.69',
    y1: '8',
    x2: '21.17',
    y2: '8'
  }], ['line', {
    x1: '7.38',
    y1: '12',
    x2: '13.12',
    y2: '2.06'
  }], ['line', {
    x1: '9.69',
    y1: '16',
    x2: '3.95',
    y2: '6.06'
  }], ['line', {
    x1: '14.31',
    y1: '16',
    x2: '2.83',
    y2: '16'
  }], ['line', {
    x1: '16.62',
    y1: '12',
    x2: '10.88',
    y2: '21.94'
  }]]];
  var Aperture$1 = Aperture;

  var Archive = ['svg', defaultAttributes, [['path', {
    d: 'M20 9v9a2 2 0 01-2 2H6a2 2 0 01-2-2V9m16-5H4a2 2 0 00-2 2v1a2 2 0 002 2h16a2 2 0 002-2V6a2 2 0 00-2-2zm-10 9h4'
  }]]];
  var Archive$1 = Archive;

  var ArrowBigDown = ['svg', defaultAttributes, [['path', {
    d: 'M9 3h6v11h4l-7 7-7-7h4z'
  }]]];
  var ArrowBigDown$1 = ArrowBigDown;

  var ArrowBigLeft = ['svg', defaultAttributes, [['path', {
    d: 'M3 12l7-7v4h11v6H10v4z'
  }]]];
  var ArrowBigLeft$1 = ArrowBigLeft;

  var ArrowBigRight = ['svg', defaultAttributes, [['path', {
    d: 'M21 12l-7-7v4H3v6h11v4z'
  }]]];
  var ArrowBigRight$1 = ArrowBigRight;

  var ArrowBigUp = ['svg', defaultAttributes, [['path', {
    d: 'M9 21V10H5l7-7 7 7h-4v11z'
  }]]];
  var ArrowBigUp$1 = ArrowBigUp;

  var ArrowDownCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '8 12 12 16 16 12'
  }], ['line', {
    x1: '12',
    y1: '8',
    x2: '12',
    y2: '16'
  }]]];
  var ArrowDownCircle$1 = ArrowDownCircle;

  var ArrowDownLeft = ['svg', defaultAttributes, [['line', {
    x1: '17',
    y1: '7',
    x2: '7',
    y2: '17'
  }], ['polyline', {
    points: '17 17 7 17 7 7'
  }]]];
  var ArrowDownLeft$1 = ArrowDownLeft;

  var ArrowDownRight = ['svg', defaultAttributes, [['line', {
    x1: '7',
    y1: '7',
    x2: '17',
    y2: '17'
  }], ['polyline', {
    points: '17 7 17 17 7 17'
  }]]];
  var ArrowDownRight$1 = ArrowDownRight;

  var ArrowDown = ['svg', defaultAttributes, [['line', {
    x1: '12',
    y1: '5',
    x2: '12',
    y2: '19'
  }], ['polyline', {
    points: '19 12 12 19 5 12'
  }]]];
  var ArrowDown$1 = ArrowDown;

  var ArrowLeftCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 8 8 12 12 16'
  }], ['line', {
    x1: '16',
    y1: '12',
    x2: '8',
    y2: '12'
  }]]];
  var ArrowLeftCircle$1 = ArrowLeftCircle;

  var ArrowLeftRight = ['svg', defaultAttributes, [['polyline', {
    points: '17 11 21 7 17 3'
  }], ['line', {
    x1: '21',
    y1: '7',
    x2: '9',
    y2: '7'
  }], ['polyline', {
    points: '7 21 3 17 7 13'
  }], ['line', {
    x1: '15',
    y1: '17',
    x2: '3',
    y2: '17'
  }]]];
  var ArrowLeftRight$1 = ArrowLeftRight;

  var ArrowLeft = ['svg', defaultAttributes, [['line', {
    x1: '19',
    y1: '12',
    x2: '5',
    y2: '12'
  }], ['polyline', {
    points: '12 19 5 12 12 5'
  }]]];
  var ArrowLeft$1 = ArrowLeft;

  var ArrowRightCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 16 16 12 12 8'
  }], ['line', {
    x1: '8',
    y1: '12',
    x2: '16',
    y2: '12'
  }]]];
  var ArrowRightCircle$1 = ArrowRightCircle;

  var ArrowRight = ['svg', defaultAttributes, [['line', {
    x1: '5',
    y1: '12',
    x2: '19',
    y2: '12'
  }], ['polyline', {
    points: '12 5 19 12 12 19'
  }]]];
  var ArrowRight$1 = ArrowRight;

  var ArrowUpCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '16 12 12 8 8 12'
  }], ['line', {
    x1: '12',
    y1: '16',
    x2: '12',
    y2: '8'
  }]]];
  var ArrowUpCircle$1 = ArrowUpCircle;

  var ArrowUpLeft = ['svg', defaultAttributes, [['line', {
    x1: '17',
    y1: '17',
    x2: '7',
    y2: '7'
  }], ['polyline', {
    points: '7 17 7 7 17 7'
  }]]];
  var ArrowUpLeft$1 = ArrowUpLeft;

  var ArrowUpRight = ['svg', defaultAttributes, [['line', {
    x1: '7',
    y1: '17',
    x2: '17',
    y2: '7'
  }], ['polyline', {
    points: '7 7 17 7 17 17'
  }]]];
  var ArrowUpRight$1 = ArrowUpRight;

  var ArrowUp = ['svg', defaultAttributes, [['line', {
    x1: '12',
    y1: '19',
    x2: '12',
    y2: '5'
  }], ['polyline', {
    points: '5 12 12 5 19 12'
  }]]];
  var ArrowUp$1 = ArrowUp;

  var Asterisk = ['svg', defaultAttributes, [['path', {
    d: 'M12 6v12'
  }], ['path', {
    d: 'M17.196 9L6.804 15'
  }], ['path', {
    d: 'M6.804 9l10.392 6'
  }]]];
  var Asterisk$1 = Asterisk;

  var AtSign = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '4'
  }], ['path', {
    d: 'M16 8v5a3 3 0 006 0v-1a10 10 0 10-3.92 7.94'
  }]]];
  var AtSign$1 = AtSign;

  var Award = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '8',
    r: '7'
  }], ['polyline', {
    points: '8.21 13.89 7 23 12 20 17 23 15.79 13.88'
  }]]];
  var Award$1 = Award;

  var Axe = ['svg', defaultAttributes, [['path', {
    d: 'M14 12l-8.501 8.501a2.12 2.12 0 01-2.998 0h-.002a2.12 2.12 0 010-2.998L11 9.002'
  }], ['path', {
    d: 'M9 7l4-4 6 6h3l-.13.648a7.648 7.648 0 01-5.081 5.756L15 16v-3z'
  }]]];
  var Axe$1 = Axe;

  var Banknote = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '6',
    width: '20',
    height: '12',
    rx: '2'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '2'
  }], ['path', {
    d: 'M6 12h.01M18 12h.01'
  }]]];
  var Banknote$1 = Banknote;

  var BarChart2 = ['svg', defaultAttributes, [['line', {
    x1: '18',
    y1: '20',
    x2: '18',
    y2: '10'
  }], ['line', {
    x1: '12',
    y1: '20',
    x2: '12',
    y2: '4'
  }], ['line', {
    x1: '6',
    y1: '20',
    x2: '6',
    y2: '14'
  }]]];
  var BarChart2$1 = BarChart2;

  var BarChart = ['svg', defaultAttributes, [['line', {
    x1: '12',
    y1: '20',
    x2: '12',
    y2: '10'
  }], ['line', {
    x1: '18',
    y1: '20',
    x2: '18',
    y2: '4'
  }], ['line', {
    x1: '6',
    y1: '20',
    x2: '6',
    y2: '16'
  }]]];
  var BarChart$1 = BarChart;

  var Baseline = ['svg', defaultAttributes, [['path', {
    d: 'M4 20h16'
  }], ['path', {
    d: 'M6 16l6-12 6 12'
  }], ['path', {
    d: 'M8 12h8'
  }]]];
  var Baseline$1 = Baseline;

  var BatteryCharging = ['svg', defaultAttributes, [['path', {
    d: 'M14 7h2a2 2 0 012 2v6a2 2 0 01-2 2h-3'
  }], ['path', {
    d: 'M7 7H4a2 2 0 00-2 2v6a2 2 0 002 2h2'
  }], ['polyline', {
    points: '11 7 8 12 12 12 9 17'
  }], ['line', {
    x1: '22',
    x2: '22',
    y1: '11',
    y2: '13'
  }]]];
  var BatteryCharging$1 = BatteryCharging;

  var BatteryFull = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '7',
    width: '16',
    height: '10',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '22',
    x2: '22',
    y1: '11',
    y2: '13'
  }], ['line', {
    x1: '6',
    x2: '6',
    y1: '10',
    y2: '14'
  }], ['line', {
    x1: '10',
    x2: '10',
    y1: '10',
    y2: '14'
  }], ['line', {
    x1: '14',
    x2: '14',
    y1: '10',
    y2: '14'
  }]]];
  var BatteryFull$1 = BatteryFull;

  var BatteryLow = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '7',
    width: '16',
    height: '10',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '22',
    x2: '22',
    y1: '11',
    y2: '13'
  }], ['line', {
    x1: '6',
    x2: '6',
    y1: '10',
    y2: '14'
  }]]];
  var BatteryLow$1 = BatteryLow;

  var BatteryMedium = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '7',
    width: '16',
    height: '10',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '22',
    x2: '22',
    y1: '11',
    y2: '13'
  }], ['line', {
    x1: '6',
    x2: '6',
    y1: '10',
    y2: '14'
  }], ['line', {
    x1: '10',
    x2: '10',
    y1: '10',
    y2: '14'
  }]]];
  var BatteryMedium$1 = BatteryMedium;

  var Battery = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '7',
    width: '16',
    height: '10',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '22',
    x2: '22',
    y1: '11',
    y2: '13'
  }]]];
  var Battery$1 = Battery;

  var Beaker = ['svg', defaultAttributes, [['path', {
    d: 'M4.5 3h15'
  }], ['path', {
    d: 'M6 3v16a2 2 0 002 2h8a2 2 0 002-2V3'
  }], ['path', {
    d: 'M6 14h12'
  }]]];
  var Beaker$1 = Beaker;

  var BellMinus = ['svg', defaultAttributes, [['path', {
    d: 'M13.73 21a2 2 0 01-3.46 0'
  }], ['path', {
    d: 'M21 5h-6'
  }], ['path', {
    d: 'M18.021 9C18.29 15.193 21 17 21 17H3s3-2 3-9a6 6 0 017-5.916'
  }]]];
  var BellMinus$1 = BellMinus;

  var BellOff = ['svg', defaultAttributes, [['path', {
    d: 'M13.73 21a2 2 0 01-3.46 0'
  }], ['path', {
    d: 'M18.63 13A17.888 17.888 0 0118 8'
  }], ['path', {
    d: 'M6.26 6.26A5.86 5.86 0 006 8c0 7-3 9-3 9h14'
  }], ['path', {
    d: 'M18 8a6 6 0 00-9.33-5'
  }], ['path', {
    d: 'M2 2l20 20'
  }]]];
  var BellOff$1 = BellOff;

  var BellPlus = ['svg', defaultAttributes, [['path', {
    d: 'M18.387 12C19.198 15.799 21 17 21 17H3s3-2 3-9a6 6 0 017-5.916'
  }], ['path', {
    d: 'M13.73 21a2 2 0 01-3.46 0'
  }], ['path', {
    d: 'M18 2v6'
  }], ['path', {
    d: 'M21 5h-6'
  }]]];
  var BellPlus$1 = BellPlus;

  var BellRing = ['svg', defaultAttributes, [['path', {
    d: 'M18 8A6 6 0 006 8c0 7-3 9-3 9h18s-3-2-3-9'
  }], ['path', {
    d: 'M13.73 21a2 2 0 01-3.46 0'
  }], ['path', {
    d: 'M2 8c0-2.2.7-4.3 2-6'
  }], ['path', {
    d: 'M22 8a10 10 0 00-2-6'
  }]]];
  var BellRing$1 = BellRing;

  var Bell = ['svg', defaultAttributes, [['path', {
    d: 'M18 8A6 6 0 006 8c0 7-3 9-3 9h18s-3-2-3-9'
  }], ['path', {
    d: 'M13.73 21a2 2 0 01-3.46 0'
  }]]];
  var Bell$1 = Bell;

  var Bike = ['svg', defaultAttributes, [['circle', {
    cx: '5.5',
    cy: '17.5',
    r: '3.5'
  }], ['circle', {
    cx: '18.5',
    cy: '17.5',
    r: '3.5'
  }], ['path', {
    d: 'M15 6a1 1 0 100-2 1 1 0 000 2zm-3 11.5V14l-3-3 4-3 2 3h2'
  }]]];
  var Bike$1 = Bike;

  var Binary = ['svg', defaultAttributes, [['path', {
    d: 'M10 4H6v6h4V4z'
  }], ['path', {
    d: 'M18 14h-4v6h4v-6z'
  }], ['path', {
    d: 'M14 4h2v6m-2 0h4'
  }], ['path', {
    d: 'M6 14h2v6m-2 0h4'
  }]]];
  var Binary$1 = Binary;

  var Bitcoin = ['svg', defaultAttributes, [['path', {
    d: 'M11.767 19.089c4.924.868 6.14-6.025 1.216-6.894m-1.216 6.894L5.86 18.047m5.908 1.042l-.347 1.97m1.563-8.864c4.924.869 6.14-6.025 1.215-6.893m-1.215 6.893l-3.94-.694m5.155-6.2L8.29 4.26m5.908 1.042l.348-1.97M7.48 20.364l3.126-17.727'
  }]]];
  var Bitcoin$1 = Bitcoin;

  var BluetoothConnected = ['svg', defaultAttributes, [['path', {
    d: 'M7 7l10 10-5 5V2l5 5L7 17'
  }], ['line', {
    x1: '18',
    y1: '12',
    y2: '12',
    x2: '21'
  }], ['line', {
    x1: '3',
    y1: '12',
    y2: '12',
    x2: '6'
  }]]];
  var BluetoothConnected$1 = BluetoothConnected;

  var BluetoothOff = ['svg', defaultAttributes, [['path', {
    d: 'M17 17l-5 5V12l-5 5'
  }], ['path', {
    d: 'M2 2l20 20'
  }], ['path', {
    d: 'M14.5 9.5L17 7l-5-5v4.5'
  }]]];
  var BluetoothOff$1 = BluetoothOff;

  var BluetoothSearching = ['svg', defaultAttributes, [['path', {
    d: 'M7 7l10 10-5 5V2l5 5L7 17'
  }], ['path', {
    d: 'M20.83 14.83a4 4 0 000-5.66'
  }], ['path', {
    d: 'M18 12h.01'
  }]]];
  var BluetoothSearching$1 = BluetoothSearching;

  var Bluetooth = ['svg', defaultAttributes, [['path', {
    d: 'M7 7l10 10-5 5V2l5 5L7 17'
  }]]];
  var Bluetooth$1 = Bluetooth;

  var Bold = ['svg', defaultAttributes, [['path', {
    d: 'M6 4h8a4 4 0 014 4 4 4 0 01-4 4H6z'
  }], ['path', {
    d: 'M6 12h9a4 4 0 014 4 4 4 0 01-4 4H6z'
  }]]];
  var Bold$1 = Bold;

  var BookOpen = ['svg', defaultAttributes, [['path', {
    d: 'M2 3h6a4 4 0 014 4v14a3 3 0 00-3-3H2z'
  }], ['path', {
    d: 'M22 3h-6a4 4 0 00-4 4v14a3 3 0 013-3h7z'
  }]]];
  var BookOpen$1 = BookOpen;

  var Book = ['svg', defaultAttributes, [['path', {
    d: 'M4 19.5A2.5 2.5 0 016.5 17H20'
  }], ['path', {
    d: 'M6.5 2H20v20H6.5A2.5 2.5 0 014 19.5v-15A2.5 2.5 0 016.5 2z'
  }]]];
  var Book$1 = Book;

  var BookmarkMinus = ['svg', defaultAttributes, [['path', {
    d: 'M19 21l-7-4-7 4V5a2 2 0 012-2h10a2 2 0 012 2v16z'
  }], ['line', {
    x1: '15',
    x2: '9',
    y1: '10',
    y2: '10'
  }]]];
  var BookmarkMinus$1 = BookmarkMinus;

  var BookmarkPlus = ['svg', defaultAttributes, [['path', {
    d: 'M19 21l-7-4-7 4V5a2 2 0 012-2h10a2 2 0 012 2v16z'
  }], ['line', {
    x1: '12',
    x2: '12',
    y1: '7',
    y2: '13'
  }], ['line', {
    x1: '15',
    x2: '9',
    y1: '10',
    y2: '10'
  }]]];
  var BookmarkPlus$1 = BookmarkPlus;

  var Bookmark = ['svg', defaultAttributes, [['path', {
    d: 'M19 21l-7-4-7 4V5a2 2 0 012-2h10a2 2 0 012 2v16z'
  }]]];
  var Bookmark$1 = Bookmark;

  var Bot = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '11',
    width: '18',
    height: '10',
    rx: '2'
  }], ['circle', {
    cx: '12',
    cy: '5',
    r: '2'
  }], ['path', {
    d: 'M12 7v4'
  }], ['line', {
    x1: '8',
    y1: '16',
    x2: '8',
    y2: '16'
  }], ['line', {
    x1: '16',
    y1: '16',
    x2: '16',
    y2: '16'
  }]]];
  var Bot$1 = Bot;

  var BoxSelect = ['svg', defaultAttributes, [['path', {
    d: 'M4 2a2 2 0 00-2 2'
  }], ['line', {
    x1: '8',
    y1: '2',
    x2: '10',
    y2: '2'
  }], ['line', {
    x1: '14',
    y1: '2',
    x2: '16',
    y2: '2'
  }], ['path', {
    d: 'M4 22a2 2 0 01-2-2'
  }], ['line', {
    x1: '22',
    y1: '8',
    x2: '22',
    y2: '10'
  }], ['line', {
    x1: '22',
    y1: '14',
    x2: '22',
    y2: '16'
  }], ['path', {
    d: 'M22 20a2 2 0 01-2 2'
  }], ['line', {
    x1: '14',
    y1: '22',
    x2: '16',
    y2: '22'
  }], ['line', {
    x1: '8',
    y1: '22',
    x2: '10',
    y2: '22'
  }], ['path', {
    d: 'M20 2a2 2 0 012 2'
  }], ['line', {
    x1: '2',
    y1: '14',
    x2: '2',
    y2: '16'
  }], ['line', {
    x1: '2',
    y1: '8',
    x2: '2',
    y2: '10'
  }]]];
  var BoxSelect$1 = BoxSelect;

  var Box = ['svg', defaultAttributes, [['path', {
    d: 'M21 16V8a2 2 0 00-1-1.73l-7-4a2 2 0 00-2 0l-7 4A2 2 0 003 8v8a2 2 0 001 1.73l7 4a2 2 0 002 0l7-4A2 2 0 0021 16z'
  }], ['polyline', {
    points: '3.27 6.96 12 12.01 20.73 6.96'
  }], ['line', {
    x1: '12',
    y1: '22.08',
    x2: '12',
    y2: '12'
  }]]];
  var Box$1 = Box;

  var Briefcase = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '7',
    width: '20',
    height: '14',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M16 21V5a2 2 0 00-2-2h-4a2 2 0 00-2 2v16'
  }]]];
  var Briefcase$1 = Briefcase;

  var Brush = ['svg', defaultAttributes, [['path', {
    d: 'M9.06 11.9l8.07-8.06a2.85 2.85 0 114.03 4.03l-8.06 8.08'
  }], ['path', {
    d: 'M7.07 14.94c-1.66 0-3 1.35-3 3.02 0 1.33-2.5 1.52-2 2.02 1.08 1.1 2.49 2.02 4 2.02 2.2 0 4-1.8 4-4.04a3.01 3.01 0 00-3-3.02z'
  }]]];
  var Brush$1 = Brush;

  var Bug = ['svg', defaultAttributes, [['rect', {
    width: '8',
    height: '14',
    x: '8',
    y: '6',
    rx: '4'
  }], ['path', {
    d: 'M19 7l-3 2'
  }], ['path', {
    d: 'M5 7l3 2'
  }], ['path', {
    d: 'M19 19l-3-2'
  }], ['path', {
    d: 'M5 19l3-2'
  }], ['path', {
    d: 'M20 13h-4'
  }], ['path', {
    d: 'M4 13h4'
  }], ['path', {
    d: 'M10 4l1 2'
  }], ['path', {
    d: 'M14 4l-1 2'
  }]]];
  var Bug$1 = Bug;

  var Building = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '2',
    width: '16',
    height: '20',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M9 22v-4h6v4'
  }], ['path', {
    d: 'M8 6h.01'
  }], ['path', {
    d: 'M16 6h.01'
  }], ['path', {
    d: 'M12 6h.01'
  }], ['path', {
    d: 'M12 10h.01'
  }], ['path', {
    d: 'M12 14h.01'
  }], ['path', {
    d: 'M16 10h.01'
  }], ['path', {
    d: 'M16 14h.01'
  }], ['path', {
    d: 'M8 10h.01'
  }], ['path', {
    d: 'M8 14h.01'
  }]]];
  var Building$1 = Building;

  var Bus = ['svg', defaultAttributes, [['path', {
    d: 'M19 17h2l.64-2.54a6 6 0 000-2.92l-1.07-4.27A3 3 0 0017.66 5H4a2 2 0 00-2 2v10h2m10 0h-4'
  }], ['circle', {
    cx: '6.5',
    cy: '17.5',
    r: '2.5'
  }], ['circle', {
    cx: '16.5',
    cy: '17.5',
    r: '2.5'
  }]]];
  var Bus$1 = Bus;

  var Calculator = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '2',
    width: '16',
    height: '20',
    rx: '2'
  }], ['line', {
    x1: '8',
    x2: '16',
    y1: '6',
    y2: '6'
  }], ['line', {
    x1: '16',
    x2: '16',
    y1: '14',
    y2: '18'
  }], ['path', {
    d: 'M16 10h.01'
  }], ['path', {
    d: 'M12 10h.01'
  }], ['path', {
    d: 'M8 10h.01'
  }], ['path', {
    d: 'M12 14h.01'
  }], ['path', {
    d: 'M8 14h.01'
  }], ['path', {
    d: 'M12 18h.01'
  }], ['path', {
    d: 'M8 18h.01'
  }]]];
  var Calculator$1 = Calculator;

  var Calendar = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '4',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '16',
    y1: '2',
    x2: '16',
    y2: '6'
  }], ['line', {
    x1: '8',
    y1: '2',
    x2: '8',
    y2: '6'
  }], ['line', {
    x1: '3',
    y1: '10',
    x2: '21',
    y2: '10'
  }]]];
  var Calendar$1 = Calendar;

  var CameraOff = ['svg', defaultAttributes, [['line', {
    x1: '2',
    y1: '2',
    x2: '22',
    y2: '22'
  }], ['path', {
    d: 'M9.5 4h5L17 7h3a2 2 0 012 2v7.5M7 7H4a2 2 0 00-2 2v9a2 2 0 002 2h16'
  }], ['path', {
    d: 'M14.121 15.121A3 3 0 119.88 10.88'
  }]]];
  var CameraOff$1 = CameraOff;

  var Camera = ['svg', defaultAttributes, [['path', {
    d: 'M14.5 4h-5L7 7H4a2 2 0 00-2 2v9a2 2 0 002 2h16a2 2 0 002-2V9a2 2 0 00-2-2h-3l-2.5-3z'
  }], ['circle', {
    cx: '12',
    cy: '13',
    r: '3'
  }]]];
  var Camera$1 = Camera;

  var Car = ['svg', defaultAttributes, [['path', {
    d: 'M14 16H9m10 0h3v-3.15a1 1 0 00-.84-.99L16 11l-2.7-3.6a1 1 0 00-.8-.4H5.24a2 2 0 00-1.8 1.1l-.8 1.63A6 6 0 002 12.42V16h2'
  }], ['circle', {
    cx: '6.5',
    cy: '16.5',
    r: '2.5'
  }], ['circle', {
    cx: '16.5',
    cy: '16.5',
    r: '2.5'
  }]]];
  var Car$1 = Car;

  var Carrot = ['svg', defaultAttributes, [['path', {
    d: 'M2.27 21.7s9.87-3.5 12.73-6.36a4.5 4.5 0 00-6.36-6.37C5.77 11.84 2.27 21.7 2.27 21.7zM8.64 14l-2.05-2.04M15.34 15l-2.46-2.46'
  }], ['path', {
    d: 'M22 9s-1.33-2-3.5-2C16.86 7 15 9 15 9s1.33 2 3.5 2S22 9 22 9z'
  }], ['path', {
    d: 'M15 2s-2 1.33-2 3.5S15 9 15 9s2-1.84 2-3.5C17 3.33 15 2 15 2z'
  }]]];
  var Carrot$1 = Carrot;

  var Cast = ['svg', defaultAttributes, [['path', {
    d: 'M2 16.1A5 5 0 015.9 20M2 12.05A9 9 0 019.95 20M2 8V6a2 2 0 012-2h16a2 2 0 012 2v12a2 2 0 01-2 2h-6'
  }], ['line', {
    x1: '2',
    y1: '20',
    x2: '2.01',
    y2: '20'
  }]]];
  var Cast$1 = Cast;

  var CheckCircle2 = ['svg', defaultAttributes, [['path', {
    d: 'M12 22c5.523 0 10-4.477 10-10S17.523 2 12 2 2 6.477 2 12s4.477 10 10 10z'
  }], ['path', {
    d: 'M9 12l2 2 4-4'
  }]]];
  var CheckCircle2$1 = CheckCircle2;

  var CheckCircle = ['svg', defaultAttributes, [['path', {
    d: 'M22 11.08V12a10 10 0 11-5.93-9.14'
  }], ['polyline', {
    points: '22 4 12 14.01 9 11.01'
  }]]];
  var CheckCircle$1 = CheckCircle;

  var CheckSquare = ['svg', defaultAttributes, [['polyline', {
    points: '9 11 12 14 22 4'
  }], ['path', {
    d: 'M21 12v7a2 2 0 01-2 2H5a2 2 0 01-2-2V5a2 2 0 012-2h11'
  }]]];
  var CheckSquare$1 = CheckSquare;

  var Check = ['svg', defaultAttributes, [['polyline', {
    points: '20 6 9 17 4 12'
  }]]];
  var Check$1 = Check;

  var ChevronDown = ['svg', defaultAttributes, [['polyline', {
    points: '6 9 12 15 18 9'
  }]]];
  var ChevronDown$1 = ChevronDown;

  var ChevronFirst = ['svg', defaultAttributes, [['polyline', {
    points: '17 18 11 12 17 6'
  }], ['path', {
    d: 'M7 6v12'
  }]]];
  var ChevronFirst$1 = ChevronFirst;

  var ChevronLast = ['svg', defaultAttributes, [['polyline', {
    points: '7 18 13 12 7 6'
  }], ['path', {
    d: 'M17 6v12'
  }]]];
  var ChevronLast$1 = ChevronLast;

  var ChevronLeft = ['svg', defaultAttributes, [['polyline', {
    points: '15 18 9 12 15 6'
  }]]];
  var ChevronLeft$1 = ChevronLeft;

  var ChevronRight = ['svg', defaultAttributes, [['polyline', {
    points: '9 18 15 12 9 6'
  }]]];
  var ChevronRight$1 = ChevronRight;

  var ChevronUp = ['svg', defaultAttributes, [['polyline', {
    points: '18 15 12 9 6 15'
  }]]];
  var ChevronUp$1 = ChevronUp;

  var ChevronsDownUp = ['svg', defaultAttributes, [['path', {
    d: 'M7 20l5-5 5 5'
  }], ['path', {
    d: 'M7 4l5 5 5-5'
  }]]];
  var ChevronsDownUp$1 = ChevronsDownUp;

  var ChevronsDown = ['svg', defaultAttributes, [['polyline', {
    points: '7 13 12 18 17 13'
  }], ['polyline', {
    points: '7 6 12 11 17 6'
  }]]];
  var ChevronsDown$1 = ChevronsDown;

  var ChevronsLeft = ['svg', defaultAttributes, [['polyline', {
    points: '11 17 6 12 11 7'
  }], ['polyline', {
    points: '18 17 13 12 18 7'
  }]]];
  var ChevronsLeft$1 = ChevronsLeft;

  var ChevronsRight = ['svg', defaultAttributes, [['polyline', {
    points: '13 17 18 12 13 7'
  }], ['polyline', {
    points: '6 17 11 12 6 7'
  }]]];
  var ChevronsRight$1 = ChevronsRight;

  var ChevronsUpDown = ['svg', defaultAttributes, [['path', {
    d: 'M7 15l5 5 5-5'
  }], ['path', {
    d: 'M7 9l5-5 5 5'
  }]]];
  var ChevronsUpDown$1 = ChevronsUpDown;

  var ChevronsUp = ['svg', defaultAttributes, [['polyline', {
    points: '17 11 12 6 7 11'
  }], ['polyline', {
    points: '17 18 12 13 7 18'
  }]]];
  var ChevronsUp$1 = ChevronsUp;

  var Chrome = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '4'
  }], ['line', {
    x1: '21.17',
    y1: '8',
    x2: '12',
    y2: '8'
  }], ['line', {
    x1: '3.95',
    y1: '6.06',
    x2: '8.54',
    y2: '14'
  }], ['line', {
    x1: '10.88',
    y1: '21.94',
    x2: '15.46',
    y2: '14'
  }]]];
  var Chrome$1 = Chrome;

  var CircleSlashed = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['path', {
    d: 'M22 2L2 22'
  }]]];
  var CircleSlashed$1 = CircleSlashed;

  var Circle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }]]];
  var Circle$1 = Circle;

  var ClipboardCheck = ['svg', defaultAttributes, [['path', {
    d: 'M16 4h2a2 2 0 012 2v14a2 2 0 01-2 2H6a2 2 0 01-2-2V6a2 2 0 012-2h2'
  }], ['path', {
    d: 'M15 2H9a1 1 0 00-1 1v2a1 1 0 001 1h6a1 1 0 001-1V3a1 1 0 00-1-1z'
  }], ['path', {
    d: 'M9 13l2 2 4-4'
  }]]];
  var ClipboardCheck$1 = ClipboardCheck;

  var ClipboardCopy = ['svg', defaultAttributes, [['path', {
    d: 'M16 4h2a2 2 0 012 2v4M8 4H6a2 2 0 00-2 2v14a2 2 0 002 2h12a2 2 0 002-2v-2'
  }], ['rect', {
    x: '8',
    y: '2',
    width: '8',
    height: '4',
    rx: '1',
    ry: '1'
  }], ['path', {
    d: 'M21 14H11'
  }], ['path', {
    d: 'M15 10l-4 4 4 4'
  }]]];
  var ClipboardCopy$1 = ClipboardCopy;

  var ClipboardList = ['svg', defaultAttributes, [['path', {
    d: 'M16 4h2a2 2 0 012 2v14a2 2 0 01-2 2H6a2 2 0 01-2-2V6a2 2 0 012-2h2'
  }], ['path', {
    d: 'M15 2H9a1 1 0 00-1 1v2a1 1 0 001 1h6a1 1 0 001-1V3a1 1 0 00-1-1z'
  }], ['path', {
    d: 'M12 11h4'
  }], ['path', {
    d: 'M12 16h4'
  }], ['path', {
    d: 'M8 11h.01'
  }], ['path', {
    d: 'M8 16h.01'
  }]]];
  var ClipboardList$1 = ClipboardList;

  var ClipboardX = ['svg', defaultAttributes, [['path', {
    d: 'M16 4h2a2 2 0 012 2v14a2 2 0 01-2 2H6a2 2 0 01-2-2V6a2 2 0 012-2h2'
  }], ['path', {
    d: 'M15 2H9a1 1 0 00-1 1v2a1 1 0 001 1h6a1 1 0 001-1V3a1 1 0 00-1-1z'
  }], ['path', {
    d: 'M15 11l-6 6'
  }], ['path', {
    d: 'M9 11l6 6'
  }]]];
  var ClipboardX$1 = ClipboardX;

  var Clipboard = ['svg', defaultAttributes, [['path', {
    d: 'M16 4h2a2 2 0 012 2v14a2 2 0 01-2 2H6a2 2 0 01-2-2V6a2 2 0 012-2h2'
  }], ['rect', {
    x: '8',
    y: '2',
    width: '8',
    height: '4',
    rx: '1',
    ry: '1'
  }]]];
  var Clipboard$1 = Clipboard;

  var Clock1 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 14.5 8'
  }]]];
  var Clock1$1 = Clock1;

  var Clock10 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 8 10'
  }]]];
  var Clock10$1 = Clock10;

  var Clock11 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 9.5 8'
  }]]];
  var Clock11$1 = Clock11;

  var Clock12 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12'
  }]]];
  var Clock12$1 = Clock12;

  var Clock2 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 16 10'
  }]]];
  var Clock2$1 = Clock2;

  var Clock3 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 16.5 12'
  }]]];
  var Clock3$1 = Clock3;

  var Clock4 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 16 14'
  }]]];
  var Clock4$1 = Clock4;

  var Clock5 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 14.5 16'
  }]]];
  var Clock5$1 = Clock5;

  var Clock6 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 12 16.5'
  }]]];
  var Clock6$1 = Clock6;

  var Clock7 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 9.5 16'
  }]]];
  var Clock7$1 = Clock7;

  var Clock8 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 8 14'
  }]]];
  var Clock8$1 = Clock8;

  var Clock9 = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 7.5 12'
  }]]];
  var Clock9$1 = Clock9;

  var Clock = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polyline', {
    points: '12 6 12 12 16 14'
  }]]];
  var Clock$1 = Clock;

  var CloudDrizzle = ['svg', defaultAttributes, [['path', {
    d: 'M20 16.2A4.5 4.5 0 0017.5 8h-1.8A7 7 0 104 14.9'
  }], ['path', {
    d: 'M8 19v1'
  }], ['path', {
    d: 'M8 14v1'
  }], ['path', {
    d: 'M16 19v1'
  }], ['path', {
    d: 'M16 14v1'
  }], ['path', {
    d: 'M12 21v1'
  }], ['path', {
    d: 'M12 16v1'
  }]]];
  var CloudDrizzle$1 = CloudDrizzle;

  var CloudFog = ['svg', defaultAttributes, [['path', {
    d: 'M20 16.2A4.5 4.5 0 0017.5 8h-1.8A7 7 0 104 14.9'
  }], ['path', {
    d: 'M16 17H7'
  }], ['path', {
    d: 'M17 21H9'
  }]]];
  var CloudFog$1 = CloudFog;

  var CloudHail = ['svg', defaultAttributes, [['path', {
    d: 'M20 16.2A4.5 4.5 0 0017.5 8h-1.8A7 7 0 104 14.9'
  }], ['path', {
    d: 'M16 14v2'
  }], ['path', {
    d: 'M8 14v2'
  }], ['path', {
    d: 'M16 20h0'
  }], ['path', {
    d: 'M8 20h0'
  }], ['path', {
    d: 'M12 16v2'
  }], ['path', {
    d: 'M12 22h0'
  }]]];
  var CloudHail$1 = CloudHail;

  var CloudLightning = ['svg', defaultAttributes, [['path', {
    d: 'M17.5 17a4.5 4.5 0 100-9h-1.8a7 7 0 10-10.3 8'
  }], ['path', {
    d: 'M12 12l-3 5h5l-3 5'
  }]]];
  var CloudLightning$1 = CloudLightning;

  var CloudMoon = ['svg', defaultAttributes, [['path', {
    d: 'M13.63 22A3.3 3.3 0 0017 18.79a3.3 3.3 0 00-3.38-3.22h-1.34A5.23 5.23 0 007.25 12 5.13 5.13 0 002 17c0 2.76 2.35 5 5.25 5h6.38z'
  }], ['path', {
    d: 'M9.95 9a6.13 6.13 0 015.5-5.18 4.77 4.77 0 006.67 6.67A6.13 6.13 0 0119.46 15'
  }]]];
  var CloudMoon$1 = CloudMoon;

  var CloudOff = ['svg', defaultAttributes, [['path', {
    d: 'M2 2l20 20'
  }], ['path', {
    d: 'M10 5a7 7 0 015.7 5h1.8a4.5 4.5 0 014 6.5'
  }], ['path', {
    d: 'M18.8 18.8c-.4.2-.8.2-1.3.2H9A7 7 0 015.8 5.8'
  }]]];
  var CloudOff$1 = CloudOff;

  var CloudRainWind = ['svg', defaultAttributes, [['path', {
    d: 'M20 16.2A4.5 4.5 0 0017.5 8h-1.8A7 7 0 104 14.9'
  }], ['path', {
    d: 'M9.2 22l3-7'
  }], ['path', {
    d: 'M9 13l-3 7'
  }], ['path', {
    d: 'M17 13l-3 7'
  }]]];
  var CloudRainWind$1 = CloudRainWind;

  var CloudRain = ['svg', defaultAttributes, [['path', {
    d: 'M20 16.2A4.5 4.5 0 0017.5 8h-1.8A7 7 0 104 14.9'
  }], ['path', {
    d: 'M16 14v6'
  }], ['path', {
    d: 'M8 14v6'
  }], ['path', {
    d: 'M12 16v6'
  }]]];
  var CloudRain$1 = CloudRain;

  var CloudSnow = ['svg', defaultAttributes, [['path', {
    d: 'M20 16.2A4.5 4.5 0 0017.5 8h-1.8A7 7 0 104 14.9'
  }], ['path', {
    d: 'M8 15h0'
  }], ['path', {
    d: 'M8 19h0'
  }], ['path', {
    d: 'M12 17h0'
  }], ['path', {
    d: 'M12 21h0'
  }], ['path', {
    d: 'M16 15h0'
  }], ['path', {
    d: 'M16 19h0'
  }]]];
  var CloudSnow$1 = CloudSnow;

  var CloudSun = ['svg', defaultAttributes, [['path', {
    d: 'M12 2v2'
  }], ['path', {
    d: 'M5.22 5.22l1.42 1.42'
  }], ['path', {
    d: 'M20 12h2'
  }], ['path', {
    d: 'M15.97 12.5A4 4 0 009.5 8.88'
  }], ['path', {
    d: 'M13.63 22A3.3 3.3 0 0017 18.79a3.3 3.3 0 00-3.38-3.22h-1.34A5.23 5.23 0 007.25 12 5.13 5.13 0 002 17c0 2.76 2.35 5 5.25 5h6.38z'
  }], ['path', {
    d: 'M17.36 6.64l1.42-1.42'
  }]]];
  var CloudSun$1 = CloudSun;

  var Cloud = ['svg', defaultAttributes, [['path', {
    d: 'M17.5 19a4.5 4.5 0 100-9h-1.8A7 7 0 109 19h8.5z'
  }]]];
  var Cloud$1 = Cloud;

  var Cloudy = ['svg', defaultAttributes, [['path', {
    d: 'M17.5 21a4.5 4.5 0 100-9h-1.8A7 7 0 109 21h8.5z'
  }], ['path', {
    d: 'M22 10c0-1.5-1.5-3-3.5-3H17c-.7-2.3-2.9-4-5.4-4-2.7 0-5 2-5.5 4.5'
  }]]];
  var Cloudy$1 = Cloudy;

  var Clover = ['svg', defaultAttributes, [['path', {
    d: 'M16.2 3.8a2.7 2.7 0 00-3.81 0l-.4.38-.4-.4a2.7 2.7 0 00-3.82 0C6.73 4.85 6.67 6.64 8 8l4 4 4-4c1.33-1.36 1.27-3.15.2-4.2z'
  }], ['path', {
    d: 'M8 8c-1.36-1.33-3.15-1.27-4.2-.2a2.7 2.7 0 000 3.81l.38.4-.4.4a2.7 2.7 0 000 3.82C4.85 17.27 6.64 17.33 8 16'
  }], ['path', {
    d: 'M16 16c1.36 1.33 3.15 1.27 4.2.2a2.7 2.7 0 000-3.81l-.38-.4.4-.4a2.7 2.7 0 000-3.82C19.15 6.73 17.36 6.67 16 8'
  }], ['path', {
    d: 'M7.8 20.2a2.7 2.7 0 003.81 0l.4-.38.4.4a2.7 2.7 0 003.82 0c1.06-1.06 1.12-2.85-.21-4.21l-4-4-4 4c-1.33 1.36-1.27 3.15-.2 4.2z'
  }], ['path', {
    d: 'M7 17l-5 5'
  }]]];
  var Clover$1 = Clover;

  var Code2 = ['svg', defaultAttributes, [['path', {
    d: 'M18 16l4-4-4-4'
  }], ['path', {
    d: 'M6 8l-4 4 4 4'
  }], ['path', {
    d: 'M14.5 4l-5 16'
  }]]];
  var Code2$1 = Code2;

  var Code = ['svg', defaultAttributes, [['polyline', {
    points: '16 18 22 12 16 6'
  }], ['polyline', {
    points: '8 6 2 12 8 18'
  }]]];
  var Code$1 = Code;

  var Codepen = ['svg', defaultAttributes, [['polygon', {
    points: '12 2 22 8.5 22 15.5 12 22 2 15.5 2 8.5 12 2'
  }], ['line', {
    x1: '12',
    y1: '22',
    x2: '12',
    y2: '15.5'
  }], ['polyline', {
    points: '22 8.5 12 15.5 2 8.5'
  }], ['polyline', {
    points: '2 15.5 12 8.5 22 15.5'
  }], ['line', {
    x1: '12',
    y1: '2',
    x2: '12',
    y2: '8.5'
  }]]];
  var Codepen$1 = Codepen;

  var Codesandbox = ['svg', defaultAttributes, [['path', {
    d: 'M21 16V8a2 2 0 00-1-1.73l-7-4a2 2 0 00-2 0l-7 4A2 2 0 003 8v8a2 2 0 001 1.73l7 4a2 2 0 002 0l7-4A2 2 0 0021 16z'
  }], ['polyline', {
    points: '7.5 4.21 12 6.81 16.5 4.21'
  }], ['polyline', {
    points: '7.5 19.79 7.5 14.6 3 12'
  }], ['polyline', {
    points: '21 12 16.5 14.6 16.5 19.79'
  }], ['polyline', {
    points: '3.27 6.96 12 12.01 20.73 6.96'
  }], ['line', {
    x1: '12',
    y1: '22.08',
    x2: '12',
    y2: '12'
  }]]];
  var Codesandbox$1 = Codesandbox;

  var Coffee = ['svg', defaultAttributes, [['path', {
    d: 'M18 8h1a4 4 0 010 8h-1'
  }], ['path', {
    d: 'M2 8h16v9a4 4 0 01-4 4H6a4 4 0 01-4-4V8z'
  }], ['line', {
    x1: '6',
    y1: '1',
    x2: '6',
    y2: '4'
  }], ['line', {
    x1: '10',
    y1: '1',
    x2: '10',
    y2: '4'
  }], ['line', {
    x1: '14',
    y1: '1',
    x2: '14',
    y2: '4'
  }]]];
  var Coffee$1 = Coffee;

  var Coins = ['svg', defaultAttributes, [['circle', {
    cx: '8',
    cy: '8',
    r: '7'
  }], ['path', {
    d: 'M19.5 9.94a7 7 0 11-9.56 9.56'
  }], ['path', {
    d: 'M7 6h1v4'
  }], ['path', {
    d: 'M17.3 14.3l.7.7-2.8 2.8'
  }]]];
  var Coins$1 = Coins;

  var Columns = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '12',
    y1: '3',
    x2: '12',
    y2: '21'
  }]]];
  var Columns$1 = Columns;

  var Command = ['svg', defaultAttributes, [['path', {
    d: 'M18 3a3 3 0 00-3 3v12a3 3 0 003 3 3 3 0 003-3 3 3 0 00-3-3H6a3 3 0 00-3 3 3 3 0 003 3 3 3 0 003-3V6a3 3 0 00-3-3 3 3 0 00-3 3 3 3 0 003 3h12a3 3 0 003-3 3 3 0 00-3-3z'
  }]]];
  var Command$1 = Command;

  var Compass = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polygon', {
    points: '16.24 7.76 14.12 14.12 7.76 16.24 9.88 9.88 16.24 7.76'
  }]]];
  var Compass$1 = Compass;

  var Contact = ['svg', defaultAttributes, [['path', {
    d: 'M19 22H5c-1.1 0-2-.9-2-2V6c0-1.1.9-2 2-2h14c1.1 0 2 .9 2 2v14c0 1.1-.9 2-2 2z'
  }], ['line', {
    x1: '16',
    y1: '2',
    x2: '16',
    y2: '4'
  }], ['line', {
    x1: '8',
    y1: '2',
    x2: '8',
    y2: '4'
  }], ['circle', {
    cx: '12',
    cy: '11',
    r: '3'
  }], ['path', {
    d: 'M17 18.5c-1.4-1-3.1-1.5-5-1.5s-3.6.6-5 1.5'
  }]]];
  var Contact$1 = Contact;

  var Contrast = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['path', {
    d: 'M12 18a6 6 0 000-12v12z'
  }]]];
  var Contrast$1 = Contrast;

  var Cookie = ['svg', defaultAttributes, [['path', {
    d: 'M12 2a10 10 0 1010 10 4 4 0 01-5-5 4 4 0 01-5-5'
  }], ['path', {
    d: 'M8.5 8.5v.01'
  }], ['path', {
    d: 'M16 15.5v.01'
  }], ['path', {
    d: 'M12 12v.01'
  }], ['path', {
    d: 'M11 17v.01'
  }], ['path', {
    d: 'M7 14v.01'
  }]]];
  var Cookie$1 = Cookie;

  var Copy = ['svg', defaultAttributes, [['rect', {
    x: '9',
    y: '9',
    width: '13',
    height: '13',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M5 15H4a2 2 0 01-2-2V4a2 2 0 012-2h9a2 2 0 012 2v1'
  }]]];
  var Copy$1 = Copy;

  var Copyleft = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['path', {
    d: 'M9 9.35a4 4 0 110 5.3'
  }]]];
  var Copyleft$1 = Copyleft;

  var Copyright = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['path', {
    d: 'M15 9.354a4 4 0 100 5.292'
  }]]];
  var Copyright$1 = Copyright;

  var CornerDownLeft = ['svg', defaultAttributes, [['polyline', {
    points: '9 10 4 15 9 20'
  }], ['path', {
    d: 'M20 4v7a4 4 0 01-4 4H4'
  }]]];
  var CornerDownLeft$1 = CornerDownLeft;

  var CornerDownRight = ['svg', defaultAttributes, [['polyline', {
    points: '15 10 20 15 15 20'
  }], ['path', {
    d: 'M4 4v7a4 4 0 004 4h12'
  }]]];
  var CornerDownRight$1 = CornerDownRight;

  var CornerLeftDown = ['svg', defaultAttributes, [['polyline', {
    points: '14 15 9 20 4 15'
  }], ['path', {
    d: 'M20 4h-7a4 4 0 00-4 4v12'
  }]]];
  var CornerLeftDown$1 = CornerLeftDown;

  var CornerLeftUp = ['svg', defaultAttributes, [['polyline', {
    points: '14 9 9 4 4 9'
  }], ['path', {
    d: 'M20 20h-7a4 4 0 01-4-4V4'
  }]]];
  var CornerLeftUp$1 = CornerLeftUp;

  var CornerRightDown = ['svg', defaultAttributes, [['polyline', {
    points: '10 15 15 20 20 15'
  }], ['path', {
    d: 'M4 4h7a4 4 0 014 4v12'
  }]]];
  var CornerRightDown$1 = CornerRightDown;

  var CornerRightUp = ['svg', defaultAttributes, [['polyline', {
    points: '10 9 15 4 20 9'
  }], ['path', {
    d: 'M4 20h7a4 4 0 004-4V4'
  }]]];
  var CornerRightUp$1 = CornerRightUp;

  var CornerUpLeft = ['svg', defaultAttributes, [['polyline', {
    points: '9 14 4 9 9 4'
  }], ['path', {
    d: 'M20 20v-7a4 4 0 00-4-4H4'
  }]]];
  var CornerUpLeft$1 = CornerUpLeft;

  var CornerUpRight = ['svg', defaultAttributes, [['polyline', {
    points: '15 14 20 9 15 4'
  }], ['path', {
    d: 'M4 20v-7a4 4 0 014-4h12'
  }]]];
  var CornerUpRight$1 = CornerUpRight;

  var Cpu = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '4',
    width: '16',
    height: '16',
    rx: '2',
    ry: '2'
  }], ['rect', {
    x: '9',
    y: '9',
    width: '6',
    height: '6'
  }], ['line', {
    x1: '9',
    y1: '1',
    x2: '9',
    y2: '4'
  }], ['line', {
    x1: '15',
    y1: '1',
    x2: '15',
    y2: '4'
  }], ['line', {
    x1: '9',
    y1: '20',
    x2: '9',
    y2: '23'
  }], ['line', {
    x1: '15',
    y1: '20',
    x2: '15',
    y2: '23'
  }], ['line', {
    x1: '20',
    y1: '9',
    x2: '23',
    y2: '9'
  }], ['line', {
    x1: '20',
    y1: '14',
    x2: '23',
    y2: '14'
  }], ['line', {
    x1: '1',
    y1: '9',
    x2: '4',
    y2: '9'
  }], ['line', {
    x1: '1',
    y1: '14',
    x2: '4',
    y2: '14'
  }]]];
  var Cpu$1 = Cpu;

  var CreditCard = ['svg', defaultAttributes, [['rect', {
    x: '1',
    y: '4',
    width: '22',
    height: '16',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '1',
    y1: '10',
    x2: '23',
    y2: '10'
  }]]];
  var CreditCard$1 = CreditCard;

  var Crop = ['svg', defaultAttributes, [['path', {
    d: 'M6.13 1L6 16a2 2 0 002 2h15'
  }], ['path', {
    d: 'M1 6.13L16 6a2 2 0 012 2v15'
  }]]];
  var Crop$1 = Crop;

  var Cross = ['svg', defaultAttributes, [['path', {
    d: 'M11 2a2 2 0 00-2 2v5H4a2 2 0 00-2 2v2c0 1.1.9 2 2 2h5v5c0 1.1.9 2 2 2h2a2 2 0 002-2v-5h5a2 2 0 002-2v-2a2 2 0 00-2-2h-5V4a2 2 0 00-2-2h-2z'
  }]]];
  var Cross$1 = Cross;

  var Crosshair = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '22',
    y1: '12',
    x2: '18',
    y2: '12'
  }], ['line', {
    x1: '6',
    y1: '12',
    x2: '2',
    y2: '12'
  }], ['line', {
    x1: '12',
    y1: '6',
    x2: '12',
    y2: '2'
  }], ['line', {
    x1: '12',
    y1: '22',
    x2: '12',
    y2: '18'
  }]]];
  var Crosshair$1 = Crosshair;

  var Crown = ['svg', defaultAttributes, [['path', {
    d: 'M2 4l3 12h14l3-12-6 7-4-7-4 7-6-7zm3 16h14'
  }]]];
  var Crown$1 = Crown;

  var Currency = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '8'
  }], ['line', {
    x1: '3',
    y1: '3',
    x2: '6',
    y2: '6'
  }], ['line', {
    x1: '21',
    y1: '3',
    x2: '18',
    y2: '6'
  }], ['line', {
    x1: '3',
    y1: '21',
    x2: '6',
    y2: '18'
  }], ['line', {
    x1: '21',
    y1: '21',
    x2: '18',
    y2: '18'
  }]]];
  var Currency$1 = Currency;

  var Database = ['svg', defaultAttributes, [['ellipse', {
    cx: '12',
    cy: '5',
    rx: '9',
    ry: '3'
  }], ['path', {
    d: 'M21 12c0 1.66-4 3-9 3s-9-1.34-9-3'
  }], ['path', {
    d: 'M3 5v14c0 1.66 4 3 9 3s9-1.34 9-3V5'
  }]]];
  var Database$1 = Database;

  var Delete = ['svg', defaultAttributes, [['path', {
    d: 'M21 4H8l-7 8 7 8h13a2 2 0 002-2V6a2 2 0 00-2-2z'
  }], ['line', {
    x1: '18',
    y1: '9',
    x2: '12',
    y2: '15'
  }], ['line', {
    x1: '12',
    y1: '9',
    x2: '18',
    y2: '15'
  }]]];
  var Delete$1 = Delete;

  var Dice1 = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M12 12h.01'
  }]]];
  var Dice1$1 = Dice1;

  var Dice2 = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M15 9h.01'
  }], ['path', {
    d: 'M9 15h.01'
  }]]];
  var Dice2$1 = Dice2;

  var Dice3 = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M16 8h.01'
  }], ['path', {
    d: 'M12 12h.01'
  }], ['path', {
    d: 'M8 16h.01'
  }]]];
  var Dice3$1 = Dice3;

  var Dice4 = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M16 8h.01'
  }], ['path', {
    d: 'M8 8h.01'
  }], ['path', {
    d: 'M8 16h.01'
  }], ['path', {
    d: 'M16 16h.01'
  }]]];
  var Dice4$1 = Dice4;

  var Dice5 = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M16 8h.01'
  }], ['path', {
    d: 'M8 8h.01'
  }], ['path', {
    d: 'M8 16h.01'
  }], ['path', {
    d: 'M16 16h.01'
  }], ['path', {
    d: 'M12 12h.01'
  }]]];
  var Dice5$1 = Dice5;

  var Dice6 = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M16 8h.01'
  }], ['path', {
    d: 'M16 12h.01'
  }], ['path', {
    d: 'M16 16h.01'
  }], ['path', {
    d: 'M8 8h.01'
  }], ['path', {
    d: 'M8 12h.01'
  }], ['path', {
    d: 'M8 16h.01'
  }]]];
  var Dice6$1 = Dice6;

  var Disc = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '3'
  }]]];
  var Disc$1 = Disc;

  var DivideCircle = ['svg', defaultAttributes, [['line', {
    x1: '8',
    y1: '12',
    x2: '16',
    y2: '12'
  }], ['line', {
    x1: '12',
    y1: '16',
    x2: '12',
    y2: '16'
  }], ['line', {
    x1: '12',
    y1: '8',
    x2: '12',
    y2: '8'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }]]];
  var DivideCircle$1 = DivideCircle;

  var DivideSquare = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '8',
    y1: '12',
    x2: '16',
    y2: '12'
  }], ['line', {
    x1: '12',
    y1: '16',
    x2: '12',
    y2: '16'
  }], ['line', {
    x1: '12',
    y1: '8',
    x2: '12',
    y2: '8'
  }]]];
  var DivideSquare$1 = DivideSquare;

  var Divide = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '6',
    r: '1'
  }], ['line', {
    x1: '5',
    y1: '12',
    x2: '19',
    y2: '12'
  }], ['circle', {
    cx: '12',
    cy: '18',
    r: '1'
  }]]];
  var Divide$1 = Divide;

  var DollarSign = ['svg', defaultAttributes, [['line', {
    x1: '12',
    y1: '1',
    x2: '12',
    y2: '23'
  }], ['path', {
    d: 'M17 5H9.5a3.5 3.5 0 000 7h5a3.5 3.5 0 010 7H6'
  }]]];
  var DollarSign$1 = DollarSign;

  var DownloadCloud = ['svg', defaultAttributes, [['path', {
    d: 'M20 16.2A4.5 4.5 0 0017.5 8h-1.8A7 7 0 104 14.9'
  }], ['path', {
    d: 'M12 12v9'
  }], ['path', {
    d: 'M8 17l4 4 4-4'
  }]]];
  var DownloadCloud$1 = DownloadCloud;

  var Download = ['svg', defaultAttributes, [['path', {
    d: 'M21 15v4a2 2 0 01-2 2H5a2 2 0 01-2-2v-4'
  }], ['polyline', {
    points: '7 10 12 15 17 10'
  }], ['line', {
    x1: '12',
    y1: '15',
    x2: '12',
    y2: '3'
  }]]];
  var Download$1 = Download;

  var Dribbble = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['path', {
    d: 'M8.56 2.75c4.37 6.03 6.02 9.42 8.03 17.72m2.54-15.38c-3.72 4.35-8.94 5.66-16.88 5.85m19.5 1.9c-3.5-.93-6.63-.82-8.94 0-2.58.92-5.01 2.86-7.44 6.32'
  }]]];
  var Dribbble$1 = Dribbble;

  var Droplet = ['svg', defaultAttributes, [['path', {
    d: 'M12 22a7 7 0 007-7c0-2-1-3.9-3-5.5s-3.5-4-4-6.5c-.5 2.5-2 4.9-4 6.5C6 11.1 5 13 5 15a7 7 0 007 7z'
  }]]];
  var Droplet$1 = Droplet;

  var Droplets = ['svg', defaultAttributes, [['path', {
    d: 'M7 16.3c2.2 0 4-1.83 4-4.05 0-1.16-.57-2.26-1.71-3.19S7.29 6.75 7 5.3c-.29 1.45-1.14 2.84-2.29 3.76S3 11.1 3 12.25c0 2.22 1.8 4.05 4 4.05z'
  }], ['path', {
    d: 'M12.56 6.6A10.97 10.97 0 0014 3.02c.5 2.5 2 4.9 4 6.5s3 3.5 3 5.5a6.98 6.98 0 01-11.91 4.97'
  }]]];
  var Droplets$1 = Droplets;

  var Drumstick = ['svg', defaultAttributes, [['path', {
    d: 'M15.45 15.4c-2.13.65-4.3.32-5.7-1.1-2.29-2.27-1.76-6.5 1.17-9.42 2.93-2.93 7.15-3.46 9.43-1.18 1.41 1.41 1.74 3.57 1.1 5.71-1.4-.51-3.26-.02-4.64 1.36-1.38 1.38-1.87 3.23-1.36 4.63z'
  }], ['path', {
    d: 'M11.25 15.6l-2.16 2.16a2.5 2.5 0 11-4.56 1.73 2.49 2.49 0 01-1.41-4.24 2.5 2.5 0 013.14-.32l2.16-2.16'
  }]]];
  var Drumstick$1 = Drumstick;

  var Edit2 = ['svg', defaultAttributes, [['path', {
    d: 'M17 3a2.828 2.828 0 114 4L7.5 20.5 2 22l1.5-5.5L17 3z'
  }]]];
  var Edit2$1 = Edit2;

  var Edit3 = ['svg', defaultAttributes, [['path', {
    d: 'M12 20h9'
  }], ['path', {
    d: 'M16.5 3.5a2.121 2.121 0 013 3L7 19l-4 1 1-4L16.5 3.5z'
  }]]];
  var Edit3$1 = Edit3;

  var Edit = ['svg', defaultAttributes, [['path', {
    d: 'M11 4H4a2 2 0 00-2 2v14a2 2 0 002 2h14a2 2 0 002-2v-7'
  }], ['path', {
    d: 'M18.5 2.5a2.121 2.121 0 013 3L12 15l-4 1 1-4 9.5-9.5z'
  }]]];
  var Edit$1 = Edit;

  var Egg = ['svg', defaultAttributes, [['path', {
    d: 'M12 22c6.23-.05 7.87-5.57 7.5-10-.36-4.34-3.95-9.96-7.5-10-3.55.04-7.14 5.66-7.5 10-.37 4.43 1.27 9.95 7.5 10z'
  }]]];
  var Egg$1 = Egg;

  var EqualNot = ['svg', defaultAttributes, [['line', {
    x1: '5',
    y1: '9',
    x2: '19',
    y2: '9'
  }], ['line', {
    x1: '5',
    y1: '15',
    x2: '19',
    y2: '15'
  }], ['line', {
    x1: '19',
    y1: '5',
    x2: '5',
    y2: '19'
  }]]];
  var EqualNot$1 = EqualNot;

  var Equal = ['svg', defaultAttributes, [['line', {
    x1: '5',
    y1: '9',
    x2: '19',
    y2: '9'
  }], ['line', {
    x1: '5',
    y1: '15',
    x2: '19',
    y2: '15'
  }]]];
  var Equal$1 = Equal;

  var Euro = ['svg', defaultAttributes, [['path', {
    d: 'M4 10h12'
  }], ['path', {
    d: 'M4 14h9'
  }], ['path', {
    d: 'M19 6a7.7 7.7 0 00-5.2-2A7.9 7.9 0 006 12c0 4.4 3.5 8 7.8 8 2 0 3.8-.8 5.2-2'
  }]]];
  var Euro$1 = Euro;

  var Expand = ['svg', defaultAttributes, [['path', {
    d: 'M21 21l-6-6m6 6v-4.8m0 4.8h-4.8'
  }], ['path', {
    d: 'M3 16.2V21m0 0h4.8M3 21l6-6'
  }], ['path', {
    d: 'M21 7.8V3m0 0h-4.8M21 3l-6 6'
  }], ['path', {
    d: 'M3 7.8V3m0 0h4.8M3 3l6 6'
  }]]];
  var Expand$1 = Expand;

  var ExternalLink = ['svg', defaultAttributes, [['path', {
    d: 'M18 13v6a2 2 0 01-2 2H5a2 2 0 01-2-2V8a2 2 0 012-2h6'
  }], ['polyline', {
    points: '15 3 21 3 21 9'
  }], ['line', {
    x1: '10',
    y1: '14',
    x2: '21',
    y2: '3'
  }]]];
  var ExternalLink$1 = ExternalLink;

  var EyeOff = ['svg', defaultAttributes, [['path', {
    d: 'M17.94 17.94A10.07 10.07 0 0112 20c-7 0-11-8-11-8a18.45 18.45 0 015.06-5.94M9.9 4.24A9.12 9.12 0 0112 4c7 0 11 8 11 8a18.5 18.5 0 01-2.16 3.19m-6.72-1.07a3 3 0 11-4.24-4.24'
  }], ['line', {
    x1: '1',
    y1: '1',
    x2: '23',
    y2: '23'
  }]]];
  var EyeOff$1 = EyeOff;

  var Eye = ['svg', defaultAttributes, [['path', {
    d: 'M1 12s4-8 11-8 11 8 11 8-4 8-11 8-11-8-11-8z'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '3'
  }]]];
  var Eye$1 = Eye;

  var Facebook = ['svg', defaultAttributes, [['path', {
    d: 'M18 2h-3a5 5 0 00-5 5v3H7v4h3v8h4v-8h3l1-4h-4V7a1 1 0 011-1h3z'
  }]]];
  var Facebook$1 = Facebook;

  var FastForward = ['svg', defaultAttributes, [['polygon', {
    points: '13 19 22 12 13 5 13 19'
  }], ['polygon', {
    points: '2 19 11 12 2 5 2 19'
  }]]];
  var FastForward$1 = FastForward;

  var Feather = ['svg', defaultAttributes, [['path', {
    d: 'M20.24 12.24a6 6 0 00-8.49-8.49L5 10.5V19h8.5z'
  }], ['line', {
    x1: '16',
    y1: '8',
    x2: '2',
    y2: '22'
  }], ['line', {
    x1: '17.5',
    y1: '15',
    x2: '9',
    y2: '15'
  }]]];
  var Feather$1 = Feather;

  var Figma = ['svg', defaultAttributes, [['path', {
    d: 'M5 5.5A3.5 3.5 0 018.5 2H12v7H8.5A3.5 3.5 0 015 5.5z'
  }], ['path', {
    d: 'M12 2h3.5a3.5 3.5 0 110 7H12V2z'
  }], ['path', {
    d: 'M12 12.5a3.5 3.5 0 117 0 3.5 3.5 0 11-7 0z'
  }], ['path', {
    d: 'M5 19.5A3.5 3.5 0 018.5 16H12v3.5a3.5 3.5 0 11-7 0z'
  }], ['path', {
    d: 'M5 12.5A3.5 3.5 0 018.5 9H12v7H8.5A3.5 3.5 0 015 12.5z'
  }]]];
  var Figma$1 = Figma;

  var FileCheck2 = ['svg', defaultAttributes, [['path', {
    d: 'M4 22h14a2 2 0 002-2V7.5L14.5 2H6a2 2 0 00-2 2v4'
  }], ['path', {
    d: 'M14 2v6h6'
  }], ['path', {
    d: 'M3 15l2 2 4-4'
  }]]];
  var FileCheck2$1 = FileCheck2;

  var FileCheck = ['svg', defaultAttributes, [['path', {
    d: 'M14.5 2H6a2 2 0 00-2 2v16a2 2 0 002 2h12a2 2 0 002-2V7.5L14.5 2z'
  }], ['polyline', {
    points: '14 2 14 8 20 8'
  }], ['path', {
    d: 'M9 15l2 2 4-4'
  }]]];
  var FileCheck$1 = FileCheck;

  var FileCode = ['svg', defaultAttributes, [['path', {
    d: 'M4 22h14a2 2 0 002-2V7.5L14.5 2H6a2 2 0 00-2 2v4'
  }], ['path', {
    d: 'M14 2v6h6'
  }], ['path', {
    d: 'M9 18l3-3-3-3'
  }], ['path', {
    d: 'M5 12l-3 3 3 3'
  }]]];
  var FileCode$1 = FileCode;

  var FileDigit = ['svg', defaultAttributes, [['path', {
    d: 'M4 22h14a2 2 0 002-2V7.5L14.5 2H6a2 2 0 00-2 2v4'
  }], ['path', {
    d: 'M14 2v6h6'
  }], ['path', {
    d: 'M10 12h2v6'
  }], ['rect', {
    x: '2',
    y: '12',
    width: '4',
    height: '6'
  }], ['path', {
    d: 'M10 18h4'
  }]]];
  var FileDigit$1 = FileDigit;

  var FileInput = ['svg', defaultAttributes, [['path', {
    d: 'M4 22h14a2 2 0 002-2V7.5L14.5 2H6a2 2 0 00-2 2v4'
  }], ['path', {
    d: 'M14 2v6h6'
  }], ['path', {
    d: 'M2 15h10'
  }], ['path', {
    d: 'M9 18l3-3-3-3'
  }]]];
  var FileInput$1 = FileInput;

  var FileMinus2 = ['svg', defaultAttributes, [['path', {
    d: 'M4 22h14a2 2 0 002-2V7.5L14.5 2H6a2 2 0 00-2 2v4'
  }], ['path', {
    d: 'M14 2v6h6'
  }], ['path', {
    d: 'M3 15h6'
  }]]];
  var FileMinus2$1 = FileMinus2;

  var FileMinus = ['svg', defaultAttributes, [['path', {
    d: 'M14.5 2H6a2 2 0 00-2 2v16a2 2 0 002 2h12a2 2 0 002-2V7.5L14.5 2z'
  }], ['polyline', {
    points: '14 2 14 8 20 8'
  }], ['line', {
    x1: '9',
    y1: '15',
    x2: '15',
    y2: '15'
  }]]];
  var FileMinus$1 = FileMinus;

  var FileOutput = ['svg', defaultAttributes, [['path', {
    d: 'M4 22h14a2 2 0 002-2V7.5L14.5 2H6a2 2 0 00-2 2v4'
  }], ['path', {
    d: 'M14 2v6h6'
  }], ['path', {
    d: 'M2 15h10'
  }], ['path', {
    d: 'M5 12l-3 3 3 3'
  }]]];
  var FileOutput$1 = FileOutput;

  var FilePlus2 = ['svg', defaultAttributes, [['path', {
    d: 'M4 22h14a2 2 0 002-2V7.5L14.5 2H6a2 2 0 00-2 2v4'
  }], ['path', {
    d: 'M14 2v6h6'
  }], ['path', {
    d: 'M3 15h6'
  }], ['path', {
    d: 'M6 12v6'
  }]]];
  var FilePlus2$1 = FilePlus2;

  var FilePlus = ['svg', defaultAttributes, [['path', {
    d: 'M14.5 2H6a2 2 0 00-2 2v16a2 2 0 002 2h12a2 2 0 002-2V7.5L14.5 2z'
  }], ['polyline', {
    points: '14 2 14 8 20 8'
  }], ['line', {
    x1: '12',
    y1: '18',
    x2: '12',
    y2: '12'
  }], ['line', {
    x1: '9',
    y1: '15',
    x2: '15',
    y2: '15'
  }]]];
  var FilePlus$1 = FilePlus;

  var FileSearch = ['svg', defaultAttributes, [['path', {
    d: 'M4 22h14a2 2 0 002-2V7.5L14.5 2H6a2 2 0 00-2 2v3'
  }], ['path', {
    d: 'M14 2v6h6'
  }], ['path', {
    d: 'M5 17a3 3 0 100-6 3 3 0 000 6z'
  }], ['path', {
    d: 'M9 18l-1.5-1.5'
  }]]];
  var FileSearch$1 = FileSearch;

  var FileText = ['svg', defaultAttributes, [['path', {
    d: 'M14.5 2H6a2 2 0 00-2 2v16a2 2 0 002 2h12a2 2 0 002-2V7.5L14.5 2z'
  }], ['polyline', {
    points: '14 2 14 8 20 8'
  }], ['line', {
    x1: '16',
    y1: '13',
    x2: '8',
    y2: '13'
  }], ['line', {
    x1: '16',
    y1: '17',
    x2: '8',
    y2: '17'
  }], ['line', {
    x1: '10',
    y1: '9',
    x2: '8',
    y2: '9'
  }]]];
  var FileText$1 = FileText;

  var FileX2 = ['svg', defaultAttributes, [['path', {
    d: 'M4 22h14a2 2 0 002-2V7.5L14.5 2H6a2 2 0 00-2 2v4'
  }], ['path', {
    d: 'M14 2v6h6'
  }], ['path', {
    d: 'M3 12.5l5 5'
  }], ['path', {
    d: 'M8 12.5l-5 5'
  }]]];
  var FileX2$1 = FileX2;

  var FileX = ['svg', defaultAttributes, [['path', {
    d: 'M14.5 2H6a2 2 0 00-2 2v16a2 2 0 002 2h12a2 2 0 002-2V7.5L14.5 2z'
  }], ['polyline', {
    points: '14 2 14 8 20 8'
  }], ['line', {
    x1: '9.5',
    y1: '12.5',
    x2: '14.5',
    y2: '17.5'
  }], ['line', {
    x1: '14.5',
    y1: '12.5',
    x2: '9.5',
    y2: '17.5'
  }]]];
  var FileX$1 = FileX;

  var File = ['svg', defaultAttributes, [['path', {
    d: 'M14.5 2H6a2 2 0 00-2 2v16a2 2 0 002 2h12a2 2 0 002-2V7.5L14.5 2z'
  }], ['polyline', {
    points: '14 2 14 8 20 8'
  }]]];
  var File$1 = File;

  var Files = ['svg', defaultAttributes, [['path', {
    d: 'M15.5 2H8.6c-.4 0-.8.2-1.1.5-.3.3-.5.7-.5 1.1v12.8c0 .4.2.8.5 1.1.3.3.7.5 1.1.5h9.8c.4 0 .8-.2 1.1-.5.3-.3.5-.7.5-1.1V6.5L15.5 2z'
  }], ['path', {
    d: 'M3 7.6v12.8c0 .4.2.8.5 1.1.3.3.7.5 1.1.5h9.8'
  }], ['path', {
    d: 'M15 2v5h5'
  }]]];
  var Files$1 = Files;

  var Film = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '2',
    width: '20',
    height: '20',
    rx: '2.18',
    ry: '2.18'
  }], ['line', {
    x1: '7',
    y1: '2',
    x2: '7',
    y2: '22'
  }], ['line', {
    x1: '17',
    y1: '2',
    x2: '17',
    y2: '22'
  }], ['line', {
    x1: '2',
    y1: '12',
    x2: '22',
    y2: '12'
  }], ['line', {
    x1: '2',
    y1: '7',
    x2: '7',
    y2: '7'
  }], ['line', {
    x1: '2',
    y1: '17',
    x2: '7',
    y2: '17'
  }], ['line', {
    x1: '17',
    y1: '17',
    x2: '22',
    y2: '17'
  }], ['line', {
    x1: '17',
    y1: '7',
    x2: '22',
    y2: '7'
  }]]];
  var Film$1 = Film;

  var Filter = ['svg', defaultAttributes, [['polygon', {
    points: '22 3 2 3 10 12.46 10 19 14 21 14 12.46 22 3'
  }]]];
  var Filter$1 = Filter;

  var FlagTriangleLeft = ['svg', defaultAttributes, [['path', {
    d: 'M17 22V2L7 7l10 5'
  }]]];
  var FlagTriangleLeft$1 = FlagTriangleLeft;

  var FlagTriangleRight = ['svg', defaultAttributes, [['path', {
    d: 'M7 22V2l10 5-10 5'
  }]]];
  var FlagTriangleRight$1 = FlagTriangleRight;

  var Flag = ['svg', defaultAttributes, [['path', {
    d: 'M4 15s1-1 4-1 5 2 8 2 4-1 4-1V3s-1 1-4 1-5-2-8-2-4 1-4 1z'
  }], ['line', {
    x1: '4',
    y1: '22',
    x2: '4',
    y2: '15'
  }]]];
  var Flag$1 = Flag;

  var Flame = ['svg', defaultAttributes, [['path', {
    d: 'M8.5 14.5A2.5 2.5 0 0011 12c0-1.38-.5-2-1-3-1.072-2.143-.224-4.054 2-6 .5 2.5 2 4.9 4 6.5 2 1.6 3 3.5 3 5.5a7 7 0 11-14 0c0-1.153.433-2.294 1-3a2.5 2.5 0 002.5 2.5z'
  }]]];
  var Flame$1 = Flame;

  var FlashlightOff = ['svg', defaultAttributes, [['path', {
    d: 'M16 16v4a2 2 0 01-2 2h-4a2 2 0 01-2-2V10c0-2-2-2-2-4'
  }], ['path', {
    d: 'M7 2h11v4c0 2-2 2-2 4v1'
  }], ['line', {
    x1: '11',
    y1: '6',
    x2: '18',
    y2: '6'
  }], ['line', {
    x1: '2',
    y1: '2',
    x2: '22',
    y2: '22'
  }]]];
  var FlashlightOff$1 = FlashlightOff;

  var Flashlight = ['svg', defaultAttributes, [['path', {
    d: 'M18 6c0 2-2 2-2 4v10a2 2 0 01-2 2h-4a2 2 0 01-2-2V10c0-2-2-2-2-4V2h12z'
  }], ['line', {
    x1: '6',
    y1: '6',
    x2: '18',
    y2: '6'
  }], ['line', {
    x1: '12',
    y1: '12',
    x2: '12',
    y2: '12'
  }]]];
  var Flashlight$1 = Flashlight;

  var FlaskConical = ['svg', defaultAttributes, [['path', {
    d: 'M10 2v8L4.72 20.55a1 1 0 00.9 1.45h12.76a1 1 0 00.9-1.45L14 10V2'
  }], ['path', {
    d: 'M8.5 2h7'
  }], ['path', {
    d: 'M7 16h10'
  }]]];
  var FlaskConical$1 = FlaskConical;

  var FlaskRound = ['svg', defaultAttributes, [['path', {
    d: 'M10 2v7.31'
  }], ['path', {
    d: 'M14 9.3V1.99'
  }], ['path', {
    d: 'M8.5 2h7'
  }], ['path', {
    d: 'M14 9.3a6.5 6.5 0 11-4 0'
  }], ['path', {
    d: 'M5.58 16.5h12.85'
  }]]];
  var FlaskRound$1 = FlaskRound;

  var FolderMinus = ['svg', defaultAttributes, [['path', {
    d: 'M22 19a2 2 0 01-2 2H4a2 2 0 01-2-2V5a2 2 0 012-2h5l2 3h9a2 2 0 012 2z'
  }], ['line', {
    x1: '9',
    y1: '14',
    x2: '15',
    y2: '14'
  }]]];
  var FolderMinus$1 = FolderMinus;

  var FolderOpen = ['svg', defaultAttributes, [['path', {
    d: 'M6 17l2-5h14l-3 8a2 2 0 01-2 1H4a2 2 0 01-2-2V5a2 2 0 012-2h5l2 3h7a2 2 0 012 2v4'
  }]]];
  var FolderOpen$1 = FolderOpen;

  var FolderPlus = ['svg', defaultAttributes, [['path', {
    d: 'M22 19a2 2 0 01-2 2H4a2 2 0 01-2-2V5a2 2 0 012-2h5l2 3h9a2 2 0 012 2z'
  }], ['line', {
    x1: '12',
    y1: '11',
    x2: '12',
    y2: '17'
  }], ['line', {
    x1: '9',
    y1: '14',
    x2: '15',
    y2: '14'
  }]]];
  var FolderPlus$1 = FolderPlus;

  var Folder = ['svg', defaultAttributes, [['path', {
    d: 'M22 19a2 2 0 01-2 2H4a2 2 0 01-2-2V5a2 2 0 012-2h5l2 3h9a2 2 0 012 2z'
  }]]];
  var Folder$1 = Folder;

  var FormInput = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '6',
    width: '20',
    height: '12',
    rx: '2'
  }], ['path', {
    d: 'M12 12h.01'
  }], ['path', {
    d: 'M17 12h.01'
  }], ['path', {
    d: 'M7 12h.01'
  }]]];
  var FormInput$1 = FormInput;

  var Forward = ['svg', defaultAttributes, [['polyline', {
    points: '15 17 20 12 15 7'
  }], ['path', {
    d: 'M4 18v-2a4 4 0 014-4h12'
  }]]];
  var Forward$1 = Forward;

  var Framer = ['svg', defaultAttributes, [['path', {
    d: 'M5 16V9h14V2H5l14 14h-7m-7 0l7 7v-7m-7 0h7'
  }]]];
  var Framer$1 = Framer;

  var Frown = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['path', {
    d: 'M16 16s-1.5-2-4-2-4 2-4 2'
  }], ['line', {
    x1: '9',
    y1: '9',
    x2: '9.01',
    y2: '9'
  }], ['line', {
    x1: '15',
    y1: '9',
    x2: '15.01',
    y2: '9'
  }]]];
  var Frown$1 = Frown;

  var FunctionSquare = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M9 17c2 0 2.8-1 2.8-2.8V10c0-2 1-3.3 3.2-3'
  }], ['path', {
    d: 'M9 11.2h5.7'
  }]]];
  var FunctionSquare$1 = FunctionSquare;

  var Gamepad2 = ['svg', defaultAttributes, [['line', {
    x1: '6',
    y1: '11',
    x2: '10',
    y2: '11'
  }], ['line', {
    x1: '8',
    y1: '9',
    x2: '8',
    y2: '13'
  }], ['line', {
    x1: '15',
    y1: '12',
    x2: '15.01',
    y2: '12'
  }], ['line', {
    x1: '18',
    y1: '10',
    x2: '18.01',
    y2: '10'
  }], ['path', {
    d: 'M17.32 5H6.68a4 4 0 00-3.978 3.59c-.006.052-.01.101-.017.152C2.604 9.416 2 14.456 2 16a3 3 0 003 3c1 0 1.5-.5 2-1l1.414-1.414A2 2 0 019.828 16h4.344a2 2 0 011.414.586L17 18c.5.5 1 1 2 1a3 3 0 003-3c0-1.545-.604-6.584-.685-7.258-.007-.05-.011-.1-.017-.151A4 4 0 0017.32 5z'
  }]]];
  var Gamepad2$1 = Gamepad2;

  var Gamepad = ['svg', defaultAttributes, [['line', {
    x1: '6',
    y1: '12',
    x2: '10',
    y2: '12'
  }], ['line', {
    x1: '8',
    y1: '10',
    x2: '8',
    y2: '14'
  }], ['line', {
    x1: '15',
    y1: '13',
    x2: '15.01',
    y2: '13'
  }], ['line', {
    x1: '18',
    y1: '11',
    x2: '18.01',
    y2: '11'
  }], ['rect', {
    x: '2',
    y: '6',
    width: '20',
    height: '12',
    rx: '2'
  }]]];
  var Gamepad$1 = Gamepad;

  var Gauge = ['svg', defaultAttributes, [['path', {
    d: 'M12 15l3.5-3.5'
  }], ['path', {
    d: 'M20.3 18c.4-1 .7-2.2.7-3.4C21 9.8 17 6 12 6s-9 3.8-9 8.6c0 1.2.3 2.4.7 3.4'
  }]]];
  var Gauge$1 = Gauge;

  var Gavel = ['svg', defaultAttributes, [['path', {
    d: 'M14 13l-7.5 7.5c-.83.83-2.17.83-3 0 0 0 0 0 0 0a2.12 2.12 0 010-3L11 10'
  }], ['path', {
    d: 'M16 16l6-6'
  }], ['path', {
    d: 'M8 8l6-6'
  }], ['path', {
    d: 'M9 7l8 8'
  }], ['path', {
    d: 'M21 11l-8-8'
  }]]];
  var Gavel$1 = Gavel;

  var Gem = ['svg', defaultAttributes, [['polygon', {
    points: '6 3 18 3 22 9 12 22 2 9'
  }], ['path', {
    d: 'M12 22l4-13-3-6'
  }], ['path', {
    d: 'M12 22L8 9l3-6'
  }], ['path', {
    d: 'M2 9h20'
  }]]];
  var Gem$1 = Gem;

  var Ghost = ['svg', defaultAttributes, [['path', {
    d: 'M9 10h.01'
  }], ['path', {
    d: 'M15 10h.01'
  }], ['path', {
    d: 'M12 2a8 8 0 00-8 8v12l3-3 2.5 2.5L12 19l2.5 2.5L17 19l3 3V10a8 8 0 00-8-8z'
  }]]];
  var Ghost$1 = Ghost;

  var Gift = ['svg', defaultAttributes, [['polyline', {
    points: '20 12 20 22 4 22 4 12'
  }], ['rect', {
    x: '2',
    y: '7',
    width: '20',
    height: '5'
  }], ['line', {
    x1: '12',
    y1: '22',
    x2: '12',
    y2: '7'
  }], ['path', {
    d: 'M12 7H7.5a2.5 2.5 0 010-5C11 2 12 7 12 7z'
  }], ['path', {
    d: 'M12 7h4.5a2.5 2.5 0 000-5C13 2 12 7 12 7z'
  }]]];
  var Gift$1 = Gift;

  var GitBranchPlus = ['svg', defaultAttributes, [['path', {
    d: 'M6 3v12'
  }], ['path', {
    d: 'M18 9a3 3 0 100-6 3 3 0 000 6z'
  }], ['path', {
    d: 'M6 21a3 3 0 100-6 3 3 0 000 6z'
  }], ['path', {
    d: 'M15 6a9 9 0 00-9 9'
  }], ['path', {
    d: 'M18 15v6'
  }], ['path', {
    d: 'M21 18h-6'
  }]]];
  var GitBranchPlus$1 = GitBranchPlus;

  var GitBranch = ['svg', defaultAttributes, [['line', {
    x1: '6',
    y1: '3',
    x2: '6',
    y2: '15'
  }], ['circle', {
    cx: '18',
    cy: '6',
    r: '3'
  }], ['circle', {
    cx: '6',
    cy: '18',
    r: '3'
  }], ['path', {
    d: 'M18 9a9 9 0 01-9 9'
  }]]];
  var GitBranch$1 = GitBranch;

  var GitCommit = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '3'
  }], ['line', {
    x1: '3',
    y1: '12',
    x2: '9',
    y2: '12'
  }], ['line', {
    x1: '15',
    y1: '12',
    x2: '21',
    y2: '12'
  }]]];
  var GitCommit$1 = GitCommit;

  var GitFork = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '18',
    r: '3'
  }], ['circle', {
    cx: '6',
    cy: '6',
    r: '3'
  }], ['circle', {
    cx: '18',
    cy: '6',
    r: '3'
  }], ['path', {
    d: 'M18 9v1a2 2 0 01-2 2H8a2 2 0 01-2-2V9'
  }], ['path', {
    d: 'M12 12v3'
  }]]];
  var GitFork$1 = GitFork;

  var GitMerge = ['svg', defaultAttributes, [['circle', {
    cx: '18',
    cy: '18',
    r: '3'
  }], ['circle', {
    cx: '6',
    cy: '6',
    r: '3'
  }], ['path', {
    d: 'M6 21V9a9 9 0 009 9'
  }]]];
  var GitMerge$1 = GitMerge;

  var GitPullRequest = ['svg', defaultAttributes, [['circle', {
    cx: '18',
    cy: '18',
    r: '3'
  }], ['circle', {
    cx: '6',
    cy: '6',
    r: '3'
  }], ['path', {
    d: 'M13 6h3a2 2 0 012 2v7'
  }], ['line', {
    x1: '6',
    y1: '9',
    x2: '6',
    y2: '21'
  }]]];
  var GitPullRequest$1 = GitPullRequest;

  var Github = ['svg', defaultAttributes, [['path', {
    d: 'M9 19c-5 1.5-5-2.5-7-3m14 6v-3.87a3.37 3.37 0 00-.94-2.61c3.14-.35 6.44-1.54 6.44-7A5.44 5.44 0 0020 4.77 5.07 5.07 0 0019.91 1S18.73.65 16 2.48a13.38 13.38 0 00-7 0C6.27.65 5.09 1 5.09 1A5.07 5.07 0 005 4.77a5.44 5.44 0 00-1.5 3.78c0 5.42 3.3 6.61 6.44 7A3.37 3.37 0 009 18.13V22'
  }]]];
  var Github$1 = Github;

  var Gitlab = ['svg', defaultAttributes, [['path', {
    d: 'M22.65 14.39L12 22.13 1.35 14.39a.84.84 0 01-.3-.94l1.22-3.78 2.44-7.51A.42.42 0 014.82 2a.43.43 0 01.58 0 .42.42 0 01.11.18l2.44 7.49h8.1l2.44-7.51A.42.42 0 0118.6 2a.43.43 0 01.58 0 .42.42 0 01.11.18l2.44 7.51L23 13.45a.84.84 0 01-.35.94z'
  }]]];
  var Gitlab$1 = Gitlab;

  var Glasses = ['svg', defaultAttributes, [['circle', {
    cx: '6',
    cy: '15',
    r: '4'
  }], ['circle', {
    cx: '18',
    cy: '15',
    r: '4'
  }], ['path', {
    d: 'M14 15a2 2 0 00-2-2 2 2 0 00-2 2'
  }], ['path', {
    d: 'M2.5 13L5 7c.7-1.3 1.4-2 3-2'
  }], ['path', {
    d: 'M21.5 13L19 7c-.7-1.3-1.5-2-3-2'
  }]]];
  var Glasses$1 = Glasses;

  var Globe2 = ['svg', defaultAttributes, [['path', {
    d: 'M15 21v-4a2 2 0 012-2h4'
  }], ['path', {
    d: 'M7 4v2a3 3 0 003 2h0a2 2 0 012 2 2 2 0 004 0 2 2 0 012-2h3'
  }], ['path', {
    d: 'M3 11h2a2 2 0 012 2v1a2 2 0 002 2 2 2 0 012 2v4'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }]]];
  var Globe2$1 = Globe2;

  var Globe = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '2',
    y1: '12',
    x2: '22',
    y2: '12'
  }], ['path', {
    d: 'M12 2a15.3 15.3 0 014 10 15.3 15.3 0 01-4 10 15.3 15.3 0 01-4-10 15.3 15.3 0 014-10z'
  }]]];
  var Globe$1 = Globe;

  var Grab = ['svg', defaultAttributes, [['path', {
    d: 'M18 11.5V9a2 2 0 00-2-2v0a2 2 0 00-2 2v1.4'
  }], ['path', {
    d: 'M14 10V8a2 2 0 00-2-2v0a2 2 0 00-2 2v2'
  }], ['path', {
    d: 'M10 9.9V9a2 2 0 00-2-2v0a2 2 0 00-2 2v5'
  }], ['path', {
    d: 'M6 14v0a2 2 0 00-2-2v0a2 2 0 00-2 2v0'
  }], ['path', {
    d: 'M18 11v0a2 2 0 114 0v3a8 8 0 01-8 8h-4a8 8 0 01-8-8 2 2 0 114 0'
  }]]];
  var Grab$1 = Grab;

  var GraduationCap = ['svg', defaultAttributes, [['path', {
    d: 'M22 10v6M2 10l10-5 10 5-10 5z'
  }], ['path', {
    d: 'M6 12v5c3 3 9 3 12 0v-5'
  }]]];
  var GraduationCap$1 = GraduationCap;

  var Grid = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '3',
    y1: '9',
    x2: '21',
    y2: '9'
  }], ['line', {
    x1: '3',
    y1: '15',
    x2: '21',
    y2: '15'
  }], ['line', {
    x1: '9',
    y1: '3',
    x2: '9',
    y2: '21'
  }], ['line', {
    x1: '15',
    y1: '3',
    x2: '15',
    y2: '21'
  }]]];
  var Grid$1 = Grid;

  var GripHorizontal = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '9',
    r: '1'
  }], ['circle', {
    cx: '19',
    cy: '9',
    r: '1'
  }], ['circle', {
    cx: '5',
    cy: '9',
    r: '1'
  }], ['circle', {
    cx: '12',
    cy: '15',
    r: '1'
  }], ['circle', {
    cx: '19',
    cy: '15',
    r: '1'
  }], ['circle', {
    cx: '5',
    cy: '15',
    r: '1'
  }]]];
  var GripHorizontal$1 = GripHorizontal;

  var GripVertical = ['svg', defaultAttributes, [['circle', {
    cx: '9',
    cy: '12',
    r: '1'
  }], ['circle', {
    cx: '9',
    cy: '5',
    r: '1'
  }], ['circle', {
    cx: '9',
    cy: '19',
    r: '1'
  }], ['circle', {
    cx: '15',
    cy: '12',
    r: '1'
  }], ['circle', {
    cx: '15',
    cy: '5',
    r: '1'
  }], ['circle', {
    cx: '15',
    cy: '19',
    r: '1'
  }]]];
  var GripVertical$1 = GripVertical;

  var Hammer = ['svg', defaultAttributes, [['path', {
    d: 'M15 12l-8.5 8.5c-.83.83-2.17.83-3 0 0 0 0 0 0 0a2.12 2.12 0 010-3L12 9'
  }], ['path', {
    d: 'M17.64 15L22 10.64'
  }], ['path', {
    d: 'M20.91 11.7l-1.25-1.25c-.6-.6-.93-1.4-.93-2.25v-.86L16.01 4.6a5.56 5.56 0 00-3.94-1.64H9l.92.82A6.18 6.18 0 0112 8.4v1.56l2 2h2.47l2.26 1.91'
  }]]];
  var Hammer$1 = Hammer;

  var HandMetal = ['svg', defaultAttributes, [['path', {
    d: 'M18 12.5V10a2 2 0 00-2-2v0a2 2 0 00-2 2v1.4'
  }], ['path', {
    d: 'M14 11V9a2 2 0 10-4 0v2'
  }], ['path', {
    d: 'M10 10.5V5a2 2 0 10-4 0v9'
  }], ['path', {
    d: 'M7 15l-1.76-1.76a2 2 0 00-2.83 2.82l3.6 3.6C7.5 21.14 9.2 22 12 22h2a8 8 0 008-8V7a2 2 0 10-4 0v5'
  }]]];
  var HandMetal$1 = HandMetal;

  var Hand = ['svg', defaultAttributes, [['path', {
    d: 'M18 11V6a2 2 0 00-2-2v0a2 2 0 00-2 2v0'
  }], ['path', {
    d: 'M14 10V4a2 2 0 00-2-2v0a2 2 0 00-2 2v2'
  }], ['path', {
    d: 'M10 10.5V6a2 2 0 00-2-2v0a2 2 0 00-2 2v8'
  }], ['path', {
    d: 'M18 8a2 2 0 114 0v6a8 8 0 01-8 8h-2c-2.8 0-4.5-.86-5.99-2.34l-3.6-3.6a2 2 0 012.83-2.82L7 15'
  }]]];
  var Hand$1 = Hand;

  var HardDrive = ['svg', defaultAttributes, [['line', {
    x1: '22',
    y1: '12',
    x2: '2',
    y2: '12'
  }], ['path', {
    d: 'M5.45 5.11L2 12v6a2 2 0 002 2h16a2 2 0 002-2v-6l-3.45-6.89A2 2 0 0016.76 4H7.24a2 2 0 00-1.79 1.11z'
  }], ['line', {
    x1: '6',
    y1: '16',
    x2: '6.01',
    y2: '16'
  }], ['line', {
    x1: '10',
    y1: '16',
    x2: '10.01',
    y2: '16'
  }]]];
  var HardDrive$1 = HardDrive;

  var HardHat = ['svg', defaultAttributes, [['path', {
    d: 'M2 18a1 1 0 001 1h18a1 1 0 001-1v-2a1 1 0 00-1-1H3a1 1 0 00-1 1v2z'
  }], ['path', {
    d: 'M10 10V5a1 1 0 011-1h2a1 1 0 011 1v5'
  }], ['path', {
    d: 'M4 15v-3a6 6 0 016-6h0'
  }], ['path', {
    d: 'M14 6h0a6 6 0 016 6v3'
  }]]];
  var HardHat$1 = HardHat;

  var Hash = ['svg', defaultAttributes, [['line', {
    x1: '4',
    y1: '9',
    x2: '20',
    y2: '9'
  }], ['line', {
    x1: '4',
    y1: '15',
    x2: '20',
    y2: '15'
  }], ['line', {
    x1: '10',
    y1: '3',
    x2: '8',
    y2: '21'
  }], ['line', {
    x1: '16',
    y1: '3',
    x2: '14',
    y2: '21'
  }]]];
  var Hash$1 = Hash;

  var Haze = ['svg', defaultAttributes, [['path', {
    d: 'M5.2 6.2l1.4 1.4'
  }], ['path', {
    d: 'M2 13h2'
  }], ['path', {
    d: 'M20 13h2'
  }], ['path', {
    d: 'M17.4 7.6l1.4-1.4'
  }], ['path', {
    d: 'M22 17H2'
  }], ['path', {
    d: 'M22 21H2'
  }], ['path', {
    d: 'M16 13a4 4 0 00-8 0'
  }], ['path', {
    d: 'M12 5V2.5'
  }]]];
  var Haze$1 = Haze;

  var Headphones = ['svg', defaultAttributes, [['path', {
    d: 'M3 18v-6a9 9 0 0118 0v6'
  }], ['path', {
    d: 'M21 19a2 2 0 01-2 2h-1a2 2 0 01-2-2v-3a2 2 0 012-2h3zM3 19a2 2 0 002 2h1a2 2 0 002-2v-3a2 2 0 00-2-2H3z'
  }]]];
  var Headphones$1 = Headphones;

  var Heart = ['svg', defaultAttributes, [['path', {
    d: 'M20.42 4.58a5.4 5.4 0 00-7.65 0l-.77.78-.77-.78a5.4 5.4 0 00-7.65 0C1.46 6.7 1.33 10.28 4 13l8 8 8-8c2.67-2.72 2.54-6.3.42-8.42z'
  }]]];
  var Heart$1 = Heart;

  var HelpCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['path', {
    d: 'M9.09 9a3 3 0 015.83 1c0 2-3 3-3 3'
  }], ['line', {
    x1: '12',
    y1: '17',
    x2: '12.01',
    y2: '17'
  }]]];
  var HelpCircle$1 = HelpCircle;

  var Hexagon = ['svg', defaultAttributes, [['path', {
    d: 'M21 16V8a2 2 0 00-1-1.73l-7-4a2 2 0 00-2 0l-7 4A2 2 0 003 8v8a2 2 0 001 1.73l7 4a2 2 0 002 0l7-4A2 2 0 0021 16z'
  }]]];
  var Hexagon$1 = Hexagon;

  var Highlighter = ['svg', defaultAttributes, [['path', {
    d: 'M9 11l-6 6v3h9l3-3'
  }], ['path', {
    d: 'M22 12l-4.6 4.6a2 2 0 01-2.8 0l-5.2-5.2a2 2 0 010-2.8L14 4'
  }]]];
  var Highlighter$1 = Highlighter;

  var History = ['svg', defaultAttributes, [['path', {
    d: 'M3 3v5h5'
  }], ['path', {
    d: 'M3.05 13A9 9 0 106 5.3L3 8'
  }], ['path', {
    d: 'M12 7v5l4 2'
  }]]];
  var History$1 = History;

  var Home = ['svg', defaultAttributes, [['path', {
    d: 'M3 9l9-7 9 7v11a2 2 0 01-2 2H5a2 2 0 01-2-2z'
  }], ['polyline', {
    points: '9 22 9 12 15 12 15 22'
  }]]];
  var Home$1 = Home;

  var ImageMinus = ['svg', defaultAttributes, [['path', {
    d: 'M21 11v8a2 2 0 01-2 2H5a2 2 0 01-2-2V5a2 2 0 012-2h8'
  }], ['path', {
    d: 'M8.5 10a1.5 1.5 0 100-3 1.5 1.5 0 000 3z'
  }], ['path', {
    d: 'M21 15l-5-5L5 21'
  }], ['path', {
    d: 'M22 5h-6'
  }]]];
  var ImageMinus$1 = ImageMinus;

  var ImageOff = ['svg', defaultAttributes, [['path', {
    d: 'M2 2l20 20'
  }], ['path', {
    d: 'M9 3h10a2 2 0 012 2v10M3.59 3.59A2 2 0 003 5v14c0 1.1.9 2 2 2h14a2 2 0 001.41-.59'
  }], ['path', {
    d: 'M9.56 9.56a1.5 1.5 0 01-2.12-2.12'
  }], ['path', {
    d: 'M21 15l-5-5M5 21l8-8'
  }]]];
  var ImageOff$1 = ImageOff;

  var ImagePlus = ['svg', defaultAttributes, [['path', {
    d: 'M21 11v8a2 2 0 01-2 2H5a2 2 0 01-2-2V5a2 2 0 012-2h8'
  }], ['path', {
    d: 'M8.5 10a1.5 1.5 0 100-3 1.5 1.5 0 000 3z'
  }], ['path', {
    d: 'M21 15l-5-5L5 21'
  }], ['path', {
    d: 'M19 2v6'
  }], ['path', {
    d: 'M22 5h-6'
  }]]];
  var ImagePlus$1 = ImagePlus;

  var Image = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['circle', {
    cx: '8.5',
    cy: '8.5',
    r: '1.5'
  }], ['polyline', {
    points: '21 15 16 10 5 21'
  }]]];
  var Image$1 = Image;

  var Import = ['svg', defaultAttributes, [['path', {
    d: 'M12 3v12'
  }], ['path', {
    d: 'M8 11l4 4 4-4'
  }], ['path', {
    d: 'M8 5H4a2 2 0 00-2 2v10a2 2 0 002 2h16a2 2 0 002-2V7a2 2 0 00-2-2h-4'
  }]]];
  var Import$1 = Import;

  var Inbox = ['svg', defaultAttributes, [['polyline', {
    points: '22 12 16 12 14 15 10 15 8 12 2 12'
  }], ['path', {
    d: 'M5.45 5.11L2 12v6a2 2 0 002 2h16a2 2 0 002-2v-6l-3.45-6.89A2 2 0 0016.76 4H7.24a2 2 0 00-1.79 1.11z'
  }]]];
  var Inbox$1 = Inbox;

  var Indent = ['svg', defaultAttributes, [['polyline', {
    points: '3 8 7 12 3 16'
  }], ['line', {
    x1: '21',
    y1: '12',
    x2: '11',
    y2: '12'
  }], ['line', {
    x1: '21',
    y1: '6',
    x2: '11',
    y2: '6'
  }], ['line', {
    x1: '21',
    y1: '18',
    x2: '11',
    y2: '18'
  }]]];
  var Indent$1 = Indent;

  var IndianRupee = ['svg', defaultAttributes, [['path', {
    d: 'M6 3h12'
  }], ['path', {
    d: 'M6 8h12'
  }], ['path', {
    d: 'M6 13l8.5 8'
  }], ['path', {
    d: 'M6 13h3'
  }], ['path', {
    d: 'M9 13c6.667 0 6.667-10 0-10'
  }]]];
  var IndianRupee$1 = IndianRupee;

  var Infinity = ['svg', defaultAttributes, [['path', {
    d: 'M18.178 8c5.096 0 5.096 8 0 8-5.095 0-7.133-8-12.739-8-4.585 0-4.585 8 0 8 5.606 0 7.644-8 12.74-8z'
  }]]];
  var Infinity$1 = Infinity;

  var Info = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '12',
    y1: '16',
    x2: '12',
    y2: '12'
  }], ['line', {
    x1: '12',
    y1: '8',
    x2: '12.01',
    y2: '8'
  }]]];
  var Info$1 = Info;

  var Inspect = ['svg', defaultAttributes, [['path', {
    d: 'M19 11V4a2 2 0 00-2-2H4a2 2 0 00-2 2v13a2 2 0 002 2h7'
  }], ['path', {
    d: 'M12 12l4.166 10 1.48-4.355L22 16.166 12 12z'
  }], ['path', {
    d: 'M18 18l3 3'
  }]]];
  var Inspect$1 = Inspect;

  var Instagram = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '2',
    width: '20',
    height: '20',
    rx: '5',
    ry: '5'
  }], ['path', {
    d: 'M16 11.37A4 4 0 1112.63 8 4 4 0 0116 11.37z'
  }], ['line', {
    x1: '17.5',
    y1: '6.5',
    x2: '17.51',
    y2: '6.5'
  }]]];
  var Instagram$1 = Instagram;

  var Italic = ['svg', defaultAttributes, [['line', {
    x1: '19',
    y1: '4',
    x2: '10',
    y2: '4'
  }], ['line', {
    x1: '14',
    y1: '20',
    x2: '5',
    y2: '20'
  }], ['line', {
    x1: '15',
    y1: '4',
    x2: '9',
    y2: '20'
  }]]];
  var Italic$1 = Italic;

  var JapaneseYen = ['svg', defaultAttributes, [['path', {
    d: 'M12 9.5V21m0-11.5L6 3m6 6.5L18 3'
  }], ['path', {
    d: 'M6 15h12'
  }], ['path', {
    d: 'M6 11h12'
  }]]];
  var JapaneseYen$1 = JapaneseYen;

  var Key = ['svg', defaultAttributes, [['path', {
    d: 'M21 2l-2 2m-7.61 7.61a5.5 5.5 0 11-7.778 7.778 5.5 5.5 0 017.777-7.777zm0 0L15.5 7.5m0 0l3 3L22 7l-3-3m-3.5 3.5L19 4'
  }]]];
  var Key$1 = Key;

  var Keyboard = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '4',
    width: '20',
    height: '16',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M6 8h.001'
  }], ['path', {
    d: 'M10 8h.001'
  }], ['path', {
    d: 'M14 8h.001'
  }], ['path', {
    d: 'M18 8h.001'
  }], ['path', {
    d: 'M8 12h.001'
  }], ['path', {
    d: 'M12 12h.001'
  }], ['path', {
    d: 'M16 12h.001'
  }], ['path', {
    d: 'M7 16h10'
  }]]];
  var Keyboard$1 = Keyboard;

  var Landmark = ['svg', defaultAttributes, [['line', {
    x1: '3',
    y1: '22',
    x2: '21',
    y2: '22'
  }], ['line', {
    x1: '6',
    y1: '18',
    x2: '6',
    y2: '11'
  }], ['line', {
    x1: '10',
    y1: '18',
    x2: '10',
    y2: '11'
  }], ['line', {
    x1: '14',
    y1: '18',
    x2: '14',
    y2: '11'
  }], ['line', {
    x1: '18',
    y1: '18',
    x2: '18',
    y2: '11'
  }], ['polygon', {
    points: '12 2 20 7 4 7'
  }]]];
  var Landmark$1 = Landmark;

  var Languages = ['svg', defaultAttributes, [['path', {
    d: 'M5 8l6 6'
  }], ['path', {
    d: 'M4 14l6-6 2-3'
  }], ['path', {
    d: 'M2 5h12'
  }], ['path', {
    d: 'M7 2h1'
  }], ['path', {
    d: 'M22 22l-5-10-5 10'
  }], ['path', {
    d: 'M14 18h6'
  }]]];
  var Languages$1 = Languages;

  var Laptop2 = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '4',
    width: '18',
    height: '12',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '2',
    y1: '20',
    x2: '22',
    y2: '20'
  }]]];
  var Laptop2$1 = Laptop2;

  var Laptop = ['svg', defaultAttributes, [['path', {
    d: 'M20 16V7a2 2 0 00-2-2H6a2 2 0 00-2 2v9m16 0H4m16 0l1.28 2.55a1 1 0 01-.9 1.45H3.62a1 1 0 01-.9-1.45L4 16'
  }]]];
  var Laptop$1 = Laptop;

  var LassoSelect = ['svg', defaultAttributes, [['path', {
    d: 'M7 22a5 5 0 01-2-4'
  }], ['path', {
    d: 'M7 16.93c.96.43 1.96.74 2.99.91'
  }], ['path', {
    d: 'M3.34 14A6.8 6.8 0 012 10c0-4.42 4.48-8 10-8s10 3.58 10 8a7.19 7.19 0 01-.33 2'
  }], ['path', {
    d: 'M5 18a2 2 0 100-4 2 2 0 000 4z'
  }], ['path', {
    d: 'M14.33 22h-.09a.35.35 0 01-.24-.32v-10a.34.34 0 01.33-.34c.08 0 .15.03.21.08l7.34 6a.33.33 0 01-.21.59h-4.49l-2.57 3.85a.35.35 0 01-.28.14v0z'
  }]]];
  var LassoSelect$1 = LassoSelect;

  var Lasso = ['svg', defaultAttributes, [['path', {
    d: 'M7 22a5 5 0 01-2-4'
  }], ['path', {
    d: 'M3.3 14A6.8 6.8 0 012 10c0-4.4 4.5-8 10-8s10 3.6 10 8-4.5 8-10 8a12 12 0 01-5-1'
  }], ['path', {
    d: 'M5 18a2 2 0 100-4 2 2 0 000 4z'
  }]]];
  var Lasso$1 = Lasso;

  var Layers = ['svg', defaultAttributes, [['polygon', {
    points: '12 2 2 7 12 12 22 7 12 2'
  }], ['polyline', {
    points: '2 17 12 22 22 17'
  }], ['polyline', {
    points: '2 12 12 17 22 12'
  }]]];
  var Layers$1 = Layers;

  var LayoutDashboard = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '7',
    height: '9'
  }], ['rect', {
    x: '14',
    y: '3',
    width: '7',
    height: '5'
  }], ['rect', {
    x: '14',
    y: '12',
    width: '7',
    height: '9'
  }], ['rect', {
    x: '3',
    y: '16',
    width: '7',
    height: '5'
  }]]];
  var LayoutDashboard$1 = LayoutDashboard;

  var LayoutGrid = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '7',
    height: '7'
  }], ['rect', {
    x: '14',
    y: '3',
    width: '7',
    height: '7'
  }], ['rect', {
    x: '14',
    y: '14',
    width: '7',
    height: '7'
  }], ['rect', {
    x: '3',
    y: '14',
    width: '7',
    height: '7'
  }]]];
  var LayoutGrid$1 = LayoutGrid;

  var LayoutList = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '14',
    width: '7',
    height: '7'
  }], ['rect', {
    x: '3',
    y: '3',
    width: '7',
    height: '7'
  }], ['line', {
    x1: '14',
    y1: '4',
    x2: '21',
    y2: '4'
  }], ['line', {
    x1: '14',
    y1: '9',
    x2: '21',
    y2: '9'
  }], ['line', {
    x1: '14',
    y1: '15',
    x2: '21',
    y2: '15'
  }], ['line', {
    x1: '14',
    y1: '20',
    x2: '21',
    y2: '20'
  }]]];
  var LayoutList$1 = LayoutList;

  var LayoutTemplate = ['svg', defaultAttributes, [['path', {
    d: 'M21 3H3v7h18V3z'
  }], ['path', {
    d: 'M21 14h-5v7h5v-7z'
  }], ['path', {
    d: 'M12 14H3v7h9v-7z'
  }]]];
  var LayoutTemplate$1 = LayoutTemplate;

  var Layout = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '3',
    y1: '9',
    x2: '21',
    y2: '9'
  }], ['line', {
    x1: '9',
    y1: '21',
    x2: '9',
    y2: '9'
  }]]];
  var Layout$1 = Layout;

  var Library = ['svg', defaultAttributes, [['path', {
    d: 'M16 6l4 14'
  }], ['path', {
    d: 'M12 6v14'
  }], ['path', {
    d: 'M8 8v12'
  }], ['path', {
    d: 'M4 4v16'
  }]]];
  var Library$1 = Library;

  var LifeBuoy = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '4'
  }], ['line', {
    x1: '4.93',
    y1: '4.93',
    x2: '9.17',
    y2: '9.17'
  }], ['line', {
    x1: '14.83',
    y1: '14.83',
    x2: '19.07',
    y2: '19.07'
  }], ['line', {
    x1: '14.83',
    y1: '9.17',
    x2: '19.07',
    y2: '4.93'
  }], ['line', {
    x1: '14.83',
    y1: '9.17',
    x2: '18.36',
    y2: '5.64'
  }], ['line', {
    x1: '4.93',
    y1: '19.07',
    x2: '9.17',
    y2: '14.83'
  }]]];
  var LifeBuoy$1 = LifeBuoy;

  var LightbulbOff = ['svg', defaultAttributes, [['path', {
    d: 'M9 18h6'
  }], ['path', {
    d: 'M10 22h4'
  }], ['path', {
    d: 'M2 2l20 20'
  }], ['path', {
    d: 'M8.91 14a4.61 4.61 0 00-1.41-2.5C6.23 10.23 6 9 6 8a6 6 0 01.084-1M9 2.804A6 6 0 0118 8a4.651 4.651 0 01-1.031 3'
  }]]];
  var LightbulbOff$1 = LightbulbOff;

  var Lightbulb = ['svg', defaultAttributes, [['line', {
    x1: '9',
    y1: '18',
    x2: '15',
    y2: '18'
  }], ['line', {
    x1: '10',
    y1: '22',
    x2: '14',
    y2: '22'
  }], ['path', {
    d: 'M15.09 14c.18-.98.65-1.74 1.41-2.5A4.65 4.65 0 0018 8 6 6 0 006 8c0 1 .23 2.23 1.5 3.5A4.61 4.61 0 018.91 14'
  }]]];
  var Lightbulb$1 = Lightbulb;

  var Link2Off = ['svg', defaultAttributes, [['path', {
    d: 'M15 7h2a5 5 0 014 8M9 17H7A5 5 0 017 7'
  }], ['line', {
    x1: '8',
    y1: '12',
    x2: '12',
    y2: '12'
  }], ['line', {
    x1: '2',
    y1: '2',
    x2: '22',
    y2: '22'
  }]]];
  var Link2Off$1 = Link2Off;

  var Link2 = ['svg', defaultAttributes, [['path', {
    d: 'M15 7h2a5 5 0 010 10h-2m-6 0H7A5 5 0 017 7h2'
  }], ['line', {
    x1: '8',
    y1: '12',
    x2: '16',
    y2: '12'
  }]]];
  var Link2$1 = Link2;

  var Link = ['svg', defaultAttributes, [['path', {
    d: 'M10 13a5 5 0 007.54.54l3-3a5 5 0 00-7.07-7.07l-1.72 1.71'
  }], ['path', {
    d: 'M14 11a5 5 0 00-7.54-.54l-3 3a5 5 0 007.07 7.07l1.71-1.71'
  }]]];
  var Link$1 = Link;

  var Linkedin = ['svg', defaultAttributes, [['path', {
    d: 'M16 8a6 6 0 016 6v7h-4v-7a2 2 0 00-2-2 2 2 0 00-2 2v7h-4v-7a6 6 0 016-6z'
  }], ['rect', {
    x: '2',
    y: '9',
    width: '4',
    height: '12'
  }], ['circle', {
    cx: '4',
    cy: '4',
    r: '2'
  }]]];
  var Linkedin$1 = Linkedin;

  var ListChecks = ['svg', defaultAttributes, [['line', {
    x1: '10',
    y1: '6',
    x2: '21',
    y2: '6'
  }], ['line', {
    x1: '10',
    y1: '12',
    x2: '21',
    y2: '12'
  }], ['line', {
    x1: '10',
    y1: '18',
    x2: '21',
    y2: '18'
  }], ['polyline', {
    points: '3 6 4 7 6 5'
  }], ['polyline', {
    points: '3 12 4 13 6 11'
  }], ['polyline', {
    points: '3 18 4 19 6 17'
  }]]];
  var ListChecks$1 = ListChecks;

  var ListMinus = ['svg', defaultAttributes, [['path', {
    d: 'M11 12H3'
  }], ['path', {
    d: 'M16 6H3'
  }], ['path', {
    d: 'M16 18H3'
  }], ['path', {
    d: 'M21 12h-6'
  }]]];
  var ListMinus$1 = ListMinus;

  var ListOrdered = ['svg', defaultAttributes, [['line', {
    x1: '10',
    y1: '6',
    x2: '21',
    y2: '6'
  }], ['line', {
    x1: '10',
    y1: '12',
    x2: '21',
    y2: '12'
  }], ['line', {
    x1: '10',
    y1: '18',
    x2: '21',
    y2: '18'
  }], ['path', {
    d: 'M4 6h1v4'
  }], ['path', {
    d: 'M4 10h2'
  }], ['path', {
    d: 'M6 18H4c0-1 2-2 2-3s-1-1.5-2-1'
  }]]];
  var ListOrdered$1 = ListOrdered;

  var ListPlus = ['svg', defaultAttributes, [['path', {
    d: 'M11 12H3'
  }], ['path', {
    d: 'M16 6H3'
  }], ['path', {
    d: 'M16 18H3'
  }], ['path', {
    d: 'M18 9v6'
  }], ['path', {
    d: 'M21 12h-6'
  }]]];
  var ListPlus$1 = ListPlus;

  var ListX = ['svg', defaultAttributes, [['path', {
    d: 'M11 12H3'
  }], ['path', {
    d: 'M16 6H3'
  }], ['path', {
    d: 'M16 18H3'
  }], ['path', {
    d: 'M19 10l-4 4'
  }], ['path', {
    d: 'M15 10l4 4'
  }]]];
  var ListX$1 = ListX;

  var List = ['svg', defaultAttributes, [['line', {
    x1: '8',
    y1: '6',
    x2: '21',
    y2: '6'
  }], ['line', {
    x1: '8',
    y1: '12',
    x2: '21',
    y2: '12'
  }], ['line', {
    x1: '8',
    y1: '18',
    x2: '21',
    y2: '18'
  }], ['line', {
    x1: '3',
    y1: '6',
    x2: '3.01',
    y2: '6'
  }], ['line', {
    x1: '3',
    y1: '12',
    x2: '3.01',
    y2: '12'
  }], ['line', {
    x1: '3',
    y1: '18',
    x2: '3.01',
    y2: '18'
  }]]];
  var List$1 = List;

  var Loader2 = ['svg', defaultAttributes, [['path', {
    d: 'M21 12a9 9 0 11-6.219-8.56'
  }]]];
  var Loader2$1 = Loader2;

  var Loader = ['svg', defaultAttributes, [['line', {
    x1: '12',
    y1: '2',
    x2: '12',
    y2: '6'
  }], ['line', {
    x1: '12',
    y1: '18',
    x2: '12',
    y2: '22'
  }], ['line', {
    x1: '4.93',
    y1: '4.93',
    x2: '7.76',
    y2: '7.76'
  }], ['line', {
    x1: '16.24',
    y1: '16.24',
    x2: '19.07',
    y2: '19.07'
  }], ['line', {
    x1: '2',
    y1: '12',
    x2: '6',
    y2: '12'
  }], ['line', {
    x1: '18',
    y1: '12',
    x2: '22',
    y2: '12'
  }], ['line', {
    x1: '4.93',
    y1: '19.07',
    x2: '7.76',
    y2: '16.24'
  }], ['line', {
    x1: '16.24',
    y1: '7.76',
    x2: '19.07',
    y2: '4.93'
  }]]];
  var Loader$1 = Loader;

  var LocateFixed = ['svg', defaultAttributes, [['line', {
    x1: '2',
    x2: '5',
    y1: '12',
    y2: '12'
  }], ['line', {
    x1: '19',
    x2: '22',
    y1: '12',
    y2: '12'
  }], ['line', {
    x1: '12',
    x2: '12',
    y1: '2',
    y2: '5'
  }], ['line', {
    x1: '12',
    x2: '12',
    y1: '19',
    y2: '22'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '7'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '3'
  }]]];
  var LocateFixed$1 = LocateFixed;

  var LocateOff = ['svg', defaultAttributes, [['line', {
    x1: '2',
    x2: '5',
    y1: '12',
    y2: '12'
  }], ['line', {
    x1: '19',
    x2: '22',
    y1: '12',
    y2: '12'
  }], ['line', {
    x1: '12',
    x2: '12',
    y1: '2',
    y2: '5'
  }], ['line', {
    x1: '12',
    x2: '12',
    y1: '19',
    y2: '22'
  }], ['path', {
    d: 'M7.11 7.11C5.83 8.39 5 10.1 5 12c0 3.87 3.13 7 7 7 1.9 0 3.61-.83 4.89-2.11'
  }], ['path', {
    d: 'M18.71 13.96c.19-.63.29-1.29.29-1.96 0-3.87-3.13-7-7-7-.67 0-1.33.1-1.96.29'
  }], ['line', {
    x1: '2',
    x2: '22',
    y1: '2',
    y2: '22'
  }]]];
  var LocateOff$1 = LocateOff;

  var Locate = ['svg', defaultAttributes, [['line', {
    x1: '2',
    x2: '5',
    y1: '12',
    y2: '12'
  }], ['line', {
    x1: '19',
    x2: '22',
    y1: '12',
    y2: '12'
  }], ['line', {
    x1: '12',
    x2: '12',
    y1: '2',
    y2: '5'
  }], ['line', {
    x1: '12',
    x2: '12',
    y1: '19',
    y2: '22'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '7'
  }]]];
  var Locate$1 = Locate;

  var Lock = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '11',
    width: '18',
    height: '11',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M7 11V7a5 5 0 0110 0v4'
  }]]];
  var Lock$1 = Lock;

  var LogIn = ['svg', defaultAttributes, [['path', {
    d: 'M15 3h4a2 2 0 012 2v14a2 2 0 01-2 2h-4'
  }], ['polyline', {
    points: '10 17 15 12 10 7'
  }], ['line', {
    x1: '15',
    y1: '12',
    x2: '3',
    y2: '12'
  }]]];
  var LogIn$1 = LogIn;

  var LogOut = ['svg', defaultAttributes, [['path', {
    d: 'M9 21H5a2 2 0 01-2-2V5a2 2 0 012-2h4'
  }], ['polyline', {
    points: '16 17 21 12 16 7'
  }], ['line', {
    x1: '21',
    y1: '12',
    x2: '9',
    y2: '12'
  }]]];
  var LogOut$1 = LogOut;

  var Mail = ['svg', defaultAttributes, [['path', {
    d: 'M4 4h16c1.1 0 2 .9 2 2v12c0 1.1-.9 2-2 2H4c-1.1 0-2-.9-2-2V6c0-1.1.9-2 2-2z'
  }], ['polyline', {
    points: '22,6 12,13 2,6'
  }]]];
  var Mail$1 = Mail;

  var MapPin = ['svg', defaultAttributes, [['path', {
    d: 'M21 10c0 7-9 13-9 13s-9-6-9-13a9 9 0 0118 0z'
  }], ['circle', {
    cx: '12',
    cy: '10',
    r: '3'
  }]]];
  var MapPin$1 = MapPin;

  var Map = ['svg', defaultAttributes, [['polygon', {
    points: '1 6 1 22 8 18 16 22 23 18 23 2 16 6 8 2 1 6'
  }], ['line', {
    x1: '8',
    y1: '2',
    x2: '8',
    y2: '18'
  }], ['line', {
    x1: '16',
    y1: '6',
    x2: '16',
    y2: '22'
  }]]];
  var Map$1 = Map;

  var Maximize2 = ['svg', defaultAttributes, [['polyline', {
    points: '15 3 21 3 21 9'
  }], ['polyline', {
    points: '9 21 3 21 3 15'
  }], ['line', {
    x1: '21',
    y1: '3',
    x2: '14',
    y2: '10'
  }], ['line', {
    x1: '3',
    y1: '21',
    x2: '10',
    y2: '14'
  }]]];
  var Maximize2$1 = Maximize2;

  var Maximize = ['svg', defaultAttributes, [['path', {
    d: 'M8 3H5a2 2 0 00-2 2v3m18 0V5a2 2 0 00-2-2h-3m0 18h3a2 2 0 002-2v-3M3 16v3a2 2 0 002 2h3'
  }]]];
  var Maximize$1 = Maximize;

  var Megaphone = ['svg', defaultAttributes, [['path', {
    d: 'M3 11l18-5v12L3 14v-3z'
  }], ['path', {
    d: 'M11.6 16.8a3 3 0 11-5.8-1.6'
  }]]];
  var Megaphone$1 = Megaphone;

  var Meh = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '8',
    y1: '15',
    x2: '16',
    y2: '15'
  }], ['line', {
    x1: '9',
    y1: '9',
    x2: '9.01',
    y2: '9'
  }], ['line', {
    x1: '15',
    y1: '9',
    x2: '15.01',
    y2: '9'
  }]]];
  var Meh$1 = Meh;

  var Menu = ['svg', defaultAttributes, [['line', {
    x1: '4',
    y1: '12',
    x2: '20',
    y2: '12'
  }], ['line', {
    x1: '4',
    y1: '6',
    x2: '20',
    y2: '6'
  }], ['line', {
    x1: '4',
    y1: '18',
    x2: '20',
    y2: '18'
  }]]];
  var Menu$1 = Menu;

  var MessageCircle = ['svg', defaultAttributes, [['path', {
    d: 'M21 11.5a8.38 8.38 0 01-.9 3.8 8.5 8.5 0 01-7.6 4.7 8.38 8.38 0 01-3.8-.9L3 21l1.9-5.7a8.38 8.38 0 01-.9-3.8 8.5 8.5 0 014.7-7.6 8.38 8.38 0 013.8-.9h.5a8.48 8.48 0 018 8v.5z'
  }]]];
  var MessageCircle$1 = MessageCircle;

  var MessageSquare = ['svg', defaultAttributes, [['path', {
    d: 'M21 15a2 2 0 01-2 2H7l-4 4V5a2 2 0 012-2h14a2 2 0 012 2z'
  }]]];
  var MessageSquare$1 = MessageSquare;

  var MicOff = ['svg', defaultAttributes, [['line', {
    x1: '1',
    y1: '1',
    x2: '23',
    y2: '23'
  }], ['path', {
    d: 'M9 9v3a3 3 0 005.12 2.12M15 9.34V4a3 3 0 00-5.94-.6'
  }], ['path', {
    d: 'M17 16.95A7 7 0 015 12v-2m14 0v2a7 7 0 01-.11 1.23'
  }], ['line', {
    x1: '12',
    y1: '19',
    x2: '12',
    y2: '23'
  }], ['line', {
    x1: '8',
    y1: '23',
    x2: '16',
    y2: '23'
  }]]];
  var MicOff$1 = MicOff;

  var Mic = ['svg', defaultAttributes, [['path', {
    d: 'M12 1a3 3 0 00-3 3v8a3 3 0 006 0V4a3 3 0 00-3-3z'
  }], ['path', {
    d: 'M19 10v2a7 7 0 01-14 0v-2'
  }], ['line', {
    x1: '12',
    y1: '19',
    x2: '12',
    y2: '23'
  }], ['line', {
    x1: '8',
    y1: '23',
    x2: '16',
    y2: '23'
  }]]];
  var Mic$1 = Mic;

  var Minimize2 = ['svg', defaultAttributes, [['polyline', {
    points: '4 14 10 14 10 20'
  }], ['polyline', {
    points: '20 10 14 10 14 4'
  }], ['line', {
    x1: '14',
    y1: '10',
    x2: '21',
    y2: '3'
  }], ['line', {
    x1: '3',
    y1: '21',
    x2: '10',
    y2: '14'
  }]]];
  var Minimize2$1 = Minimize2;

  var Minimize = ['svg', defaultAttributes, [['path', {
    d: 'M8 3v3a2 2 0 01-2 2H3m18 0h-3a2 2 0 01-2-2V3m0 18v-3a2 2 0 012-2h3M3 16h3a2 2 0 012 2v3'
  }]]];
  var Minimize$1 = Minimize;

  var MinusCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '8',
    y1: '12',
    x2: '16',
    y2: '12'
  }]]];
  var MinusCircle$1 = MinusCircle;

  var MinusSquare = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '8',
    y1: '12',
    x2: '16',
    y2: '12'
  }]]];
  var MinusSquare$1 = MinusSquare;

  var Minus = ['svg', defaultAttributes, [['line', {
    x1: '5',
    y1: '12',
    x2: '19',
    y2: '12'
  }]]];
  var Minus$1 = Minus;

  var MonitorOff = ['svg', defaultAttributes, [['path', {
    d: 'M17 17H4a2 2 0 01-2-2V5c0-1.5 1-2 1-2'
  }], ['path', {
    d: 'M22 15V5a2 2 0 00-2-2H9'
  }], ['path', {
    d: 'M8 21h8'
  }], ['path', {
    d: 'M12 17v4'
  }], ['path', {
    d: 'M2 2l20 20'
  }]]];
  var MonitorOff$1 = MonitorOff;

  var MonitorSpeaker = ['svg', defaultAttributes, [['path', {
    d: 'M5.5 20H8'
  }], ['path', {
    d: 'M17 9h.01'
  }], ['rect', {
    x: '12',
    y: '4',
    width: '10',
    height: '16',
    rx: '2'
  }], ['path', {
    d: 'M8 6H4a2 2 0 00-2 2v6a2 2 0 002 2h4'
  }], ['circle', {
    cx: '17',
    cy: '15',
    r: '1'
  }]]];
  var MonitorSpeaker$1 = MonitorSpeaker;

  var Monitor = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '3',
    width: '20',
    height: '14',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '8',
    y1: '21',
    x2: '16',
    y2: '21'
  }], ['line', {
    x1: '12',
    y1: '17',
    x2: '12',
    y2: '21'
  }]]];
  var Monitor$1 = Monitor;

  var Moon = ['svg', defaultAttributes, [['path', {
    d: 'M21 12.79A9 9 0 1111.21 3 7 7 0 0021 12.79z'
  }]]];
  var Moon$1 = Moon;

  var MoreHorizontal = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '1'
  }], ['circle', {
    cx: '19',
    cy: '12',
    r: '1'
  }], ['circle', {
    cx: '5',
    cy: '12',
    r: '1'
  }]]];
  var MoreHorizontal$1 = MoreHorizontal;

  var MoreVertical = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '1'
  }], ['circle', {
    cx: '12',
    cy: '5',
    r: '1'
  }], ['circle', {
    cx: '12',
    cy: '19',
    r: '1'
  }]]];
  var MoreVertical$1 = MoreVertical;

  var MountainSnow = ['svg', defaultAttributes, [['path', {
    d: 'M8 3l4 8 5-5 5 15H2L8 3z'
  }], ['path', {
    d: 'M4.14 15.08c2.62-1.57 5.24-1.43 7.86.42 2.74 1.94 5.49 2 8.23.19'
  }]]];
  var MountainSnow$1 = MountainSnow;

  var Mountain = ['svg', defaultAttributes, [['path', {
    d: 'M8 3l4 8 5-5 5 15H2L8 3z'
  }]]];
  var Mountain$1 = Mountain;

  var MousePointer2 = ['svg', defaultAttributes, [['path', {
    d: 'M4 4l7.07 17 2.51-7.39L21 11.07z'
  }]]];
  var MousePointer2$1 = MousePointer2;

  var MousePointerClick = ['svg', defaultAttributes, [['path', {
    d: 'M9 9l5 12 1.774-5.226L21 14 9 9z'
  }], ['path', {
    d: 'M16.071 16.071l4.243 4.243'
  }], ['path', {
    d: 'M7.188 2.239l.777 2.897M5.136 7.965l-2.898-.777M13.95 4.05l-2.122 2.122m-5.657 5.656l-2.12 2.122'
  }]]];
  var MousePointerClick$1 = MousePointerClick;

  var MousePointer = ['svg', defaultAttributes, [['path', {
    d: 'M3 3l7.07 16.97 2.51-7.39 7.39-2.51L3 3z'
  }], ['path', {
    d: 'M13 13l6 6'
  }]]];
  var MousePointer$1 = MousePointer;

  var MoveDiagonal2 = ['svg', defaultAttributes, [['polyline', {
    points: '5 11 5 5 11 5'
  }], ['polyline', {
    points: '19 13 19 19 13 19'
  }], ['line', {
    x1: '5',
    y1: '5',
    x2: '19',
    y2: '19'
  }]]];
  var MoveDiagonal2$1 = MoveDiagonal2;

  var MoveDiagonal = ['svg', defaultAttributes, [['polyline', {
    points: '13 5 19 5 19 11'
  }], ['polyline', {
    points: '11 19 5 19 5 13'
  }], ['line', {
    x1: '19',
    y1: '5',
    x2: '5',
    y2: '19'
  }]]];
  var MoveDiagonal$1 = MoveDiagonal;

  var MoveHorizontal = ['svg', defaultAttributes, [['polyline', {
    points: '18 8 22 12 18 16'
  }], ['polyline', {
    points: '6 8 2 12 6 16'
  }], ['line', {
    x1: '2',
    y1: '12',
    x2: '22',
    y2: '12'
  }]]];
  var MoveHorizontal$1 = MoveHorizontal;

  var MoveVertical = ['svg', defaultAttributes, [['polyline', {
    points: '8 18 12 22 16 18'
  }], ['polyline', {
    points: '8 6 12 2 16 6'
  }], ['line', {
    x1: '12',
    y1: '2',
    x2: '12',
    y2: '22'
  }]]];
  var MoveVertical$1 = MoveVertical;

  var Move = ['svg', defaultAttributes, [['polyline', {
    points: '5 9 2 12 5 15'
  }], ['polyline', {
    points: '9 5 12 2 15 5'
  }], ['polyline', {
    points: '15 19 12 22 9 19'
  }], ['polyline', {
    points: '19 9 22 12 19 15'
  }], ['line', {
    x1: '2',
    y1: '12',
    x2: '22',
    y2: '12'
  }], ['line', {
    x1: '12',
    y1: '2',
    x2: '12',
    y2: '22'
  }]]];
  var Move$1 = Move;

  var Music = ['svg', defaultAttributes, [['path', {
    d: 'M9 18V5l12-2v13'
  }], ['circle', {
    cx: '6',
    cy: '18',
    r: '3'
  }], ['circle', {
    cx: '18',
    cy: '16',
    r: '3'
  }]]];
  var Music$1 = Music;

  var Navigation2 = ['svg', defaultAttributes, [['polygon', {
    points: '12 2 19 21 12 17 5 21 12 2'
  }]]];
  var Navigation2$1 = Navigation2;

  var Navigation = ['svg', defaultAttributes, [['polygon', {
    points: '3 11 22 2 13 21 11 13 3 11'
  }]]];
  var Navigation$1 = Navigation;

  var Network = ['svg', defaultAttributes, [['rect', {
    x: '9',
    y: '2',
    width: '6',
    height: '6'
  }], ['rect', {
    x: '16',
    y: '16',
    width: '6',
    height: '6'
  }], ['rect', {
    x: '2',
    y: '16',
    width: '6',
    height: '6'
  }], ['path', {
    d: 'M12 8v4m0 0H5v4m7-4h7v4'
  }]]];
  var Network$1 = Network;

  var Octagon = ['svg', defaultAttributes, [['polygon', {
    points: '7.86 2 16.14 2 22 7.86 22 16.14 16.14 22 7.86 22 2 16.14 2 7.86 7.86 2'
  }]]];
  var Octagon$1 = Octagon;

  var Option = ['svg', defaultAttributes, [['path', {
    d: 'M3 3h6l6 18h6'
  }], ['path', {
    d: 'M14 3h7'
  }]]];
  var Option$1 = Option;

  var Outdent = ['svg', defaultAttributes, [['polyline', {
    points: '7 8 3 12 7 16'
  }], ['line', {
    x1: '21',
    y1: '12',
    x2: '11',
    y2: '12'
  }], ['line', {
    x1: '21',
    y1: '6',
    x2: '11',
    y2: '6'
  }], ['line', {
    x1: '21',
    y1: '18',
    x2: '11',
    y2: '18'
  }]]];
  var Outdent$1 = Outdent;

  var PackageCheck = ['svg', defaultAttributes, [['path', {
    d: 'M16 16l2 2 4-4'
  }], ['path', {
    d: 'M21 10V8a2 2 0 00-1-1.73l-7-4a2 2 0 00-2 0l-7 4A2 2 0 003 8v8a2 2 0 001 1.73l7 4a2 2 0 002 0l2-1.14'
  }], ['path', {
    d: 'M16.5 9.4L7.55 4.24'
  }], ['path', {
    d: 'M3.29 7L12 12m0 0l8.71-5M12 12v10'
  }]]];
  var PackageCheck$1 = PackageCheck;

  var PackageMinus = ['svg', defaultAttributes, [['path', {
    d: 'M16 16h6'
  }], ['path', {
    d: 'M21 10V8a2 2 0 00-1-1.73l-7-4a2 2 0 00-2 0l-7 4A2 2 0 003 8v8a2 2 0 001 1.73l7 4a2 2 0 002 0l2-1.14'
  }], ['path', {
    d: 'M16.5 9.4L7.55 4.24'
  }], ['path', {
    d: 'M3.29 7L12 12m0 0l8.71-5M12 12v10'
  }]]];
  var PackageMinus$1 = PackageMinus;

  var PackagePlus = ['svg', defaultAttributes, [['path', {
    d: 'M16 16h6'
  }], ['path', {
    d: 'M19 13v6'
  }], ['path', {
    d: 'M21 10V8a2 2 0 00-1-1.73l-7-4a2 2 0 00-2 0l-7 4A2 2 0 003 8v8a2 2 0 001 1.73l7 4a2 2 0 002 0l2-1.14'
  }], ['path', {
    d: 'M16.5 9.4L7.55 4.24'
  }], ['path', {
    d: 'M3.29 7L12 12m0 0l8.71-5M12 12v10'
  }]]];
  var PackagePlus$1 = PackagePlus;

  var PackageSearch = ['svg', defaultAttributes, [['path', {
    d: 'M21 10V8a2 2 0 00-1-1.73l-7-4a2 2 0 00-2 0l-7 4A2 2 0 003 8v8a2 2 0 001 1.73l7 4a2 2 0 002 0l2-1.14'
  }], ['path', {
    d: 'M16.5 9.4L7.55 4.24'
  }], ['path', {
    d: 'M12 12v10M3.29 7L12 12 3.29 7zM12 12l8.71-5L12 12z'
  }], ['circle', {
    cx: '18.5',
    cy: '15.5',
    r: '2.5'
  }], ['path', {
    d: 'M20.27 17.27L22 19'
  }]]];
  var PackageSearch$1 = PackageSearch;

  var PackageX = ['svg', defaultAttributes, [['path', {
    d: 'M21 10V8a2 2 0 00-1-1.73l-7-4a2 2 0 00-2 0l-7 4A2 2 0 003 8v8a2 2 0 001 1.73l7 4a2 2 0 002 0l2-1.14'
  }], ['path', {
    d: 'M16.5 9.4L7.55 4.24'
  }], ['path', {
    d: 'M3.29 7L12 12m0 0l8.71-5M12 12v10'
  }], ['path', {
    d: 'M17 13l5 5m-5 0l5-5'
  }]]];
  var PackageX$1 = PackageX;

  var Package = ['svg', defaultAttributes, [['line', {
    x1: '16.5',
    y1: '9.4',
    x2: '7.5',
    y2: '4.21'
  }], ['path', {
    d: 'M21 16V8a2 2 0 00-1-1.73l-7-4a2 2 0 00-2 0l-7 4A2 2 0 003 8v8a2 2 0 001 1.73l7 4a2 2 0 002 0l7-4A2 2 0 0021 16z'
  }], ['polyline', {
    points: '3.27 6.96 12 12.01 20.73 6.96'
  }], ['line', {
    x1: '12',
    y1: '22.08',
    x2: '12',
    y2: '12'
  }]]];
  var Package$1 = Package;

  var Palette = ['svg', defaultAttributes, [['circle', {
    cx: '13.5',
    cy: '6.5',
    r: '.5'
  }], ['circle', {
    cx: '17.5',
    cy: '10.5',
    r: '.5'
  }], ['circle', {
    cx: '8.5',
    cy: '7.5',
    r: '.5'
  }], ['circle', {
    cx: '6.5',
    cy: '12.5',
    r: '.5'
  }], ['path', {
    d: 'M12 2C6.5 2 2 6.5 2 12s4.5 10 10 10c.926 0 1.648-.746 1.648-1.688 0-.437-.18-.835-.437-1.125-.29-.289-.438-.652-.438-1.125a1.64 1.64 0 011.668-1.668h1.996c3.051 0 5.555-2.503 5.555-5.554C21.965 6.012 17.461 2 12 2z'
  }]]];
  var Palette$1 = Palette;

  var Palmtree = ['svg', defaultAttributes, [['path', {
    d: 'M13 8c0-2.76-2.46-5-5.5-5S2 5.24 2 8h2l1-1 1 1h4'
  }], ['path', {
    d: 'M13 7.14A5.82 5.82 0 0116.5 6c3.04 0 5.5 2.24 5.5 5h-3l-1-1-1 1h-3'
  }], ['path', {
    d: 'M5.89 9.71c-2.15 2.15-2.3 5.47-.35 7.43l4.24-4.25.7-.7.71-.71 2.12-2.12c-1.95-1.96-5.27-1.8-7.42.35z'
  }], ['path', {
    d: 'M11 15.5c.5 2.5-.17 4.5-1 6.5h4c2-5.5-.5-12-1-14'
  }]]];
  var Palmtree$1 = Palmtree;

  var Paperclip = ['svg', defaultAttributes, [['path', {
    d: 'M21.44 11.05l-9.19 9.19a6 6 0 01-8.49-8.49l9.19-9.19a4 4 0 015.66 5.66l-9.2 9.19a2 2 0 01-2.83-2.83l8.49-8.48'
  }]]];
  var Paperclip$1 = Paperclip;

  var PauseCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '10',
    y1: '15',
    x2: '10',
    y2: '9'
  }], ['line', {
    x1: '14',
    y1: '15',
    x2: '14',
    y2: '9'
  }]]];
  var PauseCircle$1 = PauseCircle;

  var PauseOctagon = ['svg', defaultAttributes, [['path', {
    d: 'M10 15V9'
  }], ['path', {
    d: 'M14 15V9'
  }], ['path', {
    d: 'M7.714 2h8.572L22 7.714v8.572L16.286 22H7.714L2 16.286V7.714L7.714 2z'
  }]]];
  var PauseOctagon$1 = PauseOctagon;

  var Pause = ['svg', defaultAttributes, [['rect', {
    x: '6',
    y: '4',
    width: '4',
    height: '16'
  }], ['rect', {
    x: '14',
    y: '4',
    width: '4',
    height: '16'
  }]]];
  var Pause$1 = Pause;

  var PenTool = ['svg', defaultAttributes, [['path', {
    d: 'M12 19l7-7 3 3-7 7-3-3z'
  }], ['path', {
    d: 'M18 13l-1.5-7.5L2 2l3.5 14.5L13 18l5-5z'
  }], ['path', {
    d: 'M2 2l7.586 7.586'
  }], ['circle', {
    cx: '11',
    cy: '11',
    r: '2'
  }]]];
  var PenTool$1 = PenTool;

  var Pencil = ['svg', defaultAttributes, [['line', {
    x1: '18',
    y1: '2',
    x2: '22',
    y2: '6'
  }], ['path', {
    d: 'M7.5 20.5L19 9l-4-4L3.5 16.5 2 22z'
  }]]];
  var Pencil$1 = Pencil;

  var Percent = ['svg', defaultAttributes, [['line', {
    x1: '19',
    y1: '5',
    x2: '5',
    y2: '19'
  }], ['circle', {
    cx: '6.5',
    cy: '6.5',
    r: '2.5'
  }], ['circle', {
    cx: '17.5',
    cy: '17.5',
    r: '2.5'
  }]]];
  var Percent$1 = Percent;

  var PersonStanding = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '5',
    r: '1'
  }], ['path', {
    d: 'M9 20l3-6 3 6'
  }], ['path', {
    d: 'M6 8l6 2 6-2'
  }], ['path', {
    d: 'M12 10v4'
  }]]];
  var PersonStanding$1 = PersonStanding;

  var PhoneCall = ['svg', defaultAttributes, [['path', {
    d: 'M14.05 6A5 5 0 0118 10m-3.95-8a9 9 0 018 7.94m0 7v3a2 2 0 01-2 2h-.19a19.79 19.79 0 01-8.63-3.07 19.52 19.52 0 01-6-6 19.82 19.82 0 01-3.11-8.69A2 2 0 013.93 2h3.18a2 2 0 012 1.72 13 13 0 00.7 2.81 2 2 0 01-.45 2.11L8.09 9.91a16 16 0 006 6l1.27-1.27a2 2 0 012.11-.45 13 13 0 002.81.7A2 2 0 0122 16.92z'
  }]]];
  var PhoneCall$1 = PhoneCall;

  var PhoneForwarded = ['svg', defaultAttributes, [['polyline', {
    points: '18 2 22 6 18 10'
  }], ['line', {
    x1: '14',
    y1: '6',
    x2: '22',
    y2: '6'
  }], ['path', {
    d: 'M22 16.92v3a2 2 0 01-2.18 2 19.79 19.79 0 01-8.63-3.07 19.5 19.5 0 01-6-6 19.79 19.79 0 01-3.07-8.67A2 2 0 014.11 2h3a2 2 0 012 1.72 12.84 12.84 0 00.7 2.81 2 2 0 01-.45 2.11L8.09 9.91a16 16 0 006 6l1.27-1.27a2 2 0 012.11-.45 12.84 12.84 0 002.81.7A2 2 0 0122 16.92z'
  }]]];
  var PhoneForwarded$1 = PhoneForwarded;

  var PhoneIncoming = ['svg', defaultAttributes, [['polyline', {
    points: '16 2 16 8 22 8'
  }], ['line', {
    x1: '22',
    y1: '2',
    x2: '16',
    y2: '8'
  }], ['path', {
    d: 'M22 16.92v3a2 2 0 01-2.18 2 19.79 19.79 0 01-8.63-3.07 19.5 19.5 0 01-6-6 19.79 19.79 0 01-3.07-8.67A2 2 0 014.11 2h3a2 2 0 012 1.72 12.84 12.84 0 00.7 2.81 2 2 0 01-.45 2.11L8.09 9.91a16 16 0 006 6l1.27-1.27a2 2 0 012.11-.45 12.84 12.84 0 002.81.7A2 2 0 0122 16.92z'
  }]]];
  var PhoneIncoming$1 = PhoneIncoming;

  var PhoneMissed = ['svg', defaultAttributes, [['line', {
    x1: '22',
    y1: '2',
    x2: '16',
    y2: '8'
  }], ['line', {
    x1: '16',
    y1: '2',
    x2: '22',
    y2: '8'
  }], ['path', {
    d: 'M22 16.92v3a2 2 0 01-2.18 2 19.79 19.79 0 01-8.63-3.07 19.5 19.5 0 01-6-6 19.79 19.79 0 01-3.07-8.67A2 2 0 014.11 2h3a2 2 0 012 1.72 12.84 12.84 0 00.7 2.81 2 2 0 01-.45 2.11L8.09 9.91a16 16 0 006 6l1.27-1.27a2 2 0 012.11-.45 12.84 12.84 0 002.81.7A2 2 0 0122 16.92z'
  }]]];
  var PhoneMissed$1 = PhoneMissed;

  var PhoneOff = ['svg', defaultAttributes, [['path', {
    d: 'M10.68 13.31a16 16 0 003.41 2.6l1.27-1.27a2 2 0 012.11-.45 12.84 12.84 0 002.81.7 2 2 0 011.72 2v3a2 2 0 01-2.18 2 19.79 19.79 0 01-8.63-3.07 19.42 19.42 0 01-3.33-2.67m-2.67-3.34a19.79 19.79 0 01-3.07-8.63A2 2 0 014.11 2h3a2 2 0 012 1.72 12.84 12.84 0 00.7 2.81 2 2 0 01-.45 2.11L8.09 9.91'
  }], ['line', {
    x1: '22',
    y1: '2',
    x2: '2',
    y2: '22'
  }]]];
  var PhoneOff$1 = PhoneOff;

  var PhoneOutgoing = ['svg', defaultAttributes, [['polyline', {
    points: '22 8 22 2 16 2'
  }], ['line', {
    x1: '16',
    y1: '8',
    x2: '22',
    y2: '2'
  }], ['path', {
    d: 'M22 16.92v3a2 2 0 01-2.18 2 19.79 19.79 0 01-8.63-3.07 19.5 19.5 0 01-6-6 19.79 19.79 0 01-3.07-8.67A2 2 0 014.11 2h3a2 2 0 012 1.72 12.84 12.84 0 00.7 2.81 2 2 0 01-.45 2.11L8.09 9.91a16 16 0 006 6l1.27-1.27a2 2 0 012.11-.45 12.84 12.84 0 002.81.7A2 2 0 0122 16.92z'
  }]]];
  var PhoneOutgoing$1 = PhoneOutgoing;

  var Phone = ['svg', defaultAttributes, [['path', {
    d: 'M22 16.92v3a2 2 0 01-2.18 2 19.79 19.79 0 01-8.63-3.07 19.5 19.5 0 01-6-6 19.79 19.79 0 01-3.07-8.67A2 2 0 014.11 2h3a2 2 0 012 1.72 12.84 12.84 0 00.7 2.81 2 2 0 01-.45 2.11L8.09 9.91a16 16 0 006 6l1.27-1.27a2 2 0 012.11-.45 12.84 12.84 0 002.81.7A2 2 0 0122 16.92z'
  }]]];
  var Phone$1 = Phone;

  var PieChart = ['svg', defaultAttributes, [['path', {
    d: 'M21.21 15.89A10 10 0 118 2.83'
  }], ['path', {
    d: 'M22 12A10 10 0 0012 2v10z'
  }]]];
  var PieChart$1 = PieChart;

  var PiggyBank = ['svg', defaultAttributes, [['path', {
    d: 'M19 5c-1.5 0-2.8 1.4-3 2-3.5-1.5-11-.3-11 5 0 1.8 0 3 2 4.5V20h4v-2h3v2h4v-4c1-.5 1.7-1 2-2h2v-4h-2c0-1-.5-1.5-1-2h0V5z'
  }], ['path', {
    d: 'M2 9v1c0 1.1.9 2 2 2h1'
  }], ['path', {
    d: 'M16 11h0'
  }]]];
  var PiggyBank$1 = PiggyBank;

  var Pin = ['svg', defaultAttributes, [['path', {
    d: 'M12 21v-7'
  }], ['path', {
    d: 'M9 3v5l-2 4v2h10m0 0h-5m5 0v-2l-2-4V3'
  }], ['path', {
    d: 'M7 3h10'
  }]]];
  var Pin$1 = Pin;

  var Pipette = ['svg', defaultAttributes, [['path', {
    d: 'M2 22l2-2h3l7-7'
  }], ['path', {
    d: 'M4 20v-3l7-7'
  }], ['path', {
    d: 'M14.29 13.3a1 1 0 001.41 0l.8-.8c.27-.27.27-.72 0-1s-.28-.72 0-1l4.08-4.08a2 2 0 000-2.83l-.17-.17a2 2 0 00-2.83 0L13.5 7.51c-.28.27-.73.27-1 0s-.73-.28-1 0l-.8.79a1 1 0 000 1.41l3.59 3.59z'
  }]]];
  var Pipette$1 = Pipette;

  var Plane = ['svg', defaultAttributes, [['path', {
    d: 'M17.8 19.2L16 11l3.5-3.5C21 6 21.5 4 21 3c-1-.5-3 0-4.5 1.5L13 8 4.8 6.2c-.5-.1-.9.1-1.1.5l-.3.5c-.2.5-.1 1 .3 1.3L9 12l-2 3H4l-1 1 3 2 2 3 1-1v-3l3-2 3.5 5.3c.3.4.8.5 1.3.3l.5-.2c.4-.3.6-.7.5-1.2z'
  }]]];
  var Plane$1 = Plane;

  var PlayCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['polygon', {
    points: '10 8 16 12 10 16 10 8'
  }]]];
  var PlayCircle$1 = PlayCircle;

  var Play = ['svg', defaultAttributes, [['polygon', {
    points: '5 3 19 12 5 21 5 3'
  }]]];
  var Play$1 = Play;

  var PlugZap = ['svg', defaultAttributes, [['path', {
    d: 'M13 2l-2 2.5h3L12 7'
  }], ['path', {
    d: 'M12 22v-3'
  }], ['path', {
    d: 'M10 13v-2.5'
  }], ['path', {
    d: 'M10 12.5v-2'
  }], ['path', {
    d: 'M14 12.5v-2'
  }], ['path', {
    d: 'M16 15a2 2 0 00-2-2h-4a2 2 0 00-2 2v2a2 2 0 002 2h4a2 2 0 002-2v-2z'
  }]]];
  var PlugZap$1 = PlugZap;

  var PlusCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '12',
    y1: '8',
    x2: '12',
    y2: '16'
  }], ['line', {
    x1: '8',
    y1: '12',
    x2: '16',
    y2: '12'
  }]]];
  var PlusCircle$1 = PlusCircle;

  var PlusSquare = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '12',
    y1: '8',
    x2: '12',
    y2: '16'
  }], ['line', {
    x1: '8',
    y1: '12',
    x2: '16',
    y2: '12'
  }]]];
  var PlusSquare$1 = PlusSquare;

  var Plus = ['svg', defaultAttributes, [['line', {
    x1: '12',
    y1: '5',
    x2: '12',
    y2: '19'
  }], ['line', {
    x1: '5',
    y1: '12',
    x2: '19',
    y2: '12'
  }]]];
  var Plus$1 = Plus;

  var Pocket = ['svg', defaultAttributes, [['path', {
    d: 'M4 3h16a2 2 0 012 2v6a10 10 0 01-10 10A10 10 0 012 11V5a2 2 0 012-2z'
  }], ['polyline', {
    points: '8 10 12 14 16 10'
  }]]];
  var Pocket$1 = Pocket;

  var Podcast = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '11',
    r: '1'
  }], ['path', {
    d: 'M17.03 18.46a9 9 0 10-10.02.03'
  }], ['path', {
    d: 'M16.06 13.91a5 5 0 10-7.97.2'
  }], ['path', {
    d: 'M11.11 17a.9.9 0 111.78 0l-.52 4.67a.37.37 0 01-.74 0l-.52-4.68z'
  }]]];
  var Podcast$1 = Podcast;

  var Pointer = ['svg', defaultAttributes, [['path', {
    d: 'M22 14a8 8 0 01-8 8'
  }], ['path', {
    d: 'M18 11v-1a2 2 0 00-2-2v0a2 2 0 00-2 2v0'
  }], ['path', {
    d: 'M14 10V9a2 2 0 00-2-2v0a2 2 0 00-2 2v1'
  }], ['path', {
    d: 'M10 9.5V4a2 2 0 00-2-2v0a2 2 0 00-2 2v10'
  }], ['path', {
    d: 'M18 11a2 2 0 114 0v3a8 8 0 01-8 8h-2c-2.8 0-4.5-.86-5.99-2.34l-3.6-3.6a2 2 0 012.83-2.82L7 15'
  }]]];
  var Pointer$1 = Pointer;

  var PoundSterling = ['svg', defaultAttributes, [['path', {
    d: 'M18 7c0-5.333-8-5.333-8 0'
  }], ['path', {
    d: 'M10 7v14'
  }], ['path', {
    d: 'M6 21h12'
  }], ['path', {
    d: 'M6 13h10'
  }]]];
  var PoundSterling$1 = PoundSterling;

  var PowerOff = ['svg', defaultAttributes, [['path', {
    d: 'M18.36 6.64A9 9 0 0120.77 15M5.63 6.64A9 9 0 1018.706 19'
  }], ['path', {
    d: 'M12 2v4'
  }], ['path', {
    d: 'M2 2l20 20'
  }]]];
  var PowerOff$1 = PowerOff;

  var Power = ['svg', defaultAttributes, [['path', {
    d: 'M18.36 6.64a9 9 0 11-12.73 0'
  }], ['line', {
    x1: '12',
    y1: '2',
    x2: '12',
    y2: '12'
  }]]];
  var Power$1 = Power;

  var Printer = ['svg', defaultAttributes, [['polyline', {
    points: '6 9 6 2 18 2 18 9'
  }], ['path', {
    d: 'M6 18H4a2 2 0 01-2-2v-5a2 2 0 012-2h16a2 2 0 012 2v5a2 2 0 01-2 2h-2'
  }], ['rect', {
    x: '6',
    y: '14',
    width: '12',
    height: '8'
  }]]];
  var Printer$1 = Printer;

  var QrCode = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '2',
    width: '8',
    height: '8'
  }], ['path', {
    d: 'M6 6h.01'
  }], ['rect', {
    x: '14',
    y: '2',
    width: '8',
    height: '8'
  }], ['path', {
    d: 'M18 6h.01'
  }], ['rect', {
    x: '2',
    y: '14',
    width: '8',
    height: '8'
  }], ['path', {
    d: 'M6 18h.01'
  }], ['path', {
    d: 'M14 14h.01'
  }], ['path', {
    d: 'M18 18h.01'
  }], ['path', {
    d: 'M18 22h4v-4'
  }], ['path', {
    d: 'M14 18v4'
  }], ['path', {
    d: 'M22 14h-4'
  }]]];
  var QrCode$1 = QrCode;

  var Quote = ['svg', defaultAttributes, [['path', {
    d: 'M3 21c3 0 7-1 7-8V5c0-1.25-.756-2.017-2-2H4c-1.25 0-2 .75-2 1.972V11c0 1.25.75 2 2 2 1 0 1 0 1 1v1c0 1-1 2-2 2s-1 .008-1 1.031V20c0 1 0 1 1 1z'
  }], ['path', {
    d: 'M15 21c3 0 7-1 7-8V5c0-1.25-.757-2.017-2-2h-4c-1.25 0-2 .75-2 1.972V11c0 1.25.75 2 2 2h.75c0 2.25.25 4-2.75 4v3c0 1 0 1 1 1z'
  }]]];
  var Quote$1 = Quote;

  var RadioReceiver = ['svg', defaultAttributes, [['path', {
    d: 'M5 16v2'
  }], ['path', {
    d: 'M19 16v2'
  }], ['rect', {
    x: '2',
    y: '8',
    width: '20',
    height: '8',
    rx: '2'
  }], ['path', {
    d: 'M18 12h0'
  }]]];
  var RadioReceiver$1 = RadioReceiver;

  var Radio = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '2'
  }], ['path', {
    d: 'M16.24 7.76a6 6 0 010 8.49m-8.48-.01a6 6 0 010-8.49m11.31-2.82a10 10 0 010 14.14m-14.14 0a10 10 0 010-14.14'
  }]]];
  var Radio$1 = Radio;

  var Redo = ['svg', defaultAttributes, [['path', {
    d: 'M21 7v6h-6'
  }], ['path', {
    d: 'M3 17a9 9 0 019-9 9 9 0 016 2.3l3 2.7'
  }]]];
  var Redo$1 = Redo;

  var RefreshCcw = ['svg', defaultAttributes, [['path', {
    d: 'M3 2v6h6'
  }], ['path', {
    d: 'M21 12A9 9 0 006 5.3L3 8'
  }], ['path', {
    d: 'M21 22v-6h-6'
  }], ['path', {
    d: 'M3 12a9 9 0 0015 6.7l3-2.7'
  }]]];
  var RefreshCcw$1 = RefreshCcw;

  var RefreshCw = ['svg', defaultAttributes, [['path', {
    d: 'M21 2v6h-6'
  }], ['path', {
    d: 'M3 12a9 9 0 0115-6.7L21 8'
  }], ['path', {
    d: 'M3 22v-6h6'
  }], ['path', {
    d: 'M21 12a9 9 0 01-15 6.7L3 16'
  }]]];
  var RefreshCw$1 = RefreshCw;

  var Regex = ['svg', defaultAttributes, [['path', {
    d: 'M17 3v10'
  }], ['path', {
    d: 'M12.67 5.5l8.66 5'
  }], ['path', {
    d: 'M12.67 10.5l8.66-5'
  }], ['path', {
    d: 'M9 17a2 2 0 00-2-2H5a2 2 0 00-2 2v2a2 2 0 002 2h2a2 2 0 002-2v-2z'
  }]]];
  var Regex$1 = Regex;

  var Repeat1 = ['svg', defaultAttributes, [['path', {
    d: 'M17 2l4 4-4 4'
  }], ['path', {
    d: 'M3 11v-1a4 4 0 014-4h14'
  }], ['path', {
    d: 'M7 22l-4-4 4-4'
  }], ['path', {
    d: 'M21 13v1a4 4 0 01-4 4H3'
  }], ['path', {
    d: 'M11 10h1v4'
  }]]];
  var Repeat1$1 = Repeat1;

  var Repeat = ['svg', defaultAttributes, [['path', {
    d: 'M17 2l4 4-4 4'
  }], ['path', {
    d: 'M3 11v-1a4 4 0 014-4h14'
  }], ['path', {
    d: 'M7 22l-4-4 4-4'
  }], ['path', {
    d: 'M21 13v1a4 4 0 01-4 4H3'
  }]]];
  var Repeat$1 = Repeat;

  var ReplyAll = ['svg', defaultAttributes, [['polyline', {
    points: '7 17 2 12 7 7'
  }], ['polyline', {
    points: '12 17 7 12 12 7'
  }], ['path', {
    d: 'M22 18v-2a4 4 0 00-4-4H7'
  }]]];
  var ReplyAll$1 = ReplyAll;

  var Reply = ['svg', defaultAttributes, [['polyline', {
    points: '9 17 4 12 9 7'
  }], ['path', {
    d: 'M20 18v-2a4 4 0 00-4-4H4'
  }]]];
  var Reply$1 = Reply;

  var Rewind = ['svg', defaultAttributes, [['polygon', {
    points: '11 19 2 12 11 5 11 19'
  }], ['polygon', {
    points: '22 19 13 12 22 5 22 19'
  }]]];
  var Rewind$1 = Rewind;

  var Rocket = ['svg', defaultAttributes, [['path', {
    d: 'M4.5 16.5c-1.5 1.26-2 5-2 5s3.74-.5 5-2c.71-.84.7-2.13-.09-2.91a2.18 2.18 0 00-2.91-.09z'
  }], ['path', {
    d: 'M12 15l-3-3a22 22 0 012-3.95A12.88 12.88 0 0122 2c0 2.72-.78 7.5-6 11a22.35 22.35 0 01-4 2z'
  }], ['path', {
    d: 'M9 12H4s.55-3.03 2-4c1.62-1.08 5 0 5 0'
  }], ['path', {
    d: 'M12 15v5s3.03-.55 4-2c1.08-1.62 0-5 0-5'
  }]]];
  var Rocket$1 = Rocket;

  var RockingChair = ['svg', defaultAttributes, [['polyline', {
    points: '3.5 2 6.5 12.5 18 12.5'
  }], ['line', {
    x1: '9.5',
    y1: '12.5',
    x2: '5.5',
    y2: '20'
  }], ['line', {
    x1: '15',
    y1: '12.5',
    x2: '18.5',
    y2: '20'
  }], ['path', {
    d: 'M2.75 18a13 13 0 0018.5 0'
  }]]];
  var RockingChair$1 = RockingChair;

  var RotateCcw = ['svg', defaultAttributes, [['path', {
    d: 'M3 2v6h6'
  }], ['path', {
    d: 'M3 13a9 9 0 103-7.7L3 8'
  }]]];
  var RotateCcw$1 = RotateCcw;

  var RotateCw = ['svg', defaultAttributes, [['path', {
    d: 'M21 2v6h-6'
  }], ['path', {
    d: 'M21 13a9 9 0 11-3-7.7L21 8'
  }]]];
  var RotateCw$1 = RotateCw;

  var Rss = ['svg', defaultAttributes, [['path', {
    d: 'M4 11a9 9 0 019 9'
  }], ['path', {
    d: 'M4 4a16 16 0 0116 16'
  }], ['circle', {
    cx: '5',
    cy: '19',
    r: '1'
  }]]];
  var Rss$1 = Rss;

  var Ruler = ['svg', defaultAttributes, [['path', {
    d: 'M16 2l6 6L8 22l-6-6L16 2'
  }], ['path', {
    d: 'M7.5 10.5l2 2'
  }], ['path', {
    d: 'M10.5 7.5l2 2'
  }], ['path', {
    d: 'M13.5 4.5l2 2'
  }], ['path', {
    d: 'M4.5 13.5l2 2'
  }]]];
  var Ruler$1 = Ruler;

  var RussianRuble = ['svg', defaultAttributes, [['path', {
    d: 'M14 11c5.333 0 5.333-8 0-8'
  }], ['path', {
    d: 'M6 11h8'
  }], ['path', {
    d: 'M6 15h8'
  }], ['path', {
    d: 'M9 21V3'
  }], ['path', {
    d: 'M9 3h5'
  }]]];
  var RussianRuble$1 = RussianRuble;

  var Save = ['svg', defaultAttributes, [['path', {
    d: 'M19 21H5a2 2 0 01-2-2V5a2 2 0 012-2h11l5 5v11a2 2 0 01-2 2z'
  }], ['polyline', {
    points: '17 21 17 13 7 13 7 21'
  }], ['polyline', {
    points: '7 3 7 8 15 8'
  }]]];
  var Save$1 = Save;

  var Scale = ['svg', defaultAttributes, [['path', {
    d: 'M16 16l3-8 3.001 8A5.002 5.002 0 0116 16z'
  }], ['path', {
    d: 'M2 16l3-8 3.001 8A5.002 5.002 0 012 16z'
  }], ['path', {
    d: 'M7 21h10'
  }], ['path', {
    d: 'M12 3v18'
  }], ['path', {
    d: 'M3 7h2c2 0 5-1 7-2 2 1 5 2 7 2h2'
  }]]];
  var Scale$1 = Scale;

  var ScanLine = ['svg', defaultAttributes, [['path', {
    d: 'M3 7V5a2 2 0 012-2h2'
  }], ['path', {
    d: 'M17 3h2a2 2 0 012 2v2'
  }], ['path', {
    d: 'M21 17v2a2 2 0 01-2 2h-2'
  }], ['path', {
    d: 'M7 21H5a2 2 0 01-2-2v-2'
  }], ['line', {
    x1: '7',
    y1: '12',
    x2: '17',
    y2: '12'
  }]]];
  var ScanLine$1 = ScanLine;

  var Scan = ['svg', defaultAttributes, [['path', {
    d: 'M3 7V5a2 2 0 012-2h2'
  }], ['path', {
    d: 'M17 3h2a2 2 0 012 2v2'
  }], ['path', {
    d: 'M21 17v2a2 2 0 01-2 2h-2'
  }], ['path', {
    d: 'M7 21H5a2 2 0 01-2-2v-2'
  }]]];
  var Scan$1 = Scan;

  var Scissors = ['svg', defaultAttributes, [['circle', {
    cx: '6',
    cy: '6',
    r: '3'
  }], ['circle', {
    cx: '6',
    cy: '18',
    r: '3'
  }], ['line', {
    x1: '20',
    y1: '4',
    x2: '8.12',
    y2: '15.88'
  }], ['line', {
    x1: '14.47',
    y1: '14.48',
    x2: '20',
    y2: '20'
  }], ['line', {
    x1: '8.12',
    y1: '8.12',
    x2: '12',
    y2: '12'
  }]]];
  var Scissors$1 = Scissors;

  var ScreenShareOff = ['svg', defaultAttributes, [['path', {
    d: 'M13 3H4a2 2 0 00-2 2v10a2 2 0 002 2h16a2 2 0 002-2v-3'
  }], ['path', {
    d: 'M8 21h8'
  }], ['path', {
    d: 'M12 17v4'
  }], ['path', {
    d: 'M22 3l-5 5'
  }], ['path', {
    d: 'M17 3l5 5'
  }]]];
  var ScreenShareOff$1 = ScreenShareOff;

  var ScreenShare = ['svg', defaultAttributes, [['path', {
    d: 'M13 3H4a2 2 0 00-2 2v10a2 2 0 002 2h16a2 2 0 002-2v-3'
  }], ['path', {
    d: 'M8 21h8'
  }], ['path', {
    d: 'M12 17v4'
  }], ['path', {
    d: 'M17 8l5-5'
  }], ['path', {
    d: 'M17 3h5v5'
  }]]];
  var ScreenShare$1 = ScreenShare;

  var Search = ['svg', defaultAttributes, [['circle', {
    cx: '11',
    cy: '11',
    r: '8'
  }], ['line', {
    x1: '21',
    y1: '21',
    x2: '16.65',
    y2: '16.65'
  }]]];
  var Search$1 = Search;

  var Send = ['svg', defaultAttributes, [['line', {
    x1: '22',
    y1: '2',
    x2: '11',
    y2: '13'
  }], ['polygon', {
    points: '22 2 15 22 11 13 2 9 22 2'
  }]]];
  var Send$1 = Send;

  var SeparatorHorizontal = ['svg', defaultAttributes, [['line', {
    x1: '3',
    y1: '12',
    x2: '21',
    y2: '12'
  }], ['polyline', {
    points: '8 8 12 4 16 8'
  }], ['polyline', {
    points: '16 16 12 20 8 16'
  }]]];
  var SeparatorHorizontal$1 = SeparatorHorizontal;

  var SeparatorVertical = ['svg', defaultAttributes, [['line', {
    x1: '12',
    y1: '3',
    x2: '12',
    y2: '21'
  }], ['polyline', {
    points: '8 8 4 12 8 16'
  }], ['polyline', {
    points: '16 16 20 12 16 8'
  }]]];
  var SeparatorVertical$1 = SeparatorVertical;

  var ServerCrash = ['svg', defaultAttributes, [['path', {
    d: 'M6 10H4a2 2 0 01-2-2V4a2 2 0 012-2h16a2 2 0 012 2v4a2 2 0 01-2 2h-2'
  }], ['path', {
    d: 'M6 14H4a2 2 0 00-2 2v4a2 2 0 002 2h16a2 2 0 002-2v-4a2 2 0 00-2-2h-2'
  }], ['path', {
    d: 'M6 6h.01'
  }], ['path', {
    d: 'M6 18h.01'
  }], ['path', {
    d: 'M13 6l-4 6h6l-4 6'
  }]]];
  var ServerCrash$1 = ServerCrash;

  var ServerOff = ['svg', defaultAttributes, [['path', {
    d: 'M7 2h13a2 2 0 012 2v4a2 2 0 01-2 2h-5'
  }], ['path', {
    d: 'M10 10L2.5 2.5C2 2 2 2.5 2 5v3a2 2 0 002 2h6z'
  }], ['path', {
    d: 'M22 17v-1a2 2 0 00-2-2h-1'
  }], ['path', {
    d: 'M4 14a2 2 0 00-2 2v4a2 2 0 002 2h16.5l1-.5.5.5-8-8H4z'
  }], ['path', {
    d: 'M6 18h.01'
  }], ['path', {
    d: 'M2 2l20 20'
  }]]];
  var ServerOff$1 = ServerOff;

  var Server = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '2',
    width: '20',
    height: '8',
    rx: '2',
    ry: '2'
  }], ['rect', {
    x: '2',
    y: '14',
    width: '20',
    height: '8',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '6',
    y1: '6',
    x2: '6.01',
    y2: '6'
  }], ['line', {
    x1: '6',
    y1: '18',
    x2: '6.01',
    y2: '18'
  }]]];
  var Server$1 = Server;

  var Settings2 = ['svg', defaultAttributes, [['path', {
    d: 'M20 7h-9'
  }], ['path', {
    d: 'M14 17H5'
  }], ['circle', {
    cx: '17',
    cy: '17',
    r: '3'
  }], ['circle', {
    cx: '7',
    cy: '7',
    r: '3'
  }]]];
  var Settings2$1 = Settings2;

  var Settings = ['svg', defaultAttributes, [['path', {
    d: 'M12.22 2h-.44a2 2 0 00-2 2v.18a2 2 0 01-1 1.73l-.43.25a2 2 0 01-2 0l-.15-.08a2 2 0 00-2.73.73l-.22.38a2 2 0 00.73 2.73l.15.1a2 2 0 011 1.72v.51a2 2 0 01-1 1.74l-.15.09a2 2 0 00-.73 2.73l.22.38a2 2 0 002.73.73l.15-.08a2 2 0 012 0l.43.25a2 2 0 011 1.73V20a2 2 0 002 2h.44a2 2 0 002-2v-.18a2 2 0 011-1.73l.43-.25a2 2 0 012 0l.15.08a2 2 0 002.73-.73l.22-.39a2 2 0 00-.73-2.73l-.15-.08a2 2 0 01-1-1.74v-.5a2 2 0 011-1.74l.15-.09a2 2 0 00.73-2.73l-.22-.38a2 2 0 00-2.73-.73l-.15.08a2 2 0 01-2 0l-.43-.25a2 2 0 01-1-1.73V4a2 2 0 00-2-2z'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '3'
  }]]];
  var Settings$1 = Settings;

  var Share2 = ['svg', defaultAttributes, [['circle', {
    cx: '18',
    cy: '5',
    r: '3'
  }], ['circle', {
    cx: '6',
    cy: '12',
    r: '3'
  }], ['circle', {
    cx: '18',
    cy: '19',
    r: '3'
  }], ['line', {
    x1: '8.59',
    y1: '13.51',
    x2: '15.42',
    y2: '17.49'
  }], ['line', {
    x1: '15.41',
    y1: '6.51',
    x2: '8.59',
    y2: '10.49'
  }]]];
  var Share2$1 = Share2;

  var Share = ['svg', defaultAttributes, [['path', {
    d: 'M4 12v8a2 2 0 002 2h12a2 2 0 002-2v-8'
  }], ['polyline', {
    points: '16 6 12 2 8 6'
  }], ['line', {
    x1: '12',
    y1: '2',
    x2: '12',
    y2: '15'
  }]]];
  var Share$1 = Share;

  var Sheet = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '3',
    y1: '9',
    x2: '21',
    y2: '9'
  }], ['line', {
    x1: '3',
    y1: '15',
    x2: '21',
    y2: '15'
  }], ['line', {
    x1: '9',
    y1: '9',
    x2: '9',
    y2: '21'
  }], ['line', {
    x1: '15',
    y1: '9',
    x2: '15',
    y2: '21'
  }]]];
  var Sheet$1 = Sheet;

  var ShieldAlert = ['svg', defaultAttributes, [['path', {
    d: 'M12 22s8-4 8-10V5l-8-3-8 3v7c0 6 8 10 8 10z'
  }], ['path', {
    d: 'M12 8v4'
  }], ['path', {
    d: 'M12 16h.01'
  }]]];
  var ShieldAlert$1 = ShieldAlert;

  var ShieldCheck = ['svg', defaultAttributes, [['path', {
    d: 'M12 22s8-4 8-10V5l-8-3-8 3v7c0 6 8 10 8 10z'
  }], ['path', {
    d: 'M9 12l2 2 4-4'
  }]]];
  var ShieldCheck$1 = ShieldCheck;

  var ShieldClose = ['svg', defaultAttributes, [['path', {
    d: 'M12 22s8-4 8-10V5l-8-3-8 3v7c0 6 8 10 8 10z'
  }], ['line', {
    x1: '9.5',
    y1: '9',
    x2: '14.5',
    y2: '14'
  }], ['line', {
    x1: '14.5',
    y1: '9',
    x2: '9.5',
    y2: '14'
  }]]];
  var ShieldClose$1 = ShieldClose;

  var ShieldOff = ['svg', defaultAttributes, [['path', {
    d: 'M19.69 14a6.9 6.9 0 00.31-2V5l-8-3-3.16 1.18'
  }], ['path', {
    d: 'M4.73 4.73L4 5v7c0 6 8 10 8 10a20.29 20.29 0 005.62-4.38'
  }], ['line', {
    x1: '1',
    y1: '1',
    x2: '23',
    y2: '23'
  }]]];
  var ShieldOff$1 = ShieldOff;

  var Shield = ['svg', defaultAttributes, [['path', {
    d: 'M12 22s8-4 8-10V5l-8-3-8 3v7c0 6 8 10 8 10z'
  }]]];
  var Shield$1 = Shield;

  var Shirt = ['svg', defaultAttributes, [['path', {
    d: 'M20.38 3.46L16 2a4 4 0 01-8 0L3.62 3.46a2 2 0 00-1.34 2.23l.58 3.47a1 1 0 00.99.84H6v10c0 1.1.9 2 2 2h8a2 2 0 002-2V10h2.15a1 1 0 00.99-.84l.58-3.47a2 2 0 00-1.34-2.23z'
  }]]];
  var Shirt$1 = Shirt;

  var ShoppingBag = ['svg', defaultAttributes, [['path', {
    d: 'M6 2L3 6v14a2 2 0 002 2h14a2 2 0 002-2V6l-3-4z'
  }], ['line', {
    x1: '3',
    y1: '6',
    x2: '21',
    y2: '6'
  }], ['path', {
    d: 'M16 10a4 4 0 01-8 0'
  }]]];
  var ShoppingBag$1 = ShoppingBag;

  var ShoppingCart = ['svg', defaultAttributes, [['circle', {
    cx: '9',
    cy: '21',
    r: '1'
  }], ['circle', {
    cx: '20',
    cy: '21',
    r: '1'
  }], ['path', {
    d: 'M1 1h4l2.68 13.39a2 2 0 002 1.61h9.72a2 2 0 002-1.61L23 6H6'
  }]]];
  var ShoppingCart$1 = ShoppingCart;

  var Shovel = ['svg', defaultAttributes, [['path', {
    d: 'M2 22v-5l5-5 5 5-5 5z'
  }], ['path', {
    d: 'M9.5 14.5L16 8'
  }], ['path', {
    d: 'M17 2l5 5-.5.5a3.53 3.53 0 01-5 0s0 0 0 0a3.53 3.53 0 010-5L17 2'
  }]]];
  var Shovel$1 = Shovel;

  var Shrink = ['svg', defaultAttributes, [['path', {
    d: 'M15 15l6 6m-6-6v4.8m0-4.8h4.8'
  }], ['path', {
    d: 'M9 19.8V15m0 0H4.2M9 15l-6 6'
  }], ['path', {
    d: 'M15 4.2V9m0 0h4.8M15 9l6-6'
  }], ['path', {
    d: 'M9 4.2V9m0 0H4.2M9 9L3 3'
  }]]];
  var Shrink$1 = Shrink;

  var Shuffle = ['svg', defaultAttributes, [['polyline', {
    points: '16 3 21 3 21 8'
  }], ['line', {
    x1: '4',
    y1: '20',
    x2: '21',
    y2: '3'
  }], ['polyline', {
    points: '21 16 21 21 16 21'
  }], ['line', {
    x1: '15',
    y1: '15',
    x2: '21',
    y2: '21'
  }], ['line', {
    x1: '4',
    y1: '4',
    x2: '9',
    y2: '9'
  }]]];
  var Shuffle$1 = Shuffle;

  var SidebarClose = ['svg', defaultAttributes, [['path', {
    d: 'M19 3H5a2 2 0 00-2 2v14a2 2 0 002 2h14a2 2 0 002-2V5a2 2 0 00-2-2z'
  }], ['path', {
    d: 'M9 3v18'
  }], ['path', {
    d: 'M16 15l-3-3 3-3'
  }]]];
  var SidebarClose$1 = SidebarClose;

  var SidebarOpen = ['svg', defaultAttributes, [['path', {
    d: 'M19 3H5a2 2 0 00-2 2v14a2 2 0 002 2h14a2 2 0 002-2V5a2 2 0 00-2-2z'
  }], ['path', {
    d: 'M9 3v18'
  }], ['path', {
    d: 'M14 9l3 3-3 3'
  }]]];
  var SidebarOpen$1 = SidebarOpen;

  var Sidebar = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '9',
    y1: '3',
    x2: '9',
    y2: '21'
  }]]];
  var Sidebar$1 = Sidebar;

  var Sigma = ['svg', defaultAttributes, [['path', {
    d: 'M18 7V4H6l6 8-6 8h12v-3'
  }]]];
  var Sigma$1 = Sigma;

  var SignalHigh = ['svg', defaultAttributes, [['path', {
    d: 'M2 20h.01'
  }], ['path', {
    d: 'M7 20v-4'
  }], ['path', {
    d: 'M12 20v-8'
  }], ['path', {
    d: 'M17 20V8'
  }]]];
  var SignalHigh$1 = SignalHigh;

  var SignalLow = ['svg', defaultAttributes, [['path', {
    d: 'M2 20h.01'
  }], ['path', {
    d: 'M7 20v-4'
  }]]];
  var SignalLow$1 = SignalLow;

  var SignalMedium = ['svg', defaultAttributes, [['path', {
    d: 'M2 20h.01'
  }], ['path', {
    d: 'M7 20v-4'
  }], ['path', {
    d: 'M12 20v-8'
  }]]];
  var SignalMedium$1 = SignalMedium;

  var SignalZero = ['svg', defaultAttributes, [['path', {
    d: 'M2 20h.01'
  }], ['path', {
    d: 'M7 20v-4'
  }]]];
  var SignalZero$1 = SignalZero;

  var Signal = ['svg', defaultAttributes, [['path', {
    d: 'M2 20h.01'
  }], ['path', {
    d: 'M7 20v-4'
  }], ['path', {
    d: 'M12 20v-8'
  }], ['path', {
    d: 'M17 20V8'
  }], ['path', {
    d: 'M22 4v16'
  }]]];
  var Signal$1 = Signal;

  var SkipBack = ['svg', defaultAttributes, [['polygon', {
    points: '19 20 9 12 19 4 19 20'
  }], ['line', {
    x1: '5',
    y1: '19',
    x2: '5',
    y2: '5'
  }]]];
  var SkipBack$1 = SkipBack;

  var SkipForward = ['svg', defaultAttributes, [['polygon', {
    points: '5 4 15 12 5 20 5 4'
  }], ['line', {
    x1: '19',
    y1: '5',
    x2: '19',
    y2: '19'
  }]]];
  var SkipForward$1 = SkipForward;

  var Skull = ['svg', defaultAttributes, [['circle', {
    cx: '9',
    cy: '12',
    r: '1'
  }], ['circle', {
    cx: '15',
    cy: '12',
    r: '1'
  }], ['path', {
    d: 'M8 20v2h8v-2'
  }], ['path', {
    d: 'M12.5 17l-.5-1-.5 1h1z'
  }], ['path', {
    d: 'M16 20a2 2 0 001.56-3.25 8 8 0 10-11.12 0A2 2 0 008 20'
  }]]];
  var Skull$1 = Skull;

  var Slack = ['svg', defaultAttributes, [['rect', {
    x: '13',
    y: '2',
    width: '3',
    height: '8',
    rx: '1.5'
  }], ['path', {
    d: 'M19 8.5V10h1.5A1.5 1.5 0 1019 8.5'
  }], ['rect', {
    x: '8',
    y: '14',
    width: '3',
    height: '8',
    rx: '1.5'
  }], ['path', {
    d: 'M5 15.5V14H3.5A1.5 1.5 0 105 15.5'
  }], ['rect', {
    x: '14',
    y: '13',
    width: '8',
    height: '3',
    rx: '1.5'
  }], ['path', {
    d: 'M15.5 19H14v1.5a1.5 1.5 0 101.5-1.5'
  }], ['rect', {
    x: '2',
    y: '8',
    width: '8',
    height: '3',
    rx: '1.5'
  }], ['path', {
    d: 'M8.5 5H10V3.5A1.5 1.5 0 108.5 5'
  }]]];
  var Slack$1 = Slack;

  var Slash = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '4.93',
    y1: '4.93',
    x2: '19.07',
    y2: '19.07'
  }]]];
  var Slash$1 = Slash;

  var Sliders = ['svg', defaultAttributes, [['line', {
    x1: '4',
    y1: '21',
    x2: '4',
    y2: '14'
  }], ['line', {
    x1: '4',
    y1: '10',
    x2: '4',
    y2: '3'
  }], ['line', {
    x1: '12',
    y1: '21',
    x2: '12',
    y2: '12'
  }], ['line', {
    x1: '12',
    y1: '8',
    x2: '12',
    y2: '3'
  }], ['line', {
    x1: '20',
    y1: '21',
    x2: '20',
    y2: '16'
  }], ['line', {
    x1: '20',
    y1: '12',
    x2: '20',
    y2: '3'
  }], ['line', {
    x1: '1',
    y1: '14',
    x2: '7',
    y2: '14'
  }], ['line', {
    x1: '9',
    y1: '8',
    x2: '15',
    y2: '8'
  }], ['line', {
    x1: '17',
    y1: '16',
    x2: '23',
    y2: '16'
  }]]];
  var Sliders$1 = Sliders;

  var SmartphoneCharging = ['svg', defaultAttributes, [['rect', {
    x: '5',
    y: '2',
    width: '14',
    height: '20',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M12.667 8L10 12h4l-2.667 4'
  }]]];
  var SmartphoneCharging$1 = SmartphoneCharging;

  var Smartphone = ['svg', defaultAttributes, [['rect', {
    x: '5',
    y: '2',
    width: '14',
    height: '20',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M12 18h.01'
  }]]];
  var Smartphone$1 = Smartphone;

  var Smile = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['path', {
    d: 'M8 14s1.5 2 4 2 4-2 4-2'
  }], ['line', {
    x1: '9',
    y1: '9',
    x2: '9.01',
    y2: '9'
  }], ['line', {
    x1: '15',
    y1: '9',
    x2: '15.01',
    y2: '9'
  }]]];
  var Smile$1 = Smile;

  var Snowflake = ['svg', defaultAttributes, [['line', {
    x1: '2',
    y1: '12',
    x2: '22',
    y2: '12'
  }], ['line', {
    x1: '12',
    y1: '2',
    x2: '12',
    y2: '22'
  }], ['path', {
    d: 'M20 16l-4-4 4-4'
  }], ['path', {
    d: 'M4 8l4 4-4 4'
  }], ['path', {
    d: 'M16 4l-4 4-4-4'
  }], ['path', {
    d: 'M8 20l4-4 4 4'
  }]]];
  var Snowflake$1 = Snowflake;

  var SortAsc = ['svg', defaultAttributes, [['path', {
    d: 'M11 5h4'
  }], ['path', {
    d: 'M11 9h7'
  }], ['path', {
    d: 'M11 13h10'
  }], ['path', {
    d: 'M3 17l3 3 3-3'
  }], ['path', {
    d: 'M6 18V4'
  }]]];
  var SortAsc$1 = SortAsc;

  var SortDesc = ['svg', defaultAttributes, [['path', {
    d: 'M11 5h10'
  }], ['path', {
    d: 'M11 9h7'
  }], ['path', {
    d: 'M11 13h4'
  }], ['path', {
    d: 'M3 17l3 3 3-3'
  }], ['path', {
    d: 'M6 18V4'
  }]]];
  var SortDesc$1 = SortDesc;

  var Speaker = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '2',
    width: '16',
    height: '20',
    rx: '2',
    ry: '2'
  }], ['circle', {
    cx: '12',
    cy: '14',
    r: '4'
  }], ['line', {
    x1: '12',
    y1: '6',
    x2: '12.01',
    y2: '6'
  }]]];
  var Speaker$1 = Speaker;

  var Sprout = ['svg', defaultAttributes, [['path', {
    d: 'M7 20h10'
  }], ['path', {
    d: 'M10 20c5.5-2.5.8-6.4 3-10'
  }], ['path', {
    d: 'M9.5 9.4c1.1.8 1.8 2.2 2.3 3.7-2 .4-3.5.4-4.8-.3-1.2-.6-2.3-1.9-3-4.2 2.8-.5 4.4 0 5.5.8z'
  }], ['path', {
    d: 'M14.1 6a7 7 0 00-1.1 4c1.9-.1 3.3-.6 4.3-1.4 1-1 1.6-2.3 1.7-4.6-2.7.1-4 1-4.9 2z'
  }]]];
  var Sprout$1 = Sprout;

  var Square = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }]]];
  var Square$1 = Square;

  var StarHalf = ['svg', defaultAttributes, [['path', {
    d: 'M12 17.8L5.8 21 7 14.1 2 9.3l7-1L12 2'
  }]]];
  var StarHalf$1 = StarHalf;

  var Star = ['svg', defaultAttributes, [['polygon', {
    points: '12 2 15.09 8.26 22 9.27 17 14.14 18.18 21.02 12 17.77 5.82 21.02 7 14.14 2 9.27 8.91 8.26 12 2'
  }]]];
  var Star$1 = Star;

  var StopCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['rect', {
    x: '9',
    y: '9',
    width: '6',
    height: '6'
  }]]];
  var StopCircle$1 = StopCircle;

  var StretchHorizontal = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '4',
    width: '20',
    height: '6',
    rx: '2'
  }], ['rect', {
    x: '2',
    y: '14',
    width: '20',
    height: '6',
    rx: '2'
  }]]];
  var StretchHorizontal$1 = StretchHorizontal;

  var StretchVertical = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '2',
    width: '6',
    height: '20',
    rx: '2'
  }], ['rect', {
    x: '14',
    y: '2',
    width: '6',
    height: '20',
    rx: '2'
  }]]];
  var StretchVertical$1 = StretchVertical;

  var Strikethrough = ['svg', defaultAttributes, [['path', {
    d: 'M16 4H9a3 3 0 00-2.83 4'
  }], ['path', {
    d: 'M14 12a4 4 0 010 8H6'
  }], ['line', {
    x1: '4',
    y1: '12',
    x2: '20',
    y2: '12'
  }]]];
  var Strikethrough$1 = Strikethrough;

  var Subscript = ['svg', defaultAttributes, [['path', {
    d: 'M4 5l8 8'
  }], ['path', {
    d: 'M12 5l-8 8'
  }], ['path', {
    d: 'M20 19h-4c0-1.5.44-2 1.5-2.5S20 15.33 20 14c0-.47-.17-.93-.48-1.29a2.11 2.11 0 00-2.62-.44c-.42.24-.74.62-.9 1.07'
  }]]];
  var Subscript$1 = Subscript;

  var Sun = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '4'
  }], ['path', {
    d: 'M12 2v2'
  }], ['path', {
    d: 'M12 20v2'
  }], ['path', {
    d: 'M5 5l1.5 1.5'
  }], ['path', {
    d: 'M17.5 17.5L19 19'
  }], ['path', {
    d: 'M2 12h2'
  }], ['path', {
    d: 'M20 12h2'
  }], ['path', {
    d: 'M5 19l1.5-1.5'
  }], ['path', {
    d: 'M17.5 6.5L19 5'
  }]]];
  var Sun$1 = Sun;

  var Sunrise = ['svg', defaultAttributes, [['path', {
    d: 'M12 2v8'
  }], ['path', {
    d: 'M5.2 11.2l1.4 1.4'
  }], ['path', {
    d: 'M2 18h2'
  }], ['path', {
    d: 'M20 18h2'
  }], ['path', {
    d: 'M17.4 12.6l1.4-1.4'
  }], ['path', {
    d: 'M22 22H2'
  }], ['path', {
    d: 'M8 6l4-4 4 4'
  }], ['path', {
    d: 'M16 18a4 4 0 00-8 0'
  }]]];
  var Sunrise$1 = Sunrise;

  var Sunset = ['svg', defaultAttributes, [['path', {
    d: 'M12 10V2'
  }], ['path', {
    d: 'M5.2 11.2l1.4 1.4'
  }], ['path', {
    d: 'M2 18h2'
  }], ['path', {
    d: 'M20 18h2'
  }], ['path', {
    d: 'M17.4 12.6l1.4-1.4'
  }], ['path', {
    d: 'M22 22H2'
  }], ['path', {
    d: 'M16 6l-4 4-4-4'
  }], ['path', {
    d: 'M16 18a4 4 0 00-8 0'
  }]]];
  var Sunset$1 = Sunset;

  var Superscript = ['svg', defaultAttributes, [['path', {
    d: 'M4 19l8-8'
  }], ['path', {
    d: 'M12 19l-8-8'
  }], ['path', {
    d: 'M20 12h-4c0-1.5.442-2 1.5-2.5S20 8.334 20 7.002c0-.472-.17-.93-.484-1.29a2.105 2.105 0 00-2.617-.436c-.42.239-.738.614-.899 1.06'
  }]]];
  var Superscript$1 = Superscript;

  var SwissFranc = ['svg', defaultAttributes, [['path', {
    d: 'M10 21V3h8'
  }], ['path', {
    d: 'M6 16h9'
  }], ['path', {
    d: 'M10 9.5h7'
  }]]];
  var SwissFranc$1 = SwissFranc;

  var SwitchCamera = ['svg', defaultAttributes, [['path', {
    d: 'M11 19H4a2 2 0 01-2-2V7a2 2 0 012-2h5'
  }], ['path', {
    d: 'M13 5h7a2 2 0 012 2v10a2 2 0 01-2 2h-5'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '3'
  }], ['path', {
    d: 'M18 22l-3-3 3-3'
  }], ['path', {
    d: 'M6 2l3 3-3 3'
  }]]];
  var SwitchCamera$1 = SwitchCamera;

  var Table = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '3',
    y1: '9',
    x2: '21',
    y2: '9'
  }], ['line', {
    x1: '3',
    y1: '15',
    x2: '21',
    y2: '15'
  }], ['line', {
    x1: '12',
    y1: '3',
    x2: '12',
    y2: '21'
  }]]];
  var Table$1 = Table;

  var Tablet = ['svg', defaultAttributes, [['rect', {
    x: '4',
    y: '2',
    width: '16',
    height: '20',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '12',
    y1: '18',
    x2: '12.01',
    y2: '18'
  }]]];
  var Tablet$1 = Tablet;

  var Tag = ['svg', defaultAttributes, [['path', {
    d: 'M2 12V2h10l9.44 9.44a2 2 0 0 1 0 2.82l-7.18 7.18a2 2 0 0 1-2.82 0L2 12Z'
  }], ['path', {
    d: 'M7 7h.01'
  }]]];
  var Tag$1 = Tag;

  var Target = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '6'
  }], ['circle', {
    cx: '12',
    cy: '12',
    r: '2'
  }]]];
  var Target$1 = Target;

  var Tent = ['svg', defaultAttributes, [['path', {
    d: 'M19 20L10 4'
  }], ['path', {
    d: 'M5 20l9-16'
  }], ['path', {
    d: 'M3 20h18'
  }], ['path', {
    d: 'M12 15l-3 5'
  }], ['path', {
    d: 'M12 15l3 5'
  }]]];
  var Tent$1 = Tent;

  var TerminalSquare = ['svg', defaultAttributes, [['path', {
    d: 'M7 11l2-2-2-2'
  }], ['path', {
    d: 'M11 13h4'
  }], ['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }]]];
  var TerminalSquare$1 = TerminalSquare;

  var Terminal = ['svg', defaultAttributes, [['polyline', {
    points: '4 17 10 11 4 5'
  }], ['line', {
    x1: '12',
    y1: '19',
    x2: '20',
    y2: '19'
  }]]];
  var Terminal$1 = Terminal;

  var TextCursorInput = ['svg', defaultAttributes, [['path', {
    d: 'M5.3 4h.9C7.7 4 9 5.3 9 6.7m0 0v10.5c0 1.6-1.1 2.7-2.7 2.7h-1M9 6.7v10.6a2.6 2.6 0 002.7 2.7h1M9 6.7C9 5.2 10.2 4 11.8 4h.9'
  }], ['path', {
    d: 'M13.1 7.9h6.8c1.1 0 2.1 1 2.1 2.1v4c0 1.1-.9 2.1-2.1 2.1h-6.8'
  }], ['path', {
    d: 'M4.8 16.1h-.7C3 16.1 2 15.1 2 14v-4c0-1.1 1-2.1 2.1-2.1h.7'
  }]]];
  var TextCursorInput$1 = TextCursorInput;

  var TextCursor = ['svg', defaultAttributes, [['path', {
    d: 'M17 22h-1a4 4 0 01-4-4V6a4 4 0 014-4h1'
  }], ['path', {
    d: 'M7 22h1a4 4 0 004-4v-1'
  }], ['path', {
    d: 'M7 2h1a4 4 0 014 4v1'
  }]]];
  var TextCursor$1 = TextCursor;

  var ThermometerSnowflake = ['svg', defaultAttributes, [['path', {
    d: 'M2 12h10'
  }], ['path', {
    d: 'M9 4v16'
  }], ['path', {
    d: 'M3 9l3 3-3 3'
  }], ['path', {
    d: 'M12 6L9 9 6 6'
  }], ['path', {
    d: 'M6 18l3-3 1.5 1.5'
  }], ['path', {
    d: 'M18 2a2 2 0 00-2 2v10.5a4 4 0 104 0V4a2 2 0 00-2-2z'
  }]]];
  var ThermometerSnowflake$1 = ThermometerSnowflake;

  var ThermometerSun = ['svg', defaultAttributes, [['path', {
    d: 'M12 9a4 4 0 00-2 7.5'
  }], ['path', {
    d: 'M12 3v2'
  }], ['path', {
    d: 'M6.6 18.4l-1.4 1.4'
  }], ['path', {
    d: 'M18 2a2 2 0 012 2v10.5a4 4 0 11-4 0V4c0-1.1.9-2 2-2z'
  }], ['path', {
    d: 'M4 13H2'
  }], ['path', {
    d: 'M6.6 7.6L5.2 6.2'
  }]]];
  var ThermometerSun$1 = ThermometerSun;

  var Thermometer = ['svg', defaultAttributes, [['path', {
    d: 'M12 2a2 2 0 00-2 2v10.5a4 4 0 104 0V4a2 2 0 00-2-2z'
  }]]];
  var Thermometer$1 = Thermometer;

  var ThumbsDown = ['svg', defaultAttributes, [['path', {
    d: 'M10 15v4a3 3 0 003 3l4-9V2H5.72a2 2 0 00-2 1.7l-1.38 9a2 2 0 002 2.3zm7-13h2.67A2.31 2.31 0 0122 4v7a2.31 2.31 0 01-2.33 2H17'
  }]]];
  var ThumbsDown$1 = ThumbsDown;

  var ThumbsUp = ['svg', defaultAttributes, [['path', {
    d: 'M14 9V5a3 3 0 00-3-3l-4 9v11h11.28a2 2 0 002-1.7l1.38-9a2 2 0 00-2-2.3zM7 22H4a2 2 0 01-2-2v-7a2 2 0 012-2h3'
  }]]];
  var ThumbsUp$1 = ThumbsUp;

  var Ticket = ['svg', defaultAttributes, [['path', {
    d: 'M3 9V7a2 2 0 012-2h14a2 2 0 012 2v2M3 15v2a2 2 0 002 2h14a2 2 0 002-2v-2'
  }], ['path', {
    d: 'M21 15a3 3 0 110-6'
  }], ['path', {
    d: 'M3 15a3 3 0 100-6'
  }], ['path', {
    d: 'M13 5v2'
  }], ['path', {
    d: 'M13 17v2'
  }], ['path', {
    d: 'M13 11v2'
  }]]];
  var Ticket$1 = Ticket;

  var TimerOff = ['svg', defaultAttributes, [['path', {
    d: 'M10 2h4'
  }], ['path', {
    d: 'M7.43 7.433A8 8 0 0118.566 18.57M4.582 11A8 8 0 0015 21.419'
  }], ['path', {
    d: 'M2 2l20 20'
  }], ['path', {
    d: 'M12 12v-2'
  }]]];
  var TimerOff$1 = TimerOff;

  var TimerReset = ['svg', defaultAttributes, [['path', {
    d: 'M10 2h4'
  }], ['path', {
    d: 'M12 14v-4'
  }], ['path', {
    d: 'M4 13a8 8 0 018-7 8 8 0 11-5.3 14L4 17.6'
  }], ['path', {
    d: 'M9 17H4v5'
  }]]];
  var TimerReset$1 = TimerReset;

  var Timer = ['svg', defaultAttributes, [['line', {
    x1: '10',
    x2: '14',
    y1: '2',
    y2: '2'
  }], ['line', {
    x1: '12',
    x2: '15',
    y1: '14',
    y2: '11'
  }], ['circle', {
    cx: '12',
    cy: '14',
    r: '8'
  }]]];
  var Timer$1 = Timer;

  var ToggleLeft = ['svg', defaultAttributes, [['rect', {
    x: '1',
    y: '5',
    width: '22',
    height: '14',
    rx: '7',
    ry: '7'
  }], ['circle', {
    cx: '8',
    cy: '12',
    r: '3'
  }]]];
  var ToggleLeft$1 = ToggleLeft;

  var ToggleRight = ['svg', defaultAttributes, [['rect', {
    x: '1',
    y: '5',
    width: '22',
    height: '14',
    rx: '7',
    ry: '7'
  }], ['circle', {
    cx: '16',
    cy: '12',
    r: '3'
  }]]];
  var ToggleRight$1 = ToggleRight;

  var Tornado = ['svg', defaultAttributes, [['path', {
    d: 'M21 4H3'
  }], ['path', {
    d: 'M18 8H6'
  }], ['path', {
    d: 'M19 12H9'
  }], ['path', {
    d: 'M16 16h-6'
  }], ['path', {
    d: 'M11 20H9'
  }]]];
  var Tornado$1 = Tornado;

  var Trash2 = ['svg', defaultAttributes, [['polyline', {
    points: '3 6 5 6 21 6'
  }], ['path', {
    d: 'M19 6v14a2 2 0 01-2 2H7a2 2 0 01-2-2V6m3 0V4a2 2 0 012-2h4a2 2 0 012 2v2'
  }], ['line', {
    x1: '10',
    y1: '11',
    x2: '10',
    y2: '17'
  }], ['line', {
    x1: '14',
    y1: '11',
    x2: '14',
    y2: '17'
  }]]];
  var Trash2$1 = Trash2;

  var Trash = ['svg', defaultAttributes, [['polyline', {
    points: '3 6 5 6 21 6'
  }], ['path', {
    d: 'M19 6v14a2 2 0 01-2 2H7a2 2 0 01-2-2V6m3 0V4a2 2 0 012-2h4a2 2 0 012 2v2'
  }]]];
  var Trash$1 = Trash;

  var Trello = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['rect', {
    x: '7',
    y: '7',
    width: '3',
    height: '9'
  }], ['rect', {
    x: '14',
    y: '7',
    width: '3',
    height: '5'
  }]]];
  var Trello$1 = Trello;

  var TrendingDown = ['svg', defaultAttributes, [['polyline', {
    points: '23 18 13.5 8.5 8.5 13.5 1 6'
  }], ['polyline', {
    points: '17 18 23 18 23 12'
  }]]];
  var TrendingDown$1 = TrendingDown;

  var TrendingUp = ['svg', defaultAttributes, [['polyline', {
    points: '23 6 13.5 15.5 8.5 10.5 1 18'
  }], ['polyline', {
    points: '17 6 23 6 23 12'
  }]]];
  var TrendingUp$1 = TrendingUp;

  var Triangle = ['svg', defaultAttributes, [['path', {
    d: 'M10.29 3.86L1.82 18a2 2 0 001.71 3h16.94a2 2 0 001.71-3L13.71 3.86a2 2 0 00-3.42 0z'
  }]]];
  var Triangle$1 = Triangle;

  var Truck = ['svg', defaultAttributes, [['path', {
    d: 'M10 17h4V5H2v12h3m15 0h2v-3.34a4 4 0 00-1.17-2.83L19 9h-5v8h1'
  }], ['circle', {
    cx: '7.5',
    cy: '17.5',
    r: '2.5'
  }], ['circle', {
    cx: '17.5',
    cy: '17.5',
    r: '2.5'
  }]]];
  var Truck$1 = Truck;

  var Tv2 = ['svg', defaultAttributes, [['path', {
    d: 'M7 21h10'
  }], ['rect', {
    x: '2',
    y: '3',
    width: '20',
    height: '14',
    rx: '2'
  }]]];
  var Tv2$1 = Tv2;

  var Tv = ['svg', defaultAttributes, [['rect', {
    x: '2',
    y: '7',
    width: '20',
    height: '15',
    rx: '2',
    ry: '2'
  }], ['polyline', {
    points: '17 2 12 7 7 2'
  }]]];
  var Tv$1 = Tv;

  var Twitch = ['svg', defaultAttributes, [['path', {
    d: 'M21 2H3v16h5v4l4-4h5l4-4V2zm-10 9V7m5 4V7'
  }]]];
  var Twitch$1 = Twitch;

  var Twitter = ['svg', defaultAttributes, [['path', {
    d: 'M23 3a10.9 10.9 0 01-3.14 1.53 4.48 4.48 0 00-7.86 3v1A10.66 10.66 0 013 4s-4 9 5 13a11.64 11.64 0 01-7 2c9 5 20 0 20-11.5 0-.28-.03-.56-.08-.83A7.72 7.72 0 0023 3z'
  }]]];
  var Twitter$1 = Twitter;

  var Type = ['svg', defaultAttributes, [['polyline', {
    points: '4 7 4 4 20 4 20 7'
  }], ['line', {
    x1: '9',
    y1: '20',
    x2: '15',
    y2: '20'
  }], ['line', {
    x1: '12',
    y1: '4',
    x2: '12',
    y2: '20'
  }]]];
  var Type$1 = Type;

  var Umbrella = ['svg', defaultAttributes, [['path', {
    d: 'M23 12a11.05 11.05 0 00-22 0zm-5 7a3 3 0 01-6 0v-7'
  }]]];
  var Umbrella$1 = Umbrella;

  var Underline = ['svg', defaultAttributes, [['path', {
    d: 'M6 4v6a6 6 0 0012 0V4'
  }], ['line', {
    x1: '4',
    y1: '20',
    x2: '20',
    y2: '20'
  }]]];
  var Underline$1 = Underline;

  var Undo = ['svg', defaultAttributes, [['path', {
    d: 'M3 7v6h6'
  }], ['path', {
    d: 'M21 17a9 9 0 00-9-9 9 9 0 00-6 2.3L3 13'
  }]]];
  var Undo$1 = Undo;

  var Unlink2 = ['svg', defaultAttributes, [['path', {
    d: 'M15 7h2a5 5 0 010 10h-2m-6 0H7A5 5 0 017 7h2'
  }]]];
  var Unlink2$1 = Unlink2;

  var Unlink = ['svg', defaultAttributes, [['path', {
    d: 'M18.84 12.25l1.72-1.71h-.02a5.004 5.004 0 00-.12-7.07 5.006 5.006 0 00-6.95 0l-1.72 1.71'
  }], ['path', {
    d: 'M5.17 11.75l-1.71 1.71a5.004 5.004 0 00.12 7.07 5.006 5.006 0 006.95 0l1.71-1.71'
  }], ['line', {
    x1: '8',
    y1: '2',
    x2: '8',
    y2: '5'
  }], ['line', {
    x1: '2',
    y1: '8',
    x2: '5',
    y2: '8'
  }], ['line', {
    x1: '16',
    y1: '19',
    x2: '16',
    y2: '22'
  }], ['line', {
    x1: '19',
    y1: '16',
    x2: '22',
    y2: '16'
  }]]];
  var Unlink$1 = Unlink;

  var Unlock = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '11',
    width: '18',
    height: '11',
    rx: '2',
    ry: '2'
  }], ['path', {
    d: 'M7 11V7a5 5 0 019.9-1'
  }]]];
  var Unlock$1 = Unlock;

  var UploadCloud = ['svg', defaultAttributes, [['path', {
    d: 'M20 16.2A4.5 4.5 0 0017.5 8h-1.8A7 7 0 104 14.9'
  }], ['path', {
    d: 'M12 12v9'
  }], ['path', {
    d: 'M16 16l-4-4-4 4'
  }]]];
  var UploadCloud$1 = UploadCloud;

  var Upload = ['svg', defaultAttributes, [['path', {
    d: 'M21 15v4a2 2 0 01-2 2H5a2 2 0 01-2-2v-4'
  }], ['polyline', {
    points: '17 8 12 3 7 8'
  }], ['line', {
    x1: '12',
    y1: '3',
    x2: '12',
    y2: '15'
  }]]];
  var Upload$1 = Upload;

  var UserCheck = ['svg', defaultAttributes, [['path', {
    d: 'M16 21v-2a4 4 0 00-4-4H5a4 4 0 00-4 4v2'
  }], ['circle', {
    cx: '8.5',
    cy: '7',
    r: '4'
  }], ['polyline', {
    points: '17 11 19 13 23 9'
  }]]];
  var UserCheck$1 = UserCheck;

  var UserMinus = ['svg', defaultAttributes, [['path', {
    d: 'M16 21v-2a4 4 0 00-4-4H5a4 4 0 00-4 4v2'
  }], ['circle', {
    cx: '8.5',
    cy: '7',
    r: '4'
  }], ['line', {
    x1: '23',
    y1: '11',
    x2: '17',
    y2: '11'
  }]]];
  var UserMinus$1 = UserMinus;

  var UserPlus = ['svg', defaultAttributes, [['path', {
    d: 'M16 21v-2a4 4 0 00-4-4H5a4 4 0 00-4 4v2'
  }], ['circle', {
    cx: '8.5',
    cy: '7',
    r: '4'
  }], ['line', {
    x1: '20',
    y1: '8',
    x2: '20',
    y2: '14'
  }], ['line', {
    x1: '23',
    y1: '11',
    x2: '17',
    y2: '11'
  }]]];
  var UserPlus$1 = UserPlus;

  var UserX = ['svg', defaultAttributes, [['path', {
    d: 'M16 21v-2a4 4 0 00-4-4H5a4 4 0 00-4 4v2'
  }], ['circle', {
    cx: '8.5',
    cy: '7',
    r: '4'
  }], ['line', {
    x1: '18',
    y1: '8',
    x2: '23',
    y2: '13'
  }], ['line', {
    x1: '23',
    y1: '8',
    x2: '18',
    y2: '13'
  }]]];
  var UserX$1 = UserX;

  var User = ['svg', defaultAttributes, [['path', {
    d: 'M20 21v-2a4 4 0 00-4-4H8a4 4 0 00-4 4v2'
  }], ['circle', {
    cx: '12',
    cy: '7',
    r: '4'
  }]]];
  var User$1 = User;

  var Users = ['svg', defaultAttributes, [['path', {
    d: 'M17 21v-2a4 4 0 00-4-4H5a4 4 0 00-4 4v2'
  }], ['circle', {
    cx: '9',
    cy: '7',
    r: '4'
  }], ['path', {
    d: 'M23 21v-2a4 4 0 00-3-3.87'
  }], ['path', {
    d: 'M16 3.13a4 4 0 010 7.75'
  }]]];
  var Users$1 = Users;

  var Verified = ['svg', defaultAttributes, [['path', {
    d: 'M12 3a3.6 3.6 0 00-3.05 1.68 3.6 3.6 0 00-.9-.1 3.6 3.6 0 00-2.42 1.06 3.6 3.6 0 00-.94 3.32A3.6 3.6 0 003 12a3.6 3.6 0 001.69 3.05 3.6 3.6 0 00.95 3.32 3.6 3.6 0 003.35.96A3.6 3.6 0 0012 21a3.6 3.6 0 003.04-1.67 3.6 3.6 0 004.3-4.3A3.6 3.6 0 0021 12a3.6 3.6 0 00-1.67-3.04v0a3.6 3.6 0 00-4.3-4.3A3.6 3.6 0 0012 3z'
  }], ['path', {
    d: 'M15 10l-4 4'
  }], ['path', {
    d: 'M9 12l2 2'
  }]]];
  var Verified$1 = Verified;

  var Vibrate = ['svg', defaultAttributes, [['path', {
    d: 'M2 8l2 2-2 2 2 2-2 2'
  }], ['path', {
    d: 'M22 8l-2 2 2 2-2 2 2 2'
  }], ['rect', {
    x: '8',
    y: '5',
    width: '8',
    height: '14',
    rx: '1'
  }]]];
  var Vibrate$1 = Vibrate;

  var VideoOff = ['svg', defaultAttributes, [['path', {
    d: 'M16 16v1a2 2 0 01-2 2H3a2 2 0 01-2-2V7a2 2 0 012-2h2m5.66 0H14a2 2 0 012 2v3.34l1 1L23 7v10'
  }], ['line', {
    x1: '1',
    y1: '1',
    x2: '23',
    y2: '23'
  }]]];
  var VideoOff$1 = VideoOff;

  var Video = ['svg', defaultAttributes, [['polygon', {
    points: '23 7 16 12 23 17 23 7'
  }], ['rect', {
    x: '1',
    y: '5',
    width: '15',
    height: '14',
    rx: '2',
    ry: '2'
  }]]];
  var Video$1 = Video;

  var View = ['svg', defaultAttributes, [['path', {
    d: 'M5 12s2.545-5 7-5c4.454 0 7 5 7 5s-2.546 5-7 5c-4.455 0-7-5-7-5z'
  }], ['path', {
    d: 'M12 13a1 1 0 100-2 1 1 0 000 2z'
  }], ['path', {
    d: 'M21 8V5a2 2 0 00-2-2H5a2 2 0 00-2 2v3m18 8v3a2 2 0 01-2 2H5a2 2 0 01-2-2v-3'
  }]]];
  var View$1 = View;

  var Voicemail = ['svg', defaultAttributes, [['circle', {
    cx: '5.5',
    cy: '11.5',
    r: '4.5'
  }], ['circle', {
    cx: '18.5',
    cy: '11.5',
    r: '4.5'
  }], ['line', {
    x1: '5.5',
    y1: '16',
    x2: '18.5',
    y2: '16'
  }]]];
  var Voicemail$1 = Voicemail;

  var Volume1 = ['svg', defaultAttributes, [['polygon', {
    points: '11 5 6 9 2 9 2 15 6 15 11 19 11 5'
  }], ['path', {
    d: 'M15.54 8.46a5 5 0 010 7.07'
  }]]];
  var Volume1$1 = Volume1;

  var Volume2 = ['svg', defaultAttributes, [['polygon', {
    points: '11 5 6 9 2 9 2 15 6 15 11 19 11 5'
  }], ['path', {
    d: 'M19.07 4.93a10 10 0 010 14.14M15.54 8.46a5 5 0 010 7.07'
  }]]];
  var Volume2$1 = Volume2;

  var VolumeX = ['svg', defaultAttributes, [['polygon', {
    points: '11 5 6 9 2 9 2 15 6 15 11 19 11 5'
  }], ['line', {
    x1: '23',
    y1: '9',
    x2: '17',
    y2: '15'
  }], ['line', {
    x1: '17',
    y1: '9',
    x2: '23',
    y2: '15'
  }]]];
  var VolumeX$1 = VolumeX;

  var Volume = ['svg', defaultAttributes, [['polygon', {
    points: '11 5 6 9 2 9 2 15 6 15 11 19 11 5'
  }]]];
  var Volume$1 = Volume;

  var Wallet = ['svg', defaultAttributes, [['path', {
    d: 'M20 12V8H6a2 2 0 01-2-2c0-1.1.9-2 2-2h12v4'
  }], ['path', {
    d: 'M4 6v12c0 1.1.9 2 2 2h14v-4'
  }], ['path', {
    d: 'M18 12a2 2 0 00-2 2c0 1.1.9 2 2 2h4v-4h-4z'
  }]]];
  var Wallet$1 = Wallet;

  var Wand = ['svg', defaultAttributes, [['path', {
    d: 'M15 4V2'
  }], ['path', {
    d: 'M15 16v-2'
  }], ['path', {
    d: 'M8 9h2'
  }], ['path', {
    d: 'M20 9h2'
  }], ['path', {
    d: 'M17.8 11.8L19 13'
  }], ['path', {
    d: 'M15 9h0'
  }], ['path', {
    d: 'M17.8 6.2L19 5'
  }], ['path', {
    d: 'M3 21l9-9'
  }], ['path', {
    d: 'M12.2 6.2L11 5'
  }]]];
  var Wand$1 = Wand;

  var Watch = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '7'
  }], ['polyline', {
    points: '12 9 12 12 13.5 13.5'
  }], ['path', {
    d: 'M16.51 17.35l-.35 3.83a2 2 0 01-2 1.82H9.83a2 2 0 01-2-1.82l-.35-3.83m.01-10.7l.35-3.83A2 2 0 019.83 1h4.35a2 2 0 012 1.82l.35 3.83'
  }]]];
  var Watch$1 = Watch;

  var Waves = ['svg', defaultAttributes, [['path', {
    d: 'M2 6c.6.5 1.2 1 2.5 1C7 7 7 5 9.5 5c1.3 0 1.9.5 2.5 1'
  }], ['path', {
    d: 'M2 18c.6.5 1.2 1 2.5 1 2.5 0 2.5-2 5-2 1.3 0 1.9.5 2.5 1'
  }], ['path', {
    d: 'M2 12c.6.5 1.2 1 2.5 1 2.5 0 2.5-2 5-2 1.3 0 1.9.5 2.5 1'
  }], ['path', {
    d: 'M12 6c.6.5 1.2 1 2.5 1 2.5 0 2.5-2 5-2 1.3 0 1.9.5 2.5 1'
  }], ['path', {
    d: 'M12 18c.6.5 1.2 1 2.5 1 2.5 0 2.5-2 5-2 1.3 0 1.9.5 2.5 1'
  }], ['path', {
    d: 'M12 12c.6.5 1.2 1 2.5 1 2.5 0 2.5-2 5-2 1.3 0 1.9.5 2.5 1'
  }]]];
  var Waves$1 = Waves;

  var Webcam = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '10',
    r: '8'
  }], ['circle', {
    cx: '12',
    cy: '10',
    r: '3'
  }], ['path', {
    d: 'M12 22v-4m-5 4h5-5zm10 0h-5 5z'
  }]]];
  var Webcam$1 = Webcam;

  var WifiOff = ['svg', defaultAttributes, [['line', {
    x1: '2',
    y1: '2',
    x2: '22',
    y2: '22'
  }], ['path', {
    d: 'M8.5 16.5a5 5 0 017 0'
  }], ['path', {
    d: 'M2 8.82a15 15 0 014.17-2.65'
  }], ['path', {
    d: 'M10.66 5c4.01-.36 8.14.9 11.34 3.76'
  }], ['path', {
    d: 'M16.85 11.25a10 10 0 012.22 1.68'
  }], ['path', {
    d: 'M5 13a10 10 0 015.24-2.76'
  }], ['line', {
    x1: '12',
    y1: '20',
    x2: '12.01',
    y2: '20'
  }]]];
  var WifiOff$1 = WifiOff;

  var Wifi = ['svg', defaultAttributes, [['path', {
    d: 'M5 13a10 10 0 0114 0'
  }], ['path', {
    d: 'M8.5 16.5a5 5 0 017 0'
  }], ['path', {
    d: 'M2 8.82a15 15 0 0120 0'
  }], ['line', {
    x1: '12',
    y1: '20',
    x2: '12.01',
    y2: '20'
  }]]];
  var Wifi$1 = Wifi;

  var Wind = ['svg', defaultAttributes, [['path', {
    d: 'M17.7 7.7a2.5 2.5 0 111.8 4.3H2'
  }], ['path', {
    d: 'M9.6 4.6A2 2 0 1111 8H2'
  }], ['path', {
    d: 'M12.6 19.4A2 2 0 1014 16H2'
  }]]];
  var Wind$1 = Wind;

  var WrapText = ['svg', defaultAttributes, [['line', {
    x1: '3',
    y1: '6',
    x2: '21',
    y2: '6'
  }], ['path', {
    d: 'M3 12h15a3 3 0 110 6h-4'
  }], ['polyline', {
    points: '16 16 14 18 16 20'
  }], ['line', {
    x1: '3',
    y1: '18',
    x2: '10',
    y2: '18'
  }]]];
  var WrapText$1 = WrapText;

  var Wrench = ['svg', defaultAttributes, [['path', {
    d: 'M14.7 6.3a1 1 0 000 1.4l1.6 1.6a1 1 0 001.4 0l3.77-3.77a6 6 0 01-7.94 7.94l-6.91 6.91a2.12 2.12 0 01-3-3l6.91-6.91a6 6 0 017.94-7.94l-3.76 3.76z'
  }]]];
  var Wrench$1 = Wrench;

  var XCircle = ['svg', defaultAttributes, [['circle', {
    cx: '12',
    cy: '12',
    r: '10'
  }], ['line', {
    x1: '15',
    y1: '9',
    x2: '9',
    y2: '15'
  }], ['line', {
    x1: '9',
    y1: '9',
    x2: '15',
    y2: '15'
  }]]];
  var XCircle$1 = XCircle;

  var XOctagon = ['svg', defaultAttributes, [['polygon', {
    points: '7.86 2 16.14 2 22 7.86 22 16.14 16.14 22 7.86 22 2 16.14 2 7.86 7.86 2'
  }], ['line', {
    x1: '15',
    y1: '9',
    x2: '9',
    y2: '15'
  }], ['line', {
    x1: '9',
    y1: '9',
    x2: '15',
    y2: '15'
  }]]];
  var XOctagon$1 = XOctagon;

  var XSquare = ['svg', defaultAttributes, [['rect', {
    x: '3',
    y: '3',
    width: '18',
    height: '18',
    rx: '2',
    ry: '2'
  }], ['line', {
    x1: '9',
    y1: '9',
    x2: '15',
    y2: '15'
  }], ['line', {
    x1: '15',
    y1: '9',
    x2: '9',
    y2: '15'
  }]]];
  var XSquare$1 = XSquare;

  var X = ['svg', defaultAttributes, [['line', {
    x1: '18',
    y1: '6',
    x2: '6',
    y2: '18'
  }], ['line', {
    x1: '6',
    y1: '6',
    x2: '18',
    y2: '18'
  }]]];
  var X$1 = X;

  var Youtube = ['svg', defaultAttributes, [['path', {
    d: 'M22.54 6.42a2.78 2.78 0 00-1.94-2C18.88 4 12 4 12 4s-6.88 0-8.6.46a2.78 2.78 0 00-1.94 2A29 29 0 001 11.75a29 29 0 00.46 5.33A2.78 2.78 0 003.4 19c1.72.46 8.6.46 8.6.46s6.88 0 8.6-.46a2.78 2.78 0 001.94-2 29 29 0 00.46-5.25 29 29 0 00-.46-5.33z'
  }], ['polygon', {
    points: '9.75 15.02 15.5 11.75 9.75 8.48 9.75 15.02'
  }]]];
  var Youtube$1 = Youtube;

  var ZapOff = ['svg', defaultAttributes, [['polyline', {
    points: '12.41 6.75 13 2 10.57 4.92'
  }], ['polyline', {
    points: '18.57 12.91 21 10 15.66 10'
  }], ['polyline', {
    points: '8 8 3 14 12 14 11 22 16 16'
  }], ['line', {
    x1: '1',
    y1: '1',
    x2: '23',
    y2: '23'
  }]]];
  var ZapOff$1 = ZapOff;

  var Zap = ['svg', defaultAttributes, [['polygon', {
    points: '13 2 3 14 12 14 11 22 21 10 12 10 13 2'
  }]]];
  var Zap$1 = Zap;

  var ZoomIn = ['svg', defaultAttributes, [['circle', {
    cx: '11',
    cy: '11',
    r: '8'
  }], ['line', {
    x1: '21',
    y1: '21',
    x2: '16.65',
    y2: '16.65'
  }], ['line', {
    x1: '11',
    y1: '8',
    x2: '11',
    y2: '14'
  }], ['line', {
    x1: '8',
    y1: '11',
    x2: '14',
    y2: '11'
  }]]];
  var ZoomIn$1 = ZoomIn;

  var ZoomOut = ['svg', defaultAttributes, [['circle', {
    cx: '11',
    cy: '11',
    r: '8'
  }], ['line', {
    x1: '21',
    y1: '21',
    x2: '16.65',
    y2: '16.65'
  }], ['line', {
    x1: '8',
    y1: '11',
    x2: '14',
    y2: '11'
  }]]];
  var ZoomOut$1 = ZoomOut;

  var allIcons = /*#__PURE__*/Object.freeze({
    __proto__: null,
    Activity: Activity$1,
    Airplay: Airplay$1,
    AlarmCheck: AlarmCheck$1,
    AlarmClockOff: AlarmClockOff$1,
    AlarmClock: AlarmClock$1,
    AlarmMinus: AlarmMinus$1,
    AlarmPlus: AlarmPlus$1,
    Album: Album$1,
    AlertCircle: AlertCircle$1,
    AlertOctagon: AlertOctagon$1,
    AlertTriangle: AlertTriangle$1,
    AlignCenterHorizontal: AlignCenterHorizontal$1,
    AlignCenterVertical: AlignCenterVertical$1,
    AlignCenter: AlignCenter$1,
    AlignEndHorizontal: AlignEndHorizontal$1,
    AlignEndVertical: AlignEndVertical$1,
    AlignHorizontalDistributeCenter: AlignHorizontalDistributeCenter$1,
    AlignHorizontalDistributeEnd: AlignHorizontalDistributeEnd$1,
    AlignHorizontalDistributeStart: AlignHorizontalDistributeStart$1,
    AlignHorizontalJustifyCenter: AlignHorizontalJustifyCenter$1,
    AlignHorizontalJustifyEnd: AlignHorizontalJustifyEnd$1,
    AlignHorizontalJustifyStart: AlignHorizontalJustifyStart$1,
    AlignHorizontalSpaceAround: AlignHorizontalSpaceAround$1,
    AlignHorizontalSpaceBetween: AlignHorizontalSpaceBetween$1,
    AlignJustify: AlignJustify$1,
    AlignLeft: AlignLeft$1,
    AlignRight: AlignRight$1,
    AlignStartHorizontal: AlignStartHorizontal$1,
    AlignStartVertical: AlignStartVertical$1,
    AlignVerticalDistributeCenter: AlignVerticalDistributeCenter$1,
    AlignVerticalDistributeEnd: AlignVerticalDistributeEnd$1,
    AlignVerticalDistributeStart: AlignVerticalDistributeStart$1,
    AlignVerticalJustifyCenter: AlignVerticalJustifyCenter$1,
    AlignVerticalJustifyEnd: AlignVerticalJustifyEnd$1,
    AlignVerticalJustifyStart: AlignVerticalJustifyStart$1,
    AlignVerticalSpaceAround: AlignVerticalSpaceAround$1,
    AlignVerticalSpaceBetween: AlignVerticalSpaceBetween$1,
    Anchor: Anchor$1,
    Aperture: Aperture$1,
    Archive: Archive$1,
    ArrowBigDown: ArrowBigDown$1,
    ArrowBigLeft: ArrowBigLeft$1,
    ArrowBigRight: ArrowBigRight$1,
    ArrowBigUp: ArrowBigUp$1,
    ArrowDownCircle: ArrowDownCircle$1,
    ArrowDownLeft: ArrowDownLeft$1,
    ArrowDownRight: ArrowDownRight$1,
    ArrowDown: ArrowDown$1,
    ArrowLeftCircle: ArrowLeftCircle$1,
    ArrowLeftRight: ArrowLeftRight$1,
    ArrowLeft: ArrowLeft$1,
    ArrowRightCircle: ArrowRightCircle$1,
    ArrowRight: ArrowRight$1,
    ArrowUpCircle: ArrowUpCircle$1,
    ArrowUpLeft: ArrowUpLeft$1,
    ArrowUpRight: ArrowUpRight$1,
    ArrowUp: ArrowUp$1,
    Asterisk: Asterisk$1,
    AtSign: AtSign$1,
    Award: Award$1,
    Axe: Axe$1,
    Banknote: Banknote$1,
    BarChart2: BarChart2$1,
    BarChart: BarChart$1,
    Baseline: Baseline$1,
    BatteryCharging: BatteryCharging$1,
    BatteryFull: BatteryFull$1,
    BatteryLow: BatteryLow$1,
    BatteryMedium: BatteryMedium$1,
    Battery: Battery$1,
    Beaker: Beaker$1,
    BellMinus: BellMinus$1,
    BellOff: BellOff$1,
    BellPlus: BellPlus$1,
    BellRing: BellRing$1,
    Bell: Bell$1,
    Bike: Bike$1,
    Binary: Binary$1,
    Bitcoin: Bitcoin$1,
    BluetoothConnected: BluetoothConnected$1,
    BluetoothOff: BluetoothOff$1,
    BluetoothSearching: BluetoothSearching$1,
    Bluetooth: Bluetooth$1,
    Bold: Bold$1,
    BookOpen: BookOpen$1,
    Book: Book$1,
    BookmarkMinus: BookmarkMinus$1,
    BookmarkPlus: BookmarkPlus$1,
    Bookmark: Bookmark$1,
    Bot: Bot$1,
    BoxSelect: BoxSelect$1,
    Box: Box$1,
    Briefcase: Briefcase$1,
    Brush: Brush$1,
    Bug: Bug$1,
    Building: Building$1,
    Bus: Bus$1,
    Calculator: Calculator$1,
    Calendar: Calendar$1,
    CameraOff: CameraOff$1,
    Camera: Camera$1,
    Car: Car$1,
    Carrot: Carrot$1,
    Cast: Cast$1,
    CheckCircle2: CheckCircle2$1,
    CheckCircle: CheckCircle$1,
    CheckSquare: CheckSquare$1,
    Check: Check$1,
    ChevronDown: ChevronDown$1,
    ChevronFirst: ChevronFirst$1,
    ChevronLast: ChevronLast$1,
    ChevronLeft: ChevronLeft$1,
    ChevronRight: ChevronRight$1,
    ChevronUp: ChevronUp$1,
    ChevronsDownUp: ChevronsDownUp$1,
    ChevronsDown: ChevronsDown$1,
    ChevronsLeft: ChevronsLeft$1,
    ChevronsRight: ChevronsRight$1,
    ChevronsUpDown: ChevronsUpDown$1,
    ChevronsUp: ChevronsUp$1,
    Chrome: Chrome$1,
    CircleSlashed: CircleSlashed$1,
    Circle: Circle$1,
    ClipboardCheck: ClipboardCheck$1,
    ClipboardCopy: ClipboardCopy$1,
    ClipboardList: ClipboardList$1,
    ClipboardX: ClipboardX$1,
    Clipboard: Clipboard$1,
    Clock1: Clock1$1,
    Clock10: Clock10$1,
    Clock11: Clock11$1,
    Clock12: Clock12$1,
    Clock2: Clock2$1,
    Clock3: Clock3$1,
    Clock4: Clock4$1,
    Clock5: Clock5$1,
    Clock6: Clock6$1,
    Clock7: Clock7$1,
    Clock8: Clock8$1,
    Clock9: Clock9$1,
    Clock: Clock$1,
    CloudDrizzle: CloudDrizzle$1,
    CloudFog: CloudFog$1,
    CloudHail: CloudHail$1,
    CloudLightning: CloudLightning$1,
    CloudMoon: CloudMoon$1,
    CloudOff: CloudOff$1,
    CloudRainWind: CloudRainWind$1,
    CloudRain: CloudRain$1,
    CloudSnow: CloudSnow$1,
    CloudSun: CloudSun$1,
    Cloud: Cloud$1,
    Cloudy: Cloudy$1,
    Clover: Clover$1,
    Code2: Code2$1,
    Code: Code$1,
    Codepen: Codepen$1,
    Codesandbox: Codesandbox$1,
    Coffee: Coffee$1,
    Coins: Coins$1,
    Columns: Columns$1,
    Command: Command$1,
    Compass: Compass$1,
    Contact: Contact$1,
    Contrast: Contrast$1,
    Cookie: Cookie$1,
    Copy: Copy$1,
    Copyleft: Copyleft$1,
    Copyright: Copyright$1,
    CornerDownLeft: CornerDownLeft$1,
    CornerDownRight: CornerDownRight$1,
    CornerLeftDown: CornerLeftDown$1,
    CornerLeftUp: CornerLeftUp$1,
    CornerRightDown: CornerRightDown$1,
    CornerRightUp: CornerRightUp$1,
    CornerUpLeft: CornerUpLeft$1,
    CornerUpRight: CornerUpRight$1,
    Cpu: Cpu$1,
    CreditCard: CreditCard$1,
    Crop: Crop$1,
    Cross: Cross$1,
    Crosshair: Crosshair$1,
    Crown: Crown$1,
    Currency: Currency$1,
    Database: Database$1,
    Delete: Delete$1,
    Dice1: Dice1$1,
    Dice2: Dice2$1,
    Dice3: Dice3$1,
    Dice4: Dice4$1,
    Dice5: Dice5$1,
    Dice6: Dice6$1,
    Disc: Disc$1,
    DivideCircle: DivideCircle$1,
    DivideSquare: DivideSquare$1,
    Divide: Divide$1,
    DollarSign: DollarSign$1,
    DownloadCloud: DownloadCloud$1,
    Download: Download$1,
    Dribbble: Dribbble$1,
    Droplet: Droplet$1,
    Droplets: Droplets$1,
    Drumstick: Drumstick$1,
    Edit2: Edit2$1,
    Edit3: Edit3$1,
    Edit: Edit$1,
    Egg: Egg$1,
    EqualNot: EqualNot$1,
    Equal: Equal$1,
    Euro: Euro$1,
    Expand: Expand$1,
    ExternalLink: ExternalLink$1,
    EyeOff: EyeOff$1,
    Eye: Eye$1,
    Facebook: Facebook$1,
    FastForward: FastForward$1,
    Feather: Feather$1,
    Figma: Figma$1,
    FileCheck2: FileCheck2$1,
    FileCheck: FileCheck$1,
    FileCode: FileCode$1,
    FileDigit: FileDigit$1,
    FileInput: FileInput$1,
    FileMinus2: FileMinus2$1,
    FileMinus: FileMinus$1,
    FileOutput: FileOutput$1,
    FilePlus2: FilePlus2$1,
    FilePlus: FilePlus$1,
    FileSearch: FileSearch$1,
    FileText: FileText$1,
    FileX2: FileX2$1,
    FileX: FileX$1,
    File: File$1,
    Files: Files$1,
    Film: Film$1,
    Filter: Filter$1,
    FlagTriangleLeft: FlagTriangleLeft$1,
    FlagTriangleRight: FlagTriangleRight$1,
    Flag: Flag$1,
    Flame: Flame$1,
    FlashlightOff: FlashlightOff$1,
    Flashlight: Flashlight$1,
    FlaskConical: FlaskConical$1,
    FlaskRound: FlaskRound$1,
    FolderMinus: FolderMinus$1,
    FolderOpen: FolderOpen$1,
    FolderPlus: FolderPlus$1,
    Folder: Folder$1,
    FormInput: FormInput$1,
    Forward: Forward$1,
    Framer: Framer$1,
    Frown: Frown$1,
    FunctionSquare: FunctionSquare$1,
    Gamepad2: Gamepad2$1,
    Gamepad: Gamepad$1,
    Gauge: Gauge$1,
    Gavel: Gavel$1,
    Gem: Gem$1,
    Ghost: Ghost$1,
    Gift: Gift$1,
    GitBranchPlus: GitBranchPlus$1,
    GitBranch: GitBranch$1,
    GitCommit: GitCommit$1,
    GitFork: GitFork$1,
    GitMerge: GitMerge$1,
    GitPullRequest: GitPullRequest$1,
    Github: Github$1,
    Gitlab: Gitlab$1,
    Glasses: Glasses$1,
    Globe2: Globe2$1,
    Globe: Globe$1,
    Grab: Grab$1,
    GraduationCap: GraduationCap$1,
    Grid: Grid$1,
    GripHorizontal: GripHorizontal$1,
    GripVertical: GripVertical$1,
    Hammer: Hammer$1,
    HandMetal: HandMetal$1,
    Hand: Hand$1,
    HardDrive: HardDrive$1,
    HardHat: HardHat$1,
    Hash: Hash$1,
    Haze: Haze$1,
    Headphones: Headphones$1,
    Heart: Heart$1,
    HelpCircle: HelpCircle$1,
    Hexagon: Hexagon$1,
    Highlighter: Highlighter$1,
    History: History$1,
    Home: Home$1,
    ImageMinus: ImageMinus$1,
    ImageOff: ImageOff$1,
    ImagePlus: ImagePlus$1,
    Image: Image$1,
    Import: Import$1,
    Inbox: Inbox$1,
    Indent: Indent$1,
    IndianRupee: IndianRupee$1,
    Infinity: Infinity$1,
    Info: Info$1,
    Inspect: Inspect$1,
    Instagram: Instagram$1,
    Italic: Italic$1,
    JapaneseYen: JapaneseYen$1,
    Key: Key$1,
    Keyboard: Keyboard$1,
    Landmark: Landmark$1,
    Languages: Languages$1,
    Laptop2: Laptop2$1,
    Laptop: Laptop$1,
    LassoSelect: LassoSelect$1,
    Lasso: Lasso$1,
    Layers: Layers$1,
    LayoutDashboard: LayoutDashboard$1,
    LayoutGrid: LayoutGrid$1,
    LayoutList: LayoutList$1,
    LayoutTemplate: LayoutTemplate$1,
    Layout: Layout$1,
    Library: Library$1,
    LifeBuoy: LifeBuoy$1,
    LightbulbOff: LightbulbOff$1,
    Lightbulb: Lightbulb$1,
    Link2Off: Link2Off$1,
    Link2: Link2$1,
    Link: Link$1,
    Linkedin: Linkedin$1,
    ListChecks: ListChecks$1,
    ListMinus: ListMinus$1,
    ListOrdered: ListOrdered$1,
    ListPlus: ListPlus$1,
    ListX: ListX$1,
    List: List$1,
    Loader2: Loader2$1,
    Loader: Loader$1,
    LocateFixed: LocateFixed$1,
    LocateOff: LocateOff$1,
    Locate: Locate$1,
    Lock: Lock$1,
    LogIn: LogIn$1,
    LogOut: LogOut$1,
    Mail: Mail$1,
    MapPin: MapPin$1,
    Map: Map$1,
    Maximize2: Maximize2$1,
    Maximize: Maximize$1,
    Megaphone: Megaphone$1,
    Meh: Meh$1,
    Menu: Menu$1,
    MessageCircle: MessageCircle$1,
    MessageSquare: MessageSquare$1,
    MicOff: MicOff$1,
    Mic: Mic$1,
    Minimize2: Minimize2$1,
    Minimize: Minimize$1,
    MinusCircle: MinusCircle$1,
    MinusSquare: MinusSquare$1,
    Minus: Minus$1,
    MonitorOff: MonitorOff$1,
    MonitorSpeaker: MonitorSpeaker$1,
    Monitor: Monitor$1,
    Moon: Moon$1,
    MoreHorizontal: MoreHorizontal$1,
    MoreVertical: MoreVertical$1,
    MountainSnow: MountainSnow$1,
    Mountain: Mountain$1,
    MousePointer2: MousePointer2$1,
    MousePointerClick: MousePointerClick$1,
    MousePointer: MousePointer$1,
    MoveDiagonal2: MoveDiagonal2$1,
    MoveDiagonal: MoveDiagonal$1,
    MoveHorizontal: MoveHorizontal$1,
    MoveVertical: MoveVertical$1,
    Move: Move$1,
    Music: Music$1,
    Navigation2: Navigation2$1,
    Navigation: Navigation$1,
    Network: Network$1,
    Octagon: Octagon$1,
    Option: Option$1,
    Outdent: Outdent$1,
    PackageCheck: PackageCheck$1,
    PackageMinus: PackageMinus$1,
    PackagePlus: PackagePlus$1,
    PackageSearch: PackageSearch$1,
    PackageX: PackageX$1,
    Package: Package$1,
    Palette: Palette$1,
    Palmtree: Palmtree$1,
    Paperclip: Paperclip$1,
    PauseCircle: PauseCircle$1,
    PauseOctagon: PauseOctagon$1,
    Pause: Pause$1,
    PenTool: PenTool$1,
    Pencil: Pencil$1,
    Percent: Percent$1,
    PersonStanding: PersonStanding$1,
    PhoneCall: PhoneCall$1,
    PhoneForwarded: PhoneForwarded$1,
    PhoneIncoming: PhoneIncoming$1,
    PhoneMissed: PhoneMissed$1,
    PhoneOff: PhoneOff$1,
    PhoneOutgoing: PhoneOutgoing$1,
    Phone: Phone$1,
    PieChart: PieChart$1,
    PiggyBank: PiggyBank$1,
    Pin: Pin$1,
    Pipette: Pipette$1,
    Plane: Plane$1,
    PlayCircle: PlayCircle$1,
    Play: Play$1,
    PlugZap: PlugZap$1,
    PlusCircle: PlusCircle$1,
    PlusSquare: PlusSquare$1,
    Plus: Plus$1,
    Pocket: Pocket$1,
    Podcast: Podcast$1,
    Pointer: Pointer$1,
    PoundSterling: PoundSterling$1,
    PowerOff: PowerOff$1,
    Power: Power$1,
    Printer: Printer$1,
    QrCode: QrCode$1,
    Quote: Quote$1,
    RadioReceiver: RadioReceiver$1,
    Radio: Radio$1,
    Redo: Redo$1,
    RefreshCcw: RefreshCcw$1,
    RefreshCw: RefreshCw$1,
    Regex: Regex$1,
    Repeat1: Repeat1$1,
    Repeat: Repeat$1,
    ReplyAll: ReplyAll$1,
    Reply: Reply$1,
    Rewind: Rewind$1,
    Rocket: Rocket$1,
    RockingChair: RockingChair$1,
    RotateCcw: RotateCcw$1,
    RotateCw: RotateCw$1,
    Rss: Rss$1,
    Ruler: Ruler$1,
    RussianRuble: RussianRuble$1,
    Save: Save$1,
    Scale: Scale$1,
    ScanLine: ScanLine$1,
    Scan: Scan$1,
    Scissors: Scissors$1,
    ScreenShareOff: ScreenShareOff$1,
    ScreenShare: ScreenShare$1,
    Search: Search$1,
    Send: Send$1,
    SeparatorHorizontal: SeparatorHorizontal$1,
    SeparatorVertical: SeparatorVertical$1,
    ServerCrash: ServerCrash$1,
    ServerOff: ServerOff$1,
    Server: Server$1,
    Settings2: Settings2$1,
    Settings: Settings$1,
    Share2: Share2$1,
    Share: Share$1,
    Sheet: Sheet$1,
    ShieldAlert: ShieldAlert$1,
    ShieldCheck: ShieldCheck$1,
    ShieldClose: ShieldClose$1,
    ShieldOff: ShieldOff$1,
    Shield: Shield$1,
    Shirt: Shirt$1,
    ShoppingBag: ShoppingBag$1,
    ShoppingCart: ShoppingCart$1,
    Shovel: Shovel$1,
    Shrink: Shrink$1,
    Shuffle: Shuffle$1,
    SidebarClose: SidebarClose$1,
    SidebarOpen: SidebarOpen$1,
    Sidebar: Sidebar$1,
    Sigma: Sigma$1,
    SignalHigh: SignalHigh$1,
    SignalLow: SignalLow$1,
    SignalMedium: SignalMedium$1,
    SignalZero: SignalZero$1,
    Signal: Signal$1,
    SkipBack: SkipBack$1,
    SkipForward: SkipForward$1,
    Skull: Skull$1,
    Slack: Slack$1,
    Slash: Slash$1,
    Sliders: Sliders$1,
    SmartphoneCharging: SmartphoneCharging$1,
    Smartphone: Smartphone$1,
    Smile: Smile$1,
    Snowflake: Snowflake$1,
    SortAsc: SortAsc$1,
    SortDesc: SortDesc$1,
    Speaker: Speaker$1,
    Sprout: Sprout$1,
    Square: Square$1,
    StarHalf: StarHalf$1,
    Star: Star$1,
    StopCircle: StopCircle$1,
    StretchHorizontal: StretchHorizontal$1,
    StretchVertical: StretchVertical$1,
    Strikethrough: Strikethrough$1,
    Subscript: Subscript$1,
    Sun: Sun$1,
    Sunrise: Sunrise$1,
    Sunset: Sunset$1,
    Superscript: Superscript$1,
    SwissFranc: SwissFranc$1,
    SwitchCamera: SwitchCamera$1,
    Table: Table$1,
    Tablet: Tablet$1,
    Tag: Tag$1,
    Target: Target$1,
    Tent: Tent$1,
    TerminalSquare: TerminalSquare$1,
    Terminal: Terminal$1,
    TextCursorInput: TextCursorInput$1,
    TextCursor: TextCursor$1,
    ThermometerSnowflake: ThermometerSnowflake$1,
    ThermometerSun: ThermometerSun$1,
    Thermometer: Thermometer$1,
    ThumbsDown: ThumbsDown$1,
    ThumbsUp: ThumbsUp$1,
    Ticket: Ticket$1,
    TimerOff: TimerOff$1,
    TimerReset: TimerReset$1,
    Timer: Timer$1,
    ToggleLeft: ToggleLeft$1,
    ToggleRight: ToggleRight$1,
    Tornado: Tornado$1,
    Trash2: Trash2$1,
    Trash: Trash$1,
    Trello: Trello$1,
    TrendingDown: TrendingDown$1,
    TrendingUp: TrendingUp$1,
    Triangle: Triangle$1,
    Truck: Truck$1,
    Tv2: Tv2$1,
    Tv: Tv$1,
    Twitch: Twitch$1,
    Twitter: Twitter$1,
    Type: Type$1,
    Umbrella: Umbrella$1,
    Underline: Underline$1,
    Undo: Undo$1,
    Unlink2: Unlink2$1,
    Unlink: Unlink$1,
    Unlock: Unlock$1,
    UploadCloud: UploadCloud$1,
    Upload: Upload$1,
    UserCheck: UserCheck$1,
    UserMinus: UserMinus$1,
    UserPlus: UserPlus$1,
    UserX: UserX$1,
    User: User$1,
    Users: Users$1,
    Verified: Verified$1,
    Vibrate: Vibrate$1,
    VideoOff: VideoOff$1,
    Video: Video$1,
    View: View$1,
    Voicemail: Voicemail$1,
    Volume1: Volume1$1,
    Volume2: Volume2$1,
    VolumeX: VolumeX$1,
    Volume: Volume$1,
    Wallet: Wallet$1,
    Wand: Wand$1,
    Watch: Watch$1,
    Waves: Waves$1,
    Webcam: Webcam$1,
    WifiOff: WifiOff$1,
    Wifi: Wifi$1,
    Wind: Wind$1,
    WrapText: WrapText$1,
    Wrench: Wrench$1,
    XCircle: XCircle$1,
    XOctagon: XOctagon$1,
    XSquare: XSquare$1,
    X: X$1,
    Youtube: Youtube$1,
    ZapOff: ZapOff$1,
    Zap: Zap$1,
    ZoomIn: ZoomIn$1,
    ZoomOut: ZoomOut$1
  });

  /**
   * Replaces all elements with matching nameAttr with the defined icons
   * @param {{ icons?: object, nameAttr?: string, attrs?: object }} options
   */

  var createIcons = function createIcons(_temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        _ref$icons = _ref.icons,
        icons = _ref$icons === void 0 ? allIcons : _ref$icons,
        _ref$nameAttr = _ref.nameAttr,
        nameAttr = _ref$nameAttr === void 0 ? 'icon-name' : _ref$nameAttr,
        _ref$attrs = _ref.attrs,
        attrs = _ref$attrs === void 0 ? {} : _ref$attrs;

    if (!Object.values(icons).length) {
      throw new Error("Please provide an icons object.\nIf you want to use all the icons you can import it like:\n `import { createIcons, icons } from 'lucide';\nlucide.createIcons({icons});`");
    }

    if (typeof document === 'undefined') {
      throw new Error('`createIcons()` only works in a browser environment.');
    }

    var elementsToReplace = document.querySelectorAll("[" + nameAttr + "]");
    Array.from(elementsToReplace).forEach(function (element) {
      return replaceElement(element, {
        nameAttr: nameAttr,
        icons: icons,
        attrs: attrs
      });
    });
  };

  exports.Activity = Activity$1;
  exports.Airplay = Airplay$1;
  exports.AlarmCheck = AlarmCheck$1;
  exports.AlarmClock = AlarmClock$1;
  exports.AlarmClockOff = AlarmClockOff$1;
  exports.AlarmMinus = AlarmMinus$1;
  exports.AlarmPlus = AlarmPlus$1;
  exports.Album = Album$1;
  exports.AlertCircle = AlertCircle$1;
  exports.AlertOctagon = AlertOctagon$1;
  exports.AlertTriangle = AlertTriangle$1;
  exports.AlignCenter = AlignCenter$1;
  exports.AlignCenterHorizontal = AlignCenterHorizontal$1;
  exports.AlignCenterVertical = AlignCenterVertical$1;
  exports.AlignEndHorizontal = AlignEndHorizontal$1;
  exports.AlignEndVertical = AlignEndVertical$1;
  exports.AlignHorizontalDistributeCenter = AlignHorizontalDistributeCenter$1;
  exports.AlignHorizontalDistributeEnd = AlignHorizontalDistributeEnd$1;
  exports.AlignHorizontalDistributeStart = AlignHorizontalDistributeStart$1;
  exports.AlignHorizontalJustifyCenter = AlignHorizontalJustifyCenter$1;
  exports.AlignHorizontalJustifyEnd = AlignHorizontalJustifyEnd$1;
  exports.AlignHorizontalJustifyStart = AlignHorizontalJustifyStart$1;
  exports.AlignHorizontalSpaceAround = AlignHorizontalSpaceAround$1;
  exports.AlignHorizontalSpaceBetween = AlignHorizontalSpaceBetween$1;
  exports.AlignJustify = AlignJustify$1;
  exports.AlignLeft = AlignLeft$1;
  exports.AlignRight = AlignRight$1;
  exports.AlignStartHorizontal = AlignStartHorizontal$1;
  exports.AlignStartVertical = AlignStartVertical$1;
  exports.AlignVerticalDistributeCenter = AlignVerticalDistributeCenter$1;
  exports.AlignVerticalDistributeEnd = AlignVerticalDistributeEnd$1;
  exports.AlignVerticalDistributeStart = AlignVerticalDistributeStart$1;
  exports.AlignVerticalJustifyCenter = AlignVerticalJustifyCenter$1;
  exports.AlignVerticalJustifyEnd = AlignVerticalJustifyEnd$1;
  exports.AlignVerticalJustifyStart = AlignVerticalJustifyStart$1;
  exports.AlignVerticalSpaceAround = AlignVerticalSpaceAround$1;
  exports.AlignVerticalSpaceBetween = AlignVerticalSpaceBetween$1;
  exports.Anchor = Anchor$1;
  exports.Aperture = Aperture$1;
  exports.Archive = Archive$1;
  exports.ArrowBigDown = ArrowBigDown$1;
  exports.ArrowBigLeft = ArrowBigLeft$1;
  exports.ArrowBigRight = ArrowBigRight$1;
  exports.ArrowBigUp = ArrowBigUp$1;
  exports.ArrowDown = ArrowDown$1;
  exports.ArrowDownCircle = ArrowDownCircle$1;
  exports.ArrowDownLeft = ArrowDownLeft$1;
  exports.ArrowDownRight = ArrowDownRight$1;
  exports.ArrowLeft = ArrowLeft$1;
  exports.ArrowLeftCircle = ArrowLeftCircle$1;
  exports.ArrowLeftRight = ArrowLeftRight$1;
  exports.ArrowRight = ArrowRight$1;
  exports.ArrowRightCircle = ArrowRightCircle$1;
  exports.ArrowUp = ArrowUp$1;
  exports.ArrowUpCircle = ArrowUpCircle$1;
  exports.ArrowUpLeft = ArrowUpLeft$1;
  exports.ArrowUpRight = ArrowUpRight$1;
  exports.Asterisk = Asterisk$1;
  exports.AtSign = AtSign$1;
  exports.Award = Award$1;
  exports.Axe = Axe$1;
  exports.Banknote = Banknote$1;
  exports.BarChart = BarChart$1;
  exports.BarChart2 = BarChart2$1;
  exports.Baseline = Baseline$1;
  exports.Battery = Battery$1;
  exports.BatteryCharging = BatteryCharging$1;
  exports.BatteryFull = BatteryFull$1;
  exports.BatteryLow = BatteryLow$1;
  exports.BatteryMedium = BatteryMedium$1;
  exports.Beaker = Beaker$1;
  exports.Bell = Bell$1;
  exports.BellMinus = BellMinus$1;
  exports.BellOff = BellOff$1;
  exports.BellPlus = BellPlus$1;
  exports.BellRing = BellRing$1;
  exports.Bike = Bike$1;
  exports.Binary = Binary$1;
  exports.Bitcoin = Bitcoin$1;
  exports.Bluetooth = Bluetooth$1;
  exports.BluetoothConnected = BluetoothConnected$1;
  exports.BluetoothOff = BluetoothOff$1;
  exports.BluetoothSearching = BluetoothSearching$1;
  exports.Bold = Bold$1;
  exports.Book = Book$1;
  exports.BookOpen = BookOpen$1;
  exports.Bookmark = Bookmark$1;
  exports.BookmarkMinus = BookmarkMinus$1;
  exports.BookmarkPlus = BookmarkPlus$1;
  exports.Bot = Bot$1;
  exports.Box = Box$1;
  exports.BoxSelect = BoxSelect$1;
  exports.Briefcase = Briefcase$1;
  exports.Brush = Brush$1;
  exports.Bug = Bug$1;
  exports.Building = Building$1;
  exports.Bus = Bus$1;
  exports.Calculator = Calculator$1;
  exports.Calendar = Calendar$1;
  exports.Camera = Camera$1;
  exports.CameraOff = CameraOff$1;
  exports.Car = Car$1;
  exports.Carrot = Carrot$1;
  exports.Cast = Cast$1;
  exports.Check = Check$1;
  exports.CheckCircle = CheckCircle$1;
  exports.CheckCircle2 = CheckCircle2$1;
  exports.CheckSquare = CheckSquare$1;
  exports.ChevronDown = ChevronDown$1;
  exports.ChevronFirst = ChevronFirst$1;
  exports.ChevronLast = ChevronLast$1;
  exports.ChevronLeft = ChevronLeft$1;
  exports.ChevronRight = ChevronRight$1;
  exports.ChevronUp = ChevronUp$1;
  exports.ChevronsDown = ChevronsDown$1;
  exports.ChevronsDownUp = ChevronsDownUp$1;
  exports.ChevronsLeft = ChevronsLeft$1;
  exports.ChevronsRight = ChevronsRight$1;
  exports.ChevronsUp = ChevronsUp$1;
  exports.ChevronsUpDown = ChevronsUpDown$1;
  exports.Chrome = Chrome$1;
  exports.Circle = Circle$1;
  exports.CircleSlashed = CircleSlashed$1;
  exports.Clipboard = Clipboard$1;
  exports.ClipboardCheck = ClipboardCheck$1;
  exports.ClipboardCopy = ClipboardCopy$1;
  exports.ClipboardList = ClipboardList$1;
  exports.ClipboardX = ClipboardX$1;
  exports.Clock = Clock$1;
  exports.Clock1 = Clock1$1;
  exports.Clock10 = Clock10$1;
  exports.Clock11 = Clock11$1;
  exports.Clock12 = Clock12$1;
  exports.Clock2 = Clock2$1;
  exports.Clock3 = Clock3$1;
  exports.Clock4 = Clock4$1;
  exports.Clock5 = Clock5$1;
  exports.Clock6 = Clock6$1;
  exports.Clock7 = Clock7$1;
  exports.Clock8 = Clock8$1;
  exports.Clock9 = Clock9$1;
  exports.Cloud = Cloud$1;
  exports.CloudDrizzle = CloudDrizzle$1;
  exports.CloudFog = CloudFog$1;
  exports.CloudHail = CloudHail$1;
  exports.CloudLightning = CloudLightning$1;
  exports.CloudMoon = CloudMoon$1;
  exports.CloudOff = CloudOff$1;
  exports.CloudRain = CloudRain$1;
  exports.CloudRainWind = CloudRainWind$1;
  exports.CloudSnow = CloudSnow$1;
  exports.CloudSun = CloudSun$1;
  exports.Cloudy = Cloudy$1;
  exports.Clover = Clover$1;
  exports.Code = Code$1;
  exports.Code2 = Code2$1;
  exports.Codepen = Codepen$1;
  exports.Codesandbox = Codesandbox$1;
  exports.Coffee = Coffee$1;
  exports.Coins = Coins$1;
  exports.Columns = Columns$1;
  exports.Command = Command$1;
  exports.Compass = Compass$1;
  exports.Contact = Contact$1;
  exports.Contrast = Contrast$1;
  exports.Cookie = Cookie$1;
  exports.Copy = Copy$1;
  exports.Copyleft = Copyleft$1;
  exports.Copyright = Copyright$1;
  exports.CornerDownLeft = CornerDownLeft$1;
  exports.CornerDownRight = CornerDownRight$1;
  exports.CornerLeftDown = CornerLeftDown$1;
  exports.CornerLeftUp = CornerLeftUp$1;
  exports.CornerRightDown = CornerRightDown$1;
  exports.CornerRightUp = CornerRightUp$1;
  exports.CornerUpLeft = CornerUpLeft$1;
  exports.CornerUpRight = CornerUpRight$1;
  exports.Cpu = Cpu$1;
  exports.CreditCard = CreditCard$1;
  exports.Crop = Crop$1;
  exports.Cross = Cross$1;
  exports.Crosshair = Crosshair$1;
  exports.Crown = Crown$1;
  exports.Currency = Currency$1;
  exports.Database = Database$1;
  exports.Delete = Delete$1;
  exports.Dice1 = Dice1$1;
  exports.Dice2 = Dice2$1;
  exports.Dice3 = Dice3$1;
  exports.Dice4 = Dice4$1;
  exports.Dice5 = Dice5$1;
  exports.Dice6 = Dice6$1;
  exports.Disc = Disc$1;
  exports.Divide = Divide$1;
  exports.DivideCircle = DivideCircle$1;
  exports.DivideSquare = DivideSquare$1;
  exports.DollarSign = DollarSign$1;
  exports.Download = Download$1;
  exports.DownloadCloud = DownloadCloud$1;
  exports.Dribbble = Dribbble$1;
  exports.Droplet = Droplet$1;
  exports.Droplets = Droplets$1;
  exports.Drumstick = Drumstick$1;
  exports.Edit = Edit$1;
  exports.Edit2 = Edit2$1;
  exports.Edit3 = Edit3$1;
  exports.Egg = Egg$1;
  exports.Equal = Equal$1;
  exports.EqualNot = EqualNot$1;
  exports.Euro = Euro$1;
  exports.Expand = Expand$1;
  exports.ExternalLink = ExternalLink$1;
  exports.Eye = Eye$1;
  exports.EyeOff = EyeOff$1;
  exports.Facebook = Facebook$1;
  exports.FastForward = FastForward$1;
  exports.Feather = Feather$1;
  exports.Figma = Figma$1;
  exports.File = File$1;
  exports.FileCheck = FileCheck$1;
  exports.FileCheck2 = FileCheck2$1;
  exports.FileCode = FileCode$1;
  exports.FileDigit = FileDigit$1;
  exports.FileInput = FileInput$1;
  exports.FileMinus = FileMinus$1;
  exports.FileMinus2 = FileMinus2$1;
  exports.FileOutput = FileOutput$1;
  exports.FilePlus = FilePlus$1;
  exports.FilePlus2 = FilePlus2$1;
  exports.FileSearch = FileSearch$1;
  exports.FileText = FileText$1;
  exports.FileX = FileX$1;
  exports.FileX2 = FileX2$1;
  exports.Files = Files$1;
  exports.Film = Film$1;
  exports.Filter = Filter$1;
  exports.Flag = Flag$1;
  exports.FlagTriangleLeft = FlagTriangleLeft$1;
  exports.FlagTriangleRight = FlagTriangleRight$1;
  exports.Flame = Flame$1;
  exports.Flashlight = Flashlight$1;
  exports.FlashlightOff = FlashlightOff$1;
  exports.FlaskConical = FlaskConical$1;
  exports.FlaskRound = FlaskRound$1;
  exports.Folder = Folder$1;
  exports.FolderMinus = FolderMinus$1;
  exports.FolderOpen = FolderOpen$1;
  exports.FolderPlus = FolderPlus$1;
  exports.FormInput = FormInput$1;
  exports.Forward = Forward$1;
  exports.Framer = Framer$1;
  exports.Frown = Frown$1;
  exports.FunctionSquare = FunctionSquare$1;
  exports.Gamepad = Gamepad$1;
  exports.Gamepad2 = Gamepad2$1;
  exports.Gauge = Gauge$1;
  exports.Gavel = Gavel$1;
  exports.Gem = Gem$1;
  exports.Ghost = Ghost$1;
  exports.Gift = Gift$1;
  exports.GitBranch = GitBranch$1;
  exports.GitBranchPlus = GitBranchPlus$1;
  exports.GitCommit = GitCommit$1;
  exports.GitFork = GitFork$1;
  exports.GitMerge = GitMerge$1;
  exports.GitPullRequest = GitPullRequest$1;
  exports.Github = Github$1;
  exports.Gitlab = Gitlab$1;
  exports.Glasses = Glasses$1;
  exports.Globe = Globe$1;
  exports.Globe2 = Globe2$1;
  exports.Grab = Grab$1;
  exports.GraduationCap = GraduationCap$1;
  exports.Grid = Grid$1;
  exports.GripHorizontal = GripHorizontal$1;
  exports.GripVertical = GripVertical$1;
  exports.Hammer = Hammer$1;
  exports.Hand = Hand$1;
  exports.HandMetal = HandMetal$1;
  exports.HardDrive = HardDrive$1;
  exports.HardHat = HardHat$1;
  exports.Hash = Hash$1;
  exports.Haze = Haze$1;
  exports.Headphones = Headphones$1;
  exports.Heart = Heart$1;
  exports.HelpCircle = HelpCircle$1;
  exports.Hexagon = Hexagon$1;
  exports.Highlighter = Highlighter$1;
  exports.History = History$1;
  exports.Home = Home$1;
  exports.Image = Image$1;
  exports.ImageMinus = ImageMinus$1;
  exports.ImageOff = ImageOff$1;
  exports.ImagePlus = ImagePlus$1;
  exports.Import = Import$1;
  exports.Inbox = Inbox$1;
  exports.Indent = Indent$1;
  exports.IndianRupee = IndianRupee$1;
  exports.Infinity = Infinity$1;
  exports.Info = Info$1;
  exports.Inspect = Inspect$1;
  exports.Instagram = Instagram$1;
  exports.Italic = Italic$1;
  exports.JapaneseYen = JapaneseYen$1;
  exports.Key = Key$1;
  exports.Keyboard = Keyboard$1;
  exports.Landmark = Landmark$1;
  exports.Languages = Languages$1;
  exports.Laptop = Laptop$1;
  exports.Laptop2 = Laptop2$1;
  exports.Lasso = Lasso$1;
  exports.LassoSelect = LassoSelect$1;
  exports.Layers = Layers$1;
  exports.Layout = Layout$1;
  exports.LayoutDashboard = LayoutDashboard$1;
  exports.LayoutGrid = LayoutGrid$1;
  exports.LayoutList = LayoutList$1;
  exports.LayoutTemplate = LayoutTemplate$1;
  exports.Library = Library$1;
  exports.LifeBuoy = LifeBuoy$1;
  exports.Lightbulb = Lightbulb$1;
  exports.LightbulbOff = LightbulbOff$1;
  exports.Link = Link$1;
  exports.Link2 = Link2$1;
  exports.Link2Off = Link2Off$1;
  exports.Linkedin = Linkedin$1;
  exports.List = List$1;
  exports.ListChecks = ListChecks$1;
  exports.ListMinus = ListMinus$1;
  exports.ListOrdered = ListOrdered$1;
  exports.ListPlus = ListPlus$1;
  exports.ListX = ListX$1;
  exports.Loader = Loader$1;
  exports.Loader2 = Loader2$1;
  exports.Locate = Locate$1;
  exports.LocateFixed = LocateFixed$1;
  exports.LocateOff = LocateOff$1;
  exports.Lock = Lock$1;
  exports.LogIn = LogIn$1;
  exports.LogOut = LogOut$1;
  exports.Mail = Mail$1;
  exports.Map = Map$1;
  exports.MapPin = MapPin$1;
  exports.Maximize = Maximize$1;
  exports.Maximize2 = Maximize2$1;
  exports.Megaphone = Megaphone$1;
  exports.Meh = Meh$1;
  exports.Menu = Menu$1;
  exports.MessageCircle = MessageCircle$1;
  exports.MessageSquare = MessageSquare$1;
  exports.Mic = Mic$1;
  exports.MicOff = MicOff$1;
  exports.Minimize = Minimize$1;
  exports.Minimize2 = Minimize2$1;
  exports.Minus = Minus$1;
  exports.MinusCircle = MinusCircle$1;
  exports.MinusSquare = MinusSquare$1;
  exports.Monitor = Monitor$1;
  exports.MonitorOff = MonitorOff$1;
  exports.MonitorSpeaker = MonitorSpeaker$1;
  exports.Moon = Moon$1;
  exports.MoreHorizontal = MoreHorizontal$1;
  exports.MoreVertical = MoreVertical$1;
  exports.Mountain = Mountain$1;
  exports.MountainSnow = MountainSnow$1;
  exports.MousePointer = MousePointer$1;
  exports.MousePointer2 = MousePointer2$1;
  exports.MousePointerClick = MousePointerClick$1;
  exports.Move = Move$1;
  exports.MoveDiagonal = MoveDiagonal$1;
  exports.MoveDiagonal2 = MoveDiagonal2$1;
  exports.MoveHorizontal = MoveHorizontal$1;
  exports.MoveVertical = MoveVertical$1;
  exports.Music = Music$1;
  exports.Navigation = Navigation$1;
  exports.Navigation2 = Navigation2$1;
  exports.Network = Network$1;
  exports.Octagon = Octagon$1;
  exports.Option = Option$1;
  exports.Outdent = Outdent$1;
  exports.Package = Package$1;
  exports.PackageCheck = PackageCheck$1;
  exports.PackageMinus = PackageMinus$1;
  exports.PackagePlus = PackagePlus$1;
  exports.PackageSearch = PackageSearch$1;
  exports.PackageX = PackageX$1;
  exports.Palette = Palette$1;
  exports.Palmtree = Palmtree$1;
  exports.Paperclip = Paperclip$1;
  exports.Pause = Pause$1;
  exports.PauseCircle = PauseCircle$1;
  exports.PauseOctagon = PauseOctagon$1;
  exports.PenTool = PenTool$1;
  exports.Pencil = Pencil$1;
  exports.Percent = Percent$1;
  exports.PersonStanding = PersonStanding$1;
  exports.Phone = Phone$1;
  exports.PhoneCall = PhoneCall$1;
  exports.PhoneForwarded = PhoneForwarded$1;
  exports.PhoneIncoming = PhoneIncoming$1;
  exports.PhoneMissed = PhoneMissed$1;
  exports.PhoneOff = PhoneOff$1;
  exports.PhoneOutgoing = PhoneOutgoing$1;
  exports.PieChart = PieChart$1;
  exports.PiggyBank = PiggyBank$1;
  exports.Pin = Pin$1;
  exports.Pipette = Pipette$1;
  exports.Plane = Plane$1;
  exports.Play = Play$1;
  exports.PlayCircle = PlayCircle$1;
  exports.PlugZap = PlugZap$1;
  exports.Plus = Plus$1;
  exports.PlusCircle = PlusCircle$1;
  exports.PlusSquare = PlusSquare$1;
  exports.Pocket = Pocket$1;
  exports.Podcast = Podcast$1;
  exports.Pointer = Pointer$1;
  exports.PoundSterling = PoundSterling$1;
  exports.Power = Power$1;
  exports.PowerOff = PowerOff$1;
  exports.Printer = Printer$1;
  exports.QrCode = QrCode$1;
  exports.Quote = Quote$1;
  exports.Radio = Radio$1;
  exports.RadioReceiver = RadioReceiver$1;
  exports.Redo = Redo$1;
  exports.RefreshCcw = RefreshCcw$1;
  exports.RefreshCw = RefreshCw$1;
  exports.Regex = Regex$1;
  exports.Repeat = Repeat$1;
  exports.Repeat1 = Repeat1$1;
  exports.Reply = Reply$1;
  exports.ReplyAll = ReplyAll$1;
  exports.Rewind = Rewind$1;
  exports.Rocket = Rocket$1;
  exports.RockingChair = RockingChair$1;
  exports.RotateCcw = RotateCcw$1;
  exports.RotateCw = RotateCw$1;
  exports.Rss = Rss$1;
  exports.Ruler = Ruler$1;
  exports.RussianRuble = RussianRuble$1;
  exports.Save = Save$1;
  exports.Scale = Scale$1;
  exports.Scan = Scan$1;
  exports.ScanLine = ScanLine$1;
  exports.Scissors = Scissors$1;
  exports.ScreenShare = ScreenShare$1;
  exports.ScreenShareOff = ScreenShareOff$1;
  exports.Search = Search$1;
  exports.Send = Send$1;
  exports.SeparatorHorizontal = SeparatorHorizontal$1;
  exports.SeparatorVertical = SeparatorVertical$1;
  exports.Server = Server$1;
  exports.ServerCrash = ServerCrash$1;
  exports.ServerOff = ServerOff$1;
  exports.Settings = Settings$1;
  exports.Settings2 = Settings2$1;
  exports.Share = Share$1;
  exports.Share2 = Share2$1;
  exports.Sheet = Sheet$1;
  exports.Shield = Shield$1;
  exports.ShieldAlert = ShieldAlert$1;
  exports.ShieldCheck = ShieldCheck$1;
  exports.ShieldClose = ShieldClose$1;
  exports.ShieldOff = ShieldOff$1;
  exports.Shirt = Shirt$1;
  exports.ShoppingBag = ShoppingBag$1;
  exports.ShoppingCart = ShoppingCart$1;
  exports.Shovel = Shovel$1;
  exports.Shrink = Shrink$1;
  exports.Shuffle = Shuffle$1;
  exports.Sidebar = Sidebar$1;
  exports.SidebarClose = SidebarClose$1;
  exports.SidebarOpen = SidebarOpen$1;
  exports.Sigma = Sigma$1;
  exports.Signal = Signal$1;
  exports.SignalHigh = SignalHigh$1;
  exports.SignalLow = SignalLow$1;
  exports.SignalMedium = SignalMedium$1;
  exports.SignalZero = SignalZero$1;
  exports.SkipBack = SkipBack$1;
  exports.SkipForward = SkipForward$1;
  exports.Skull = Skull$1;
  exports.Slack = Slack$1;
  exports.Slash = Slash$1;
  exports.Sliders = Sliders$1;
  exports.Smartphone = Smartphone$1;
  exports.SmartphoneCharging = SmartphoneCharging$1;
  exports.Smile = Smile$1;
  exports.Snowflake = Snowflake$1;
  exports.SortAsc = SortAsc$1;
  exports.SortDesc = SortDesc$1;
  exports.Speaker = Speaker$1;
  exports.Sprout = Sprout$1;
  exports.Square = Square$1;
  exports.Star = Star$1;
  exports.StarHalf = StarHalf$1;
  exports.StopCircle = StopCircle$1;
  exports.StretchHorizontal = StretchHorizontal$1;
  exports.StretchVertical = StretchVertical$1;
  exports.Strikethrough = Strikethrough$1;
  exports.Subscript = Subscript$1;
  exports.Sun = Sun$1;
  exports.Sunrise = Sunrise$1;
  exports.Sunset = Sunset$1;
  exports.Superscript = Superscript$1;
  exports.SwissFranc = SwissFranc$1;
  exports.SwitchCamera = SwitchCamera$1;
  exports.Table = Table$1;
  exports.Tablet = Tablet$1;
  exports.Tag = Tag$1;
  exports.Target = Target$1;
  exports.Tent = Tent$1;
  exports.Terminal = Terminal$1;
  exports.TerminalSquare = TerminalSquare$1;
  exports.TextCursor = TextCursor$1;
  exports.TextCursorInput = TextCursorInput$1;
  exports.Thermometer = Thermometer$1;
  exports.ThermometerSnowflake = ThermometerSnowflake$1;
  exports.ThermometerSun = ThermometerSun$1;
  exports.ThumbsDown = ThumbsDown$1;
  exports.ThumbsUp = ThumbsUp$1;
  exports.Ticket = Ticket$1;
  exports.Timer = Timer$1;
  exports.TimerOff = TimerOff$1;
  exports.TimerReset = TimerReset$1;
  exports.ToggleLeft = ToggleLeft$1;
  exports.ToggleRight = ToggleRight$1;
  exports.Tornado = Tornado$1;
  exports.Trash = Trash$1;
  exports.Trash2 = Trash2$1;
  exports.Trello = Trello$1;
  exports.TrendingDown = TrendingDown$1;
  exports.TrendingUp = TrendingUp$1;
  exports.Triangle = Triangle$1;
  exports.Truck = Truck$1;
  exports.Tv = Tv$1;
  exports.Tv2 = Tv2$1;
  exports.Twitch = Twitch$1;
  exports.Twitter = Twitter$1;
  exports.Type = Type$1;
  exports.Umbrella = Umbrella$1;
  exports.Underline = Underline$1;
  exports.Undo = Undo$1;
  exports.Unlink = Unlink$1;
  exports.Unlink2 = Unlink2$1;
  exports.Unlock = Unlock$1;
  exports.Upload = Upload$1;
  exports.UploadCloud = UploadCloud$1;
  exports.User = User$1;
  exports.UserCheck = UserCheck$1;
  exports.UserMinus = UserMinus$1;
  exports.UserPlus = UserPlus$1;
  exports.UserX = UserX$1;
  exports.Users = Users$1;
  exports.Verified = Verified$1;
  exports.Vibrate = Vibrate$1;
  exports.Video = Video$1;
  exports.VideoOff = VideoOff$1;
  exports.View = View$1;
  exports.Voicemail = Voicemail$1;
  exports.Volume = Volume$1;
  exports.Volume1 = Volume1$1;
  exports.Volume2 = Volume2$1;
  exports.VolumeX = VolumeX$1;
  exports.Wallet = Wallet$1;
  exports.Wand = Wand$1;
  exports.Watch = Watch$1;
  exports.Waves = Waves$1;
  exports.Webcam = Webcam$1;
  exports.Wifi = Wifi$1;
  exports.WifiOff = WifiOff$1;
  exports.Wind = Wind$1;
  exports.WrapText = WrapText$1;
  exports.Wrench = Wrench$1;
  exports.X = X$1;
  exports.XCircle = XCircle$1;
  exports.XOctagon = XOctagon$1;
  exports.XSquare = XSquare$1;
  exports.Youtube = Youtube$1;
  exports.Zap = Zap$1;
  exports.ZapOff = ZapOff$1;
  exports.ZoomIn = ZoomIn$1;
  exports.ZoomOut = ZoomOut$1;
  exports.createElement = createElement$1;
  exports.createIcons = createIcons;
  exports.icons = allIcons;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=lucide.js.map
